//////////////////////////////////////////////////////////////////////////////
//                                                                         
//  NAME: Daniel Abadi
//  FILE: DEDebugController.C
//  PROJ: COKO Debugger
//  DATE: SUMMER 2001
//
// SYNOPSIS
//  Methods definitions for DDebugController class.
//  
//////////////////////////////////////////////////////////////////////////////

#include "DEDebugController.H"

//#include "KGlobals.H"
//#ifdef DEBUG
//#include <KONonType.H>
//#endif

#define TRUE 1
#define FALSE 0



//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  DDebugController
// Parameters:	()
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

DDebugController::DDebugController(int doVisual,int argc, char *argv[])
{
  currentlyRunning = 0;
  breakStep = 0;
  breakNext = 0;
  displayCurrTree=0;
  displayRootTree=0;
  displayCounter=1;
  rule_on=0;
  success_on=0;
  pattern_on=0;
  restart = 1;
  varCounter = 4;
  _curr_rbList = new RuleBlockList();
  _break_rbList = new RuleBlockList();
  _symList = new SymbolList();
  _histList = new HistoryList();
  _transList = new DTransformationList();

  visual = doVisual;
  readyToGo = 0;
  toExec = NULL;
  execName = NULL;

  //If the constructor was called with the visual flag, get ready to use QT
  if (visual) {
    a = new QApplication(argc, argv);
    vis = new MyMainWindow(this, _curr_rbList, _break_rbList, _transList);
    a->setMainWidget(vis); 
    vis->show();
    displayRootTree=1;
    _svl = new DSymbolVisList();
    vis->setSVL(_svl);
    zoomCounter = 1;
  }
  tempStmt = NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~DDebugController
// Parameters:	(void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

DDebugController::~DDebugController()
{
  _root = NULL;
  _current = NULL;
  if (_curr_rbList) {delete _curr_rbList; _curr_rbList = NULL;}
  if (_break_rbList) {delete _break_rbList; _break_rbList = NULL;}
  if (_symList) {delete _symList; _symList = NULL;}
  if (_histList) {delete _histList; _histList = NULL;}
  if (execName) {delete[] execName; execName = NULL;}
  if (execKey) {delete[] execKey; execKey = NULL;}
  if (_svl) {delete _svl; _svl = NULL;}
  if (_transList) {delete _transList; _transList = NULL;}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  getInput
// Parameters:	(string*)
// Returns: void
//
// This method is called by the command line version of the debugger to get
// a line that the user inputted.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::getInput(string* x) {
  getline(cin,*x);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  parseInput
// Parameters:	(const char* charArrayToParse, char** parsedArray, int count)
// Returns: void
//
// This method is called by the command line version of the debugger to pasre
// the char* charArrayToParse input string into an array of char* with each
// element of the array pointing to 1 word (space delimited)
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::parseInput(const char* charArrayToParse, char** parsedArray, int count) {

  if (!charArrayToParse) {
    parsedArray[0] = NULL;
    return;
  }
  int i = 0;
  int wordCount = 0;
  char* temp;
  int len = 0;
  while ((charArrayToParse[i] != '\0') && (wordCount <= count)) {
    while (charArrayToParse[i] == ' ') //skip spaces, plus use to separate parameters
      i++;
    if (charArrayToParse[i] != '\0') //if we haven't reached the end then we have found a new word
      wordCount++;
    //for each character of that new word add it to the temporary string 'temp' which will be used to be put in the parsedArray
    while ((charArrayToParse[i] != ' ') && (charArrayToParse[i] != '\0')) {
      len++;
      i++;
    }
    i-=len;
    temp = new char[len + 1];
    len = 0;
    while ((charArrayToParse[i] != ' ') && (charArrayToParse[i] != '\0')) {
      temp[len] = charArrayToParse[i];
      i++;
      len++;
    }
    temp[len] = '\0';
    len = 0;
    parsedArray[wordCount-1] = temp;
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  outputInput
// Parameters:	(string* parsedInput, int count)
// Returns: void
//
// This method is simply a debugging method used to make sure that I am
// parsing the input correctly.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::outputInput(string* parsedInput, int count) {
  ost << "Command " << parsedInput[0] << " was called with parameters: ";
  for (int i = 1; parsedInput[i] != "" && i < count; i++)
    ost << parsedInput[i] << " ";
  ost << endl;
  doPrintOST();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  really_execute
// Parameters:	(void)
// Returns: bool
//
// This method is called from every ExecDebug method in every COKO statement
// to make sure that it should really go ahead with the execution. The reason
// why this check has to be made is that if the user wants to restart the
// program, no more execution should occur until the the current execution
// unwinds from all its recursive calls and the while loop and begin again.
//         								    
//////////////////////////////////////////////////////////////////////////////

int DDebugController::really_execute() {
  return (!(restart));
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  do_restart
// Parameters:	(void)
// Returns: bool
//
// Once the outer while loop is ready to restart the COKO program (ready to
// begin execution again, do_restart will be called to reinitialze the
// restart variable to 0). But, before restarting, it checks to see if it
// really should restart by checking the value of the restart boolean.
//         								    
//////////////////////////////////////////////////////////////////////////////

int DDebugController::do_restart() {
  if (restart) {
    restart = 0;
    return 1;
  }
  else
    return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  doBreakOptions
// Parameters:	(void)
// Returns: void
//
// While the command was renamed to "list", the method name wasn't. This method
// is only called in the command line version of the debugger. It displays
// the current transformation that is being debugged to the user and beneath
// it displays the same transformation with numbers before all the breakable
// statements. That way, it is easy for the user to select which statement
// upon which to break.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::doBreakOptions() {
  ost << "BREAK OPTIONS:" << endl << endl;
  ost << "CODE:" << endl;
  char* rep = _current->representation(); //display the current transformation
  ost << rep << endl << endl;
  delete [] rep;

  //display the same transformation with enumerated breakable statements
  _current->output_self(this);
  doPrintOST();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  findCurrent
// Parameters:	(CRuleBlock*, char*)
// Returns: int
//
// This is an important method that goes through the break rule block list
// (this list maintains all of the transformations that have been called so
// far and any break information that the debugger might need to know about
// each transformation) and by default finds the one that is being currently
// run (the top element in the current rule clock stack). Alternatively, if
// a char* transformation name is given (transName) it will find that
// transformation (and not the current one) or if a pointer to a RuleBlock
// is given it will get the name of the RuleBlock by following the pointer
// and calling the get_name() method and will find that transformation.
// The key assumption here is that transformation name is a key. This method
// returns a boolean value indicating whether it found that transformation in
// the break list and additionally will result in the iterator of the global
// variable _break_rbList pointing to the approporate element. 
//         								    
//////////////////////////////////////////////////////////////////////////////

int DDebugController::findCurrent(CRuleBlock* actuallyFindThisOne=NULL, char* transName = NULL) {
  char* tmp2; //the transformation to find
  if (transName)
    tmp2 = transName; //find this transformatioon
  else {
    if (actuallyFindThisOne)
      tmp2 = actuallyFindThisOne->get_name(); //find this transformation
    else {
      _current = _curr_rbList->topRB(); //else find the current transformation
      if (!(_current))
	return 0;
      tmp2 = _current->get_name();
    }
  }

  //the following code is a little bit of optimization. The break list
  //iterator is somewhat likely to be pointing to the correct element
  //so check the current element first before initialize the iterator
  //and start searching from the beginning of the list
  char* iterator_name = _break_rbList->iteratorName();
  if ((iterator_name) && (strcmp(iterator_name,tmp2) == 0)) {
    delete[] iterator_name;
    iterator_name = NULL;
    return 1;
  }
  else //search from the beginning of the list
    {
      _break_rbList->initIterator();
      iterator_name = _break_rbList->iteratorName();

      //check to see if the current element of _break_rbList has its name
      //equal to tmp2 - the one we are searching for
      while ((iterator_name) && (!(strcmp(iterator_name,tmp2) == 0))) {
	delete[] iterator_name;
	iterator_name = NULL;
	iterator_name = _break_rbList->getNextName();
      }
      
      //clean up
      if (tmp2) {delete [] tmp2; tmp2 = NULL;}
      if (iterator_name) {
	delete [] iterator_name;
	iterator_name = NULL;
	return 1;
      }
      else return 0;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doClearBreak
// Parameters:	(char*, int)
// Returns: void
//
// This method will clear a break at the indicated key number. If the parameter
// whereToClear is NULL then the parameter key_number is the indication of
// which statement to add the break. Otherwise the integer conversion from the
// char* whereToClear is used.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::doClearBreak(char* whereToClear, int key_number) {
  //get _break_rbList pointing the the current transformation
  int temp = findCurrent();
  if (temp == 0) {
    ost << "ERROR";
    doPrintOST();
    exit(1);
  }
  else {
    //get break information from the transformation that the _break_rbList
    //is currently pointing to
    int* breakArray = _break_rbList->iteratorBA();
    int* conditionArray = _break_rbList->iteratorCA();
    int len = _break_rbList->iteratorArraySize();
    int kn = 0;

    //if whereToClear is empty then use key_number parameter instead
    if ((!whereToClear) || (strcmp(whereToClear,"")==0))
      kn = key_number;
    else {
      kn = atoi(whereToClear);
    }
    if (kn) {
      if (kn <= len) {
	int a = breakArray[kn-1];
	int b = conditionArray[kn-1];
	if ((!a) && (!b))
	  ost << "No break at this key number." << endl;
	else {
	  breakArray[kn-1] = 0;
	  conditionArray[kn-1] = 0;
	  ost << "Breaks cleared at key number: " << kn << endl;
	}
      }
      else
	ost << "Illegal clear number. Maximum key number is: " << len << endl;
    }
    else
      ost << "ERROR: looking for a number" << endl;
  }
  doPrintOST();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doClearBreakSpecific
// Parameters:	(char*, char*, int)
// Returns: void
//
// This method will clear a break at the indicated key number in the
// transformation transName. If the parameter whereToClear is NULL then the
// parameter key_number is the indication of which statement to clear the
// break. Otherwise the integer conversion from the char* whereToClear is used.
// The parameter transName is expected to come in as "<transname>" so the <
// and > have to be eliminated before searching for the transformation in the
// transformation list _transList.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::doClearBreakSpecific(char* transName, char* whereToClear, int key_number) {
  int kn;
  if ((!whereToClear) || (strcmp(whereToClear,"")==0))
    kn = key_number;
  else {
    kn = atoi(whereToClear);
  }
  if (kn) {
    _transList->initIterator();
    //need to strip < and > from transName;
    char* realTransName = new char[strlen(transName)-1];
    for (int i = 0; i < strlen(transName)-2; i++)
      realTransName[i] = transName[i+1];
    realTransName[strlen(transName)-2] = '\0';
    int x = _transList->find(realTransName);
    int* breakArray;
    int* conditionArray;
    int len;
    if (x == 1) {
      breakArray = _transList->iteratorNextValue()->_breakArray;
      conditionArray = _transList->iteratorNextValue()->_conditionArray;
      len = _transList->iteratorNextValue()->_arraySize;
    }
    else if (x == 2) {
      breakArray = _transList->iteratorValue()->_breakArray;
      conditionArray = _transList->iteratorValue()->_conditionArray;
      len = _transList->iteratorValue()->_arraySize;
    }
    else {
      ost << "Transformation: " << realTransName << " not in the tracking list.\nTry: track " << realTransName << endl;
      return;
    }
    if (kn <= len) {
      int a = breakArray[kn-1];
      int b = conditionArray[kn-1];
      if ((!a) && (!b))
	ost << "No break at this key number." << endl;
      else {
	breakArray[kn-1] = 0;
	conditionArray[kn-1] = 0;
	ost << "Breaks cleared at key number: " << kn << endl;
      }
    }
    else
      ost << "Illegal clear number. Maximum key number is: " << len << endl;
    delete[] realTransName;
  }
  else
    ost << "ERROR: looking for a number" << endl;
  doPrintOST();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doAddBreak
// Parameters:	(char*, int, RuleBlock* toAddBreak=NULL)
// Returns: void
//
// This method will add a break to the indicated key number. If the parameter
// whereToBreak is NULL then the parameter key_number is the indication of
// which statement to add the break. Otherwise the integer conversion from the
// char* whereToBreak is used. The default transformation to add the break is
// the current transformation. Otherwise, a RuleBlock* must be passed to
// indicate which transformation to add the break.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::doAddBreak(char* whereToBreak, int key_number, CRuleBlock* toAddBreak = NULL) {
  //get _break_rbList pointing the the current transformation
  int temp = findCurrent(toAddBreak);
  if (temp == 0) {
    ost << "ERROR";
    doPrintOST();
    exit(1);
  }
  else {
    //if whereToBreak is empty then use key_number parameter instead
    int temp;
    if ((!whereToBreak) || (strcmp(whereToBreak,"")==0))
      temp = key_number;
    else {
      temp = atoi(whereToBreak);
    }
    if (temp) {
      //get break information from the transformation that the _break_rbList
      //is currently pointing to
      int* breakArray = _break_rbList->iteratorBA();
      int* conditionArray = _break_rbList->iteratorCA();
      int len = _break_rbList->iteratorArraySize();
      if (temp > len)
	ost << "Illegal break number. Maximum key number is: " << len << endl;
      else if (breakArray[temp-1])
	ost << "Break ALREADY placed on key number: " << temp << endl;
      else if (conditionArray[temp-1])
	ost << "ERROR:  Conditional break already placed on key number: " << temp << endl << "Must clear conditional break first." << endl;
      else {
	breakArray[temp-1] = 1;
	ost << "Break placed on key number: " << temp << endl;
      }
    }
    else
      ost << "ERROR: looking for a number" << endl;
  }
  doPrintOST();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doAddBreakSpecific
// Parameters:	(char*, char*, int)
// Returns: void
//
// This method will add a break to the indicated key number to the
// transformation transName. If the parameter whereToBreak is NULL then the
// parameter key_number is the indication of which statement to add the break.
// Otherwise the integer conversion from the char* whereToBreak is used.
// The parameter transName is expected to come in as "<transname>" so the <
// and > have to be eliminated before searching for the transformation in the
// transformation list _transList.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::doAddBreakSpecific(char* transName, char* whereToBreak, int key_number) {
  int temp;
  if ((!whereToBreak) || (strcmp(whereToBreak,"")==0))
    temp = key_number;
  else {
    temp = atoi(whereToBreak);
  }
  if (temp) {
    _transList->initIterator();
    //need to strip < and > from transName;
    char* realTransName = new char[strlen(transName)-1];
    for (int i = 0; i < strlen(transName)-2; i++)
      realTransName[i] = transName[i+1];
    realTransName[strlen(transName)-2] = '\0';
    int x = _transList->find(realTransName);
    int* breakArray;
    int* conditionArray;
    int len;
    if (x == 1) {
      breakArray = _transList->iteratorNextValue()->_breakArray;
      conditionArray = _transList->iteratorNextValue()->_conditionArray;
      len = _transList->iteratorNextValue()->_arraySize;
    }
    else if (x == 2) {
      breakArray = _transList->iteratorValue()->_breakArray;
      conditionArray = _transList->iteratorValue()->_conditionArray;
      len = _transList->iteratorValue()->_arraySize;
    }
    else {
      ost << "Transformation: " << realTransName << " not in the tracking list.\nTry: track " << realTransName << endl;
      return;
    }
    if (temp > len)
      ost << "Illegal break number. Maximum key number is: " << len << endl;
    else if (breakArray[temp-1])
      ost << "Break ALREADY placed on key number: " << temp << endl;
    else if (conditionArray[temp-1])
      ost << "ERROR:  Conditional break already placed on key number: " << temp << endl << "Must clear conditional break first." << endl;
    else {
      breakArray[temp-1] = 1;
      ost << "Break placed on key number: " << temp << endl;
    }
    delete[] realTransName;
  }
  else
    ost << "ERROR: looking for a number" << endl;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doAddConditionBreak
// Parameters:	(char*, int)
// Returns: void
//
// This method will add a conditional break to the indicated key number. If
// the parameter whereToBreak is NULL then the parameter key_number is the
// indication of which statement to add the break. Otherwise the integer
// conversion from the char* whereToBreak is used.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::doAddConditionBreak(char* whereToBreak, int key_number) {
  //get _break_rbList pointing the the current transformation
  int temp = findCurrent();
  if (temp == 0) {
    ost << "ERROR";
    doPrintOST();
    exit(1);
  }
  else {
    //if whereToBreak is empty then use key_number parameter instead
    int temp;
    if ((!whereToBreak) || (strcmp(whereToBreak,"")==0))
      temp = key_number;
    else {
      temp = atoi(whereToBreak);
    }
    if (temp) {
      //get break information from the transformation that the _break_rbList
      //is currently pointing to
      int* breakArray = _break_rbList->iteratorBA();
      int* conditionArray = _break_rbList->iteratorCA();
      int len = _break_rbList->iteratorArraySize();
      if (temp > len)
	ost << "Illegal break number. Maximum key number is: " << len << endl;
      else if (conditionArray[temp-1])
	ost << "Conditional break  ALREADY placed on key number: " << temp << endl;
      else if (breakArray[temp-1])
	ost << "ERROR:  Regular break already placed on key number: " << temp << endl << "Must clear regular break first." << endl;
      else {
	conditionArray[temp-1] = 1;
	ost << "Conditional break placed on key number: " << temp << endl;
      }
    }
    else
      ost << "ERROR: looking for a number" << endl;
  }
  doPrintOST();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doAddConditionBreakSpecific
// Parameters:	(char*, char*, int)
// Returns: void
//
// This method will add a condition break to the indicated key number to the
// transformation transName. If the parameter whereToBreak is NULL then the
// parameter key_number is the indication of which statement to add the break.
// Otherwise the integer conversion from the char* whereToBreak is used.
// The parameter transName is expected to come in as "<transname>" so the <
// and > have to be eliminated before searching for the transformation in the
// transformation list _transList.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::doAddConditionBreakSpecific(char* transName, char* whereToBreak, int key_number) {
  int temp;
  if ((!whereToBreak) || (strcmp(whereToBreak,"")==0))
    temp = key_number;
  else {
    temp = atoi(whereToBreak);
  }
  if (temp) {
    _transList->initIterator();
    //need to strip < and > from transName;
    char* realTransName = new char[strlen(transName)-1];
    for (int i = 0; i < strlen(transName)-2; i++)
      realTransName[i] = transName[i+1];
    realTransName[strlen(transName)-2] = '\0';
    int x = _transList->find(realTransName);
    int* breakArray;
    int* conditionArray;
    int len;
    if (x == 1) {
      breakArray = _transList->iteratorNextValue()->_breakArray;
      conditionArray = _transList->iteratorNextValue()->_conditionArray;
      len = _transList->iteratorNextValue()->_arraySize;
    }
    else if (x == 2) {
      breakArray = _transList->iteratorValue()->_breakArray;
      conditionArray = _transList->iteratorValue()->_conditionArray;
      len = _transList->iteratorValue()->_arraySize;
    }
    else {
      ost << "Transformation: " << realTransName << " not in the tracking list.\nTry: track " << realTransName << endl;
      return;
    }
    
    if (temp > len)
      ost << "Illegal break number. Maximum key number is: " << len << endl;
    else if (conditionArray[temp-1])
      ost << "Conditional break  ALREADY placed on key number: " << temp << endl;
    else if (breakArray[temp-1])
      ost << "ERROR:  Regular break already placed on key number: " << temp << endl << "Must clear regular break first." << endl;
    else {
      conditionArray[temp-1] = 1;
      ost << "Conditional break placed on key number: " << temp << endl;
    }
    delete[] realTransName;
  }
  else
    ost << "ERROR: looking for a number" << endl;
  doPrintOST();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doPrint
// Parameters: (char* toPrint, CState* s)
// Returns: void
//
// This method will print in textual form the KOLA tree depending on toPrint.
// If toPrint is null or is current, this method will print to the screen the
// textual version of the current KOLA tree. If toPrint is "root", then this
// method will print the KOLA tree starting from the query root. Otherwise, it
// will print the KOLA tree bound to the variable inidicated in doPrint.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::doPrint(char* toPrint, CState* s) {
  if ((!toPrint) || (strcmp(toPrint,"")==0) || (strcmp(toPrint,"current")==0)) {
    char * res = NULL;
    ost << "Current Tree: " << (res = (s->Store())->representation()) << endl;
    if (res) {delete [] res; res = NULL; }
  }
  else if (strcmp(toPrint,"root")==0) {
    NonType* treeToDisplay;
    //the root KOLA tree is not maintained anywhere, so it has to be
    //recreated from the current state, s using the histUnravel method
    if (_histList->topVal())
      treeToDisplay = histUnravel(s);
    else
      treeToDisplay = s->Store()->Copy_kTree();
    char * res = NULL;
    ost << "Root Tree: " << (res = treeToDisplay->representation()) << endl;
    if (res) {delete [] res; res = NULL; }
    if (treeToDisplay) {delete treeToDisplay; treeToDisplay = NULL; }
  }
  else {
    CIdent * myNode;
    char * res = NULL;
    myNode = s->IFindDeepDB(toPrint);
    if (myNode)
      ost << (res = (myNode->Child_Look_Up())->representation()) << endl;
    else
      ost << "Symbol: '" << toPrint << "' is not in the environment stack." << endl;
    if(res) { delete [] res; res = NULL; }	
  }
  doPrintOST();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doDisplay
// Parameters: (char* toDisplay, CState* s)
// Returns: void
//
// This method will display in textual form (or if Visual COKO is running then
// in visual form) a KOLA tree which is indicated by the toDisplay parameter.
// If toDisplay is null or is current, this method will display the textual (or
// visual) version of the current KOLA tree. If toPrint is "root", then this
// method will display the KOLA tree starting from the query root. Otherwise,
// it will print the KOLA tree bound to the variable inidicated in doDisplay.
// The biggest difference between the doDisplay and doPrint methods is the
// display means continually display it (don't just print it once). For
// instance, if a variable is to be displayed, it should be displayed every
// time the debugger stops until the variable is no longer in context. This
// method accomplishes this by adding the variable name to a display list
// which is checked every time the debgger stops. There is one variable list
// in every rule block in _curr_rbList and accessed using the topSym() method.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::doDisplay(char* toDisplay, CState* s) {
  if ((!toDisplay) || (strcmp(toDisplay,"")==0) || (strcmp(toDisplay,"current")==0)) {
    if (visual)
      vis->setDisplayCT(1);
    else {
      char * res = NULL;
      //display current list once
      ost << "Current Tree: " << (res = (s->Store())->representation()) << endl;
      if (res) {delete [] res; res = NULL; }
    }
    ost << "Current tree display turned on." << endl;
    //then set a variable which says to display it every time
    displayCurrTree=1;
  }
  else if (strcmp(toDisplay,"root")==0) {
    if (visual)
      vis->setDisplayRT(1);
    else {
      //the root KOLA tree is not maintained anywhere, so it has to be
      //recreated from the current state, s using the histUnravel method
      NonType* treeToDisplay;
      if (_histList->topVal())
	treeToDisplay = histUnravel(s);
      else
	treeToDisplay = s->Store()->Copy_kTree();
      char * res = NULL;
      ost << "Root Tree: " << (res = treeToDisplay->representation()) << endl;
      if (res) {delete [] res; res = NULL; }
      if (treeToDisplay) {delete treeToDisplay; treeToDisplay = NULL; }
    }
    ost << "Root tree display turned on." << endl;
    displayRootTree=1;
  }
  else { //display a variable
    CIdent * myNode;
    char * res = NULL;
    myNode = s->IFindDeepDB(toDisplay);
    if (myNode) {
      ost << "Display of " << toDisplay << " turned on." << endl;
      if (!visual) {
	if (_curr_rbList->topSym()->find(toDisplay))
	  ost << toDisplay << "is already being displayed" << endl;
	else {
	  ost << displayCounter << ": " << toDisplay << " = " << (res = (myNode->Child_Look_Up())->representation()) << endl;
	  
	  //add that variable to the display list of the curreny ruleblock
	  _curr_rbList->topSym()->insert(toDisplay, displayCounter++);}
      }
      else {
	if (_svl->find(toDisplay))
	  ost << toDisplay << "is already being displayed" << endl;
	else {
	  //create a variable display window to display toDisplay
	  DScrollWidget* tempGB = new DScrollWidget(NULL, 0, NULL);
	  char* tempName = new char[strlen(toDisplay) + 19];
	  sprintf(tempName, "Variable Display: %s", toDisplay);
	  DPaintWindow* tempPW = new DPaintWindow(NULL, 0, vis, tempGB, tempName);
	  tempPW->setCurrent(myNode->Child_Look_Up());
	  tempGB->setSon(tempPW);
	  int tempIndex = vis->insertIntoView(toDisplay);
	  tempGB->setViewVals(vis->getView(), tempIndex);
	  //add this window to the _svl list
	  _svl->insert(toDisplay, NULL, displayCounter++, tempGB, tempPW, tempIndex);
	}
      }
    }
    else
      ost << "No symbol '" << toDisplay << "' in current context." << endl;
    if(res) { delete [] res; res = NULL; }
  }
  doPrintOST();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doUndisplay
// Parameters: (char* toDisplay, CState* s)
// Returns: void
//
// This method turns of the KOLA tree indicated by the toDisplay paramater that
// the doDisplay method turned on. If it was a variable that was being
// displayed, it is removed from the current rule block list. Otherwise, 
// the approporiate flag is turned off.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::doUndisplay(char* toDisplay, CState* s) {
  if ((!toDisplay) || (strcmp(toDisplay,"")==0) || (strcmp(toDisplay,"current")==0)) {
    if (displayCurrTree) {
      if (visual)
	vis->setDisplayCT(0);
      ost << "Current tree display turned off." << endl;
      displayCurrTree=0;
    }
    else
      ost << "Current tree display is not turned on." << endl;
  }
  else if (strcmp(toDisplay,"root")==0) {
    if (displayRootTree) {
      if (visual)
	vis->setDisplayRT(0);	
      ost << "Root tree display turned off." << endl;
      displayRootTree=0;
    }
    else
      ost << "Root tree display is not turned on." << endl;
  }
  else {
    int val = 0;
    int succ = 0;
    if (visual)
      val = _svl->find(toDisplay);
    else
      val = _curr_rbList->topSym()->find(toDisplay);
    if (val == 0)
      ost << "Display of " << toDisplay << " was not turned on." << endl;
    else if (val == 1) {
      if (visual) {
	int index = _svl->iteratorNextViewIndex();
	vis->removeFromView(index);
	succ = _svl->deleteNext();
      }
      else
	succ = _curr_rbList->topSym()->deleteNext();
      if (succ)
	ost << "Display of " << toDisplay << " turned off." << endl;
      else
	ost << "ERROR in deleting node from symList" << endl;
    }
    else if (val == 2) {
      if (visual) {
	int index = _svl->iteratorViewIndex();
	vis->removeFromView(index);
	succ = _svl->pop();
      }
      else
	succ = _curr_rbList->topSym()->pop();
      if (succ)
	ost << "Display of " << toDisplay << " turned off." << endl;
      else
	ost << "ERROR in deleting node from symList" << endl;
    }
  }
  doPrintOST();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: outputNode
// Parameters: (int num, char* val, CIdent* myNode)
// Returns: void
//
// This method is used only in the command line version of the debugger. It
// takes a CIdent (that is holding the value of a variable) and outputs this
// variable (val) and its textual value along with its display number (num).
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::outputNode(int num, char* val, CIdent* myNode) {
  char* res = (myNode->Child_Look_Up())->representation();
  ost << num << ": " << val << " = " << res << endl;
  delete[] res;
  doPrintOST();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: displayAndDeleteList
// Parameters: (SymbolList * sym, CState* s)
// Returns: void
//
// This method takes a symbol list (sym) and checks to see if each variable
// in this list is currently bound in the current state (s). If it is, then
// it is displayed. Otherwise, it is removed from the list. It also displays
// the current and root trees of the appropriate flags are set to true
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::displayAndDeleteList(SymbolList * sym, CState* s) {
  char* val;
  char * res = NULL;
  if ((displayCurrTree) && !visual){ //if should textually display current tree
    ost << "Current Tree: " << (res = (s->Store())->representation()) << endl;
    if (res) {delete [] res; res = NULL; }
  }
  if ((displayRootTree) && (!visual)) { //if should textually display root tree
    NonType* treeToDisplay;
    //the root KOLA tree is not maintained anywhere, so it has to be
    //recreated from the current state, s using the histUnravel method
    if (_histList->topVal())
      treeToDisplay = histUnravel(s);
    else
      treeToDisplay = s->Store()->Copy_kTree();
    ost << "Root Tree: " << (res = treeToDisplay->representation()) << endl;
    if (res) {delete [] res; res = NULL; }
    if (treeToDisplay) {delete treeToDisplay; treeToDisplay = NULL; }
  }
  CIdent * myNode;

  // now check the variables in the symbol list to see if
  // they should be displayed
  sym->initIterator();
  if (val = sym->iteratorValue()) {
    myNode = s->IFindDeepDB(val);
    if (myNode) 
      outputNode(sym->iteratorNum(), val, myNode);
    else
	sym->setIteratorNum(-1);
  }
  if (val = sym->iteratorNextValue()) {
    myNode = s->IFindDeepDB(val);
    if (myNode)
      outputNode(sym->iteratorNextNum(), val, myNode);
    else
      sym->setIteratorNextNum(-1);
  }
  while (val = sym->getNextValue()) {
    myNode = s->IFindDeepDB(val);
    if (myNode) 
      outputNode(sym->iteratorNextNum(), val, myNode);
    else
      sym->setIteratorNextNum(-1);
  }
  sym->delete_marked_nodes();
  doPrintOST();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: reaffirmAndDeleteList
// Parameters: (CState* s)
// Returns: void
//
// This method is used only in Visual COKO. The _svl is a list of all windows
// currently being displayed by the visual debugger besides the root, current,
// and isolate tree display windows (leaving just windows that display branches
// of the tree that are bound to variables). For each window specified, this
// method looks up the value of the variable and sets the approprate variable
// in the window so that the correct tree will be displayed. In other words,
// this method "refreshes" all of the variable display windows.
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::reaffirmAndDeleteList(CState* s) {
  char* val;
  char * res = NULL;
  CIdent * myNode;

  //get ready to iterate thorugh the symbol/window list
  _svl->initIterator();

  // if the first element of the svl list is not null
  if (val = _svl->iteratorValue()) {
    myNode = s->IFindDeepDB(val);  //see if it is in the current context
    if (myNode) //if it is, then set the approprate window to display the value
      _svl->iteratorPW()->setCurrent(myNode->Child_Look_Up());
    else {
      _svl->setIteratorNum(-1); //else, flag it for later deletion
      int index = _svl->iteratorViewIndex();
      vis->removeFromView(index); //automatically close the window
    }
  }

  // do the same thing for the next element in the svl list
  if (val = _svl->iteratorNextValue()) {
    myNode = s->IFindDeepDB(val); //see if it is in the current context
    if (myNode) //if it is, then set the approprate window to display the value
      _svl->iteratorPW()->setCurrent(myNode->Child_Look_Up());
    else {
      _svl->setIteratorNextNum(-1); //else, flag it for later deletion
      int index = _svl->iteratorNextViewIndex();
      vis->removeFromView(index); //automatically close the window
    }
  }

  //now do the same thing for every other element in the svl list.
  //I had to do the first two seperately because in order to iterate,
  //I had to use the getNextValue method
  while (val = _svl->getNextValue()) {
    myNode = s->IFindDeepDB(val); //see if it is in the current context
    if (myNode) //if it is, then set the approprate window to display the value
      _svl->iteratorPW()->setCurrent(myNode->Child_Look_Up());
    else {
      _svl->setIteratorNextNum(-1); //else, flag it for later deletion
      int index = _svl->iteratorNextViewIndex();
      vis->removeFromView(index);//automatically close the window
    }
  }
  _svl->delete_marked_nodes(); //delete the marked nodes
  doPrintOST();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doUndefindCommand
// Parameters: (char* command)
// Returns: void
//
// If the user typed a command that is not recognized by the debugger, this
// method will be called to inform the user.
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::doUndefinedCommand(char* command) {
  ost << "Undefined command: \"" << command << "\". Try \"help\"." << endl;
  doPrintOST();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doSet
// Parameters: (char* par1, char* par2)
// Returns: void
//
// If the user typed a command that started with the word "set", this method
// will be called. There are three possible flags that the user can set:
// set rule on/off, set pattern on/off, and set success on/off. Set rule on
// will display to the screen information about rule firing every time COKO
// attempts to fire a rule. Set pattern on right now does nothing. Sst success
// on will print to the screen the success value of every COKO statement after
// that statement is finished executing. This method just checks the value of
// the second parameter (par1) for "rule". "pattern", and "success" (and 
// prints an error if it is not one of those three) and sets the appropriate
// flag to on or off depending on the thrid parameter (par2).
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::doSet(char* par1, char* par2) {
  if (strcmp(par1,"rule")==0)
    if (strcmp(par2,"on")==0)
      if (rule_on)
	ost << "Rule display already on." << endl;
      else {
	rule_on = 1;
	ost << "Rule display turned on." << endl;
      }
    else if (strcmp(par2,"off")==0)
      if (!rule_on)
	ost << "Rule display already off." << endl;
      else {
	rule_on = 0;
	ost << "Rule display turned off." << endl;
      }
    else
      ost << "on or off expected." << endl;
  else if (strcmp(par1,"success")==0)
    if (strcmp(par2,"on")==0)
      if (success_on)
	ost << "Success display already on." << endl;
      else {
	success_on = 1;
	ost << "Success display turned on." << endl;
      }
    else if (strcmp(par2,"off")==0)
      if (!success_on)
	ost << "Success display already off." << endl;
      else {
	success_on = 0;
	ost << "Success display turned off." << endl;
      }
    else
      ost << "on or off expected." << endl;
  else if (strcmp(par1,"pattern")==0)
    if (strcmp(par2,"on")==0)
      if (pattern_on)
	ost << "Pattern display already on." << endl;
      else {
	pattern_on = 1;
	ost << "Pattern display turned on." << endl;
      }
    else if (strcmp(par2,"off")==0)
      if (!pattern_on)
	ost << "Pattern display already off." << endl;
      else {
	pattern_on = 0;
	ost << "Pattern display turned off." << endl;
      }
    else
      ost << "on or off expected." << endl;
  else
    ost << "rule or success expected." << endl;
  doPrintOST();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doHelp
// Parameters: (char* par1)
// Returns: void
//
// If the user typed the command: "help" this method will be called. It simply
// displays a list of all the command the user can type. At some point, this
// should be updated so that the user can type "help clear" or "help print"
// and get more detailed instructions on those specific commands.
//
//////////////////////////////////////////////////////////////////////////////
    
void DDebugController::doHelp(char* par1) {
  if ((!par1) || (strcmp(par1,"")==0)) {
    ost << "Legal Commands:" << endl << "quit" << endl << "exit" << endl << "list" << endl << "break" << endl << "condition" << endl << "clear" << endl << "continue" << endl << "step" << endl << "next" << endl << "output" << endl << "print" << endl << "display" << endl << "set rule on" << endl << "set rule off" << endl << "set success on" << endl << "set success off" << endl;
  }
  else
    ost << "Documentation not completed yet." << endl;
  doPrintOST();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doStack
// Parameters: ()
// Returns: void
//
// If the user typed the command: "stack" this method is callled and will call
// the displayStack in vis (if Visual COKO) or on the _curr_rbList directly if
// the command line version is being run. These methods will simply print
// to the screen in some way the stack of all the COKO transformation calls
// up to the current COKO transformation.
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::doStack() {
  if (visual) {
    vis->displayStack();
  }
  else {
    char* temp = _curr_rbList->displayStack();
    ost << temp << endl;
    doPrintOST();
    delete[] temp; //don't want to leak memory
    temp = NULL;
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doExit
// Parameters: ()
// Returns: void
//
// If the user typed the command: "exit" or "quit" this method is callled and
// will ask the user if they really want to exit. If they do, a system exit
// is called.
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::doExit() {
  if (!visual) {
    ost << "The program is running.  Exit anyway? (y or n) ";
    doPrintOST();
    string x;
    while (true) {
      getInput(&x);
      if ((x) == "y") {
	exit(0);
      }
      else if ((x) == "n") {
	break;
      }
      else {
	ost << "Please answer y or n." << endl;
	ost << "The program is running.  Exit anyway? (y or n) ";
	doPrintOST();
      }
    }
  }
  else {
    int ans = vis->checkDoAgain(3);
    if (ans) {
      exit(0);
    }
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doRun
// Parameters: ()
// Returns: int
//
// If the user typed the command: "run" this method is called and will ask
// the user if they really want to restart the program. If they do, the
// necessary variables are reset, the restart flag is set to true, and the
// program continues. Later on, the program will check the value of the
// restart flag (using the really_execute method) to stop excecuting the
// current run. 
//
//////////////////////////////////////////////////////////////////////////////

int DDebugController::doRun() {
  if (!visual) {
    ost << "The program being debugged has been started already." << endl;
    ost << "Start it from the beginning? (y or n) ";
    doPrintOST();
    string x;
    while (true) {
      getInput(&x);
      if ((x) == "y") {
	restart = 1;
	resetVariables();
	//breakStep=1;
	return 0;
      }
      else if ((x) == "n") {
	ost << "Program not restarted." << endl;
	doPrintOST();
	return 1;
      }
      else {
	ost << "Please answer y or n." << endl;
	ost << "The program being debugged has been started already." << endl;
	ost << "Start it from the beginning? (y or n) ";
	doPrintOST();
      }
    }
  }
  else {
    int ans = vis->checkDoAgain(2); //make sure user really wants to restart
    if (ans) {
      restart = 1;
      resetVariables();
      return 0;
    }
    else
      return 1;
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doTrack
// Parameters: ()
// Returns: void
//
// If the user typed the command: "track <transname>" this method is called and
// will place the specified transformation on the tracking list. Once on the
// tracking list, the user can place breaks on that transformation even if
// it is not the current tranformation that is being executed.
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::doTrack(char* transName)
{
  if (visual)
    vis->expandTrans(transName);
  else {
    //see if the transformation is already there
    int inList = _transList->find(transName);
    if (!inList) {
      int temp = findCurrent(NULL, transName);
      if (temp == 0)
	cout << "Transformation " << transName << " not found in the stack." << endl;
      else {
	_transList->insert(_break_rbList->getIterator(), NULL, NULL, 0);
	cout << "Transfomation " << transName << " placed in tracking list." << endl;
      }
    }
    else if (inList){
      cout << "Transformation " << transName << " already being tracked." << endl;
    }
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: testRunAgain
// Parameters: ()
// Returns: void
//
// Once the COKO program has finished executing, this method will be called to
// check if the user wants to run the program again, or just finish.
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::testRunAgain()
{
  if (!restart) {
    if (!visual) {
      ost << "The program has finished." << endl;
      ost << "Start it again from the beggginning? (y or n) ";
      doPrintOST();
      string x;
      while (true) {
	getInput(&x);
	if ((x) == "y") {
	  restart = 1;
	  breakStep=0;
	  resetVariables();
	  //currentlyRunning = 0;
	  //breakStep=1;
	  return;
	}
	else if ((x) == "n") {
	  return;
	}
	else {
	  ost << "Please answer y or n." << endl;
	  ost << "The program has finished." << endl;
	  ost << "Start it again from the beginning? (y or n) ";
	  doPrintOST();
	}
      }
    //delete x;
    }
    else {
      //close any variable display or branch isolation windows
      vis->closeAllSVLWindows();
      int ans = vis->checkDoAgain(1); //ask the user if they want to restart
      if (ans) {
	restart = 1;
	breakStep = 0;
	resetVariables();
      }
    }
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: analyzeInput
// Parameters: (char** parsedInput, int kn, CState* s)
// Returns: int
//
// This method is called once the input is parsed to perform the appropriate
// command. It checkes the first parameter (which in every case is the command)
// and calls the appropriate method depending on the command. Some commands
// don't need separate methods, so they are dealt with directly in this method.
// An integer is returned that specifies whether the command should return
// control back to the COKO execution (such as continue or next) or if the
// user should retain control (as in the case of most other commands). 0
// means return control to the execution, and 1 means retain control in the
// debugger.
//
//////////////////////////////////////////////////////////////////////////////

int DDebugController::analyzeInput(char** parsedInput, int kn, CState* s) {
  if (!parsedInput[0])
    return 1;
  else if ((strcmp(parsedInput[0], "quit") == 0) || (strcmp(parsedInput[0],"exit")==0)) {
    doExit();
    return 1;
  }
  else if (strcmp(parsedInput[0],"list") == 0) {
    doBreakOptions();
    return 1;
  }
  else if (strcmp(parsedInput[0],"break") == 0) {
    doAddBreak(parsedInput[1], kn);
    return 1;
  }
  else if (strcmp(parsedInput[0],"condition") == 0) {
    doAddConditionBreak(parsedInput[1], kn);
    return 1;
  }
  else if (strcmp(parsedInput[0],"clear") == 0) {
    doClearBreak(parsedInput[1], kn);
    return 1;
  }
  else if ((strcmp(parsedInput[0],"continue")==0) || (strcmp(parsedInput[0],"c")==0)) {
    return 0;
  }
  else if ((strcmp(parsedInput[0],"step")==0) ||(strcmp(parsedInput[0],"s")==0)) {
    breakStep=1;
    return 0;
  }
  else if ((strcmp(parsedInput[0],"next")==0) || (strcmp(parsedInput[0],"n")==0)) {
    _curr_rbList->set_topBN(1);
    return 0;
  }
  else if ((strcmp(parsedInput[0],"output")==0) || (strcmp(parsedInput[0],"print")==0)) {
    doPrint(parsedInput[1], s);
    return 1;
  }
  else if (strcmp(parsedInput[0],"display") == 0) {
    doDisplay(parsedInput[1], s);
    return 1;
  }
  else if (strcmp(parsedInput[0],"undisplay") == 0) {
    doUndisplay(parsedInput[1], s);
    return 1;
  }
  else if (strcmp(parsedInput[0],"set") == 0) {
    doSet(parsedInput[1], parsedInput[2]);
    return 1;
  }
  else if (strcmp(parsedInput[0],"help") == 0) {
    doHelp(parsedInput[1]);
    return 1;
  }
  else if (strcmp(parsedInput[0],"stack") == 0) {
    doStack();
    return 1;
  }
  else if (strcmp(parsedInput[0],"run") == 0) {
    return doRun();
  }
  else if ((parsedInput[0][0]=='<') && (strcmp(parsedInput[1],"break") == 0)){
    doAddBreakSpecific(parsedInput[0], parsedInput[2], kn);
    return 1;
  }
  else if ((parsedInput[0][0]=='<') && (strcmp(parsedInput[1],"condition") == 0)){
    doAddConditionBreakSpecific(parsedInput[0], parsedInput[2], kn);
    return 1;
  }
  else if ((parsedInput[0][0]=='<') && (strcmp(parsedInput[1],"clear") == 0)){
    doClearBreakSpecific(parsedInput[0], parsedInput[2], kn);
    return 1;
  }
  else if (strcmp(parsedInput[0],"track") == 0) {
    doTrack(parsedInput[1]);
    return 1;
  }
  else if (strcmp(parsedInput[0],"eddie") == 0) {
    ost << "Easter egg eddie eerily eats empty eggs." << endl;
    doPrintOST();
    return 1;
  }
  else if (strcmp(parsedInput[0],"dna") == 0) {
    ost << "Tell me what you think. My email address: dna@alumni.brandeis.edu." << endl;
    doPrintOST();
    return 1;
  }
  else if (strcmp(parsedInput[0],"mfc") == 0) {
    ost << "There once was an advisor called Mitch" << endl;
    ost << "For pseudocode he always insists" << endl;
    ost << "While a pain at the time" << endl;
    ost << "It's clear down the line" << endl;
    ost << "The final code this planning did enrich" << endl;
  }
  else {
    doUndefinedCommand(parsedInput[0]);
    return 1;
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ruleOn
// Parameters: ()
// Returns: int
//
// This 1-line method simply returns the value of the rule_on flag.
//
//////////////////////////////////////////////////////////////////////////////

int DDebugController::ruleOn() {
  return rule_on;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: check_for_condition
// Parameters: (CState* s, char* rep, int key_number)
// Returns: void
//
// This method is called from the COKO statement classes after they are
// finished to check if the success flag is set. If it is, then the success
// variable in the CState parameter s checked and printed to the screen. In
// addition, if the success value was true, this method checks to see if there
// was a conditional break set on that statement (in which case a global
// variable is set saying that the debugger should stop as soon as it can).
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::check_for_condition(CState* s, char* rep, int key_number) {
  int succ = s->ISuccess();
  if (success_on)
    {
	_current = _curr_rbList->topRB();
	if (_current) {
	  ost << endl << "Statement: ";
	  // get the name of the transformation
	  char* name = _current->get_name();
	  ost << name << "::";
	  // format the result using the key_number
	  format_output(key_number, rep);
	  ost << "evaluated to ";
	  if (succ)
	    ost << "TRUE." << endl;
	  else
	    ost << "FALSE." << endl;
	  doPrintOST();
	  delete [] rep;
	  delete [] name;
	}
    }
  if (succ) //now see if there was a conditional break on this statement
    {
      int temp = findCurrent(); //find the current transformation
      if ((temp) && ((_break_rbList->iteratorCA())[key_number-1])) {
	_current = _curr_rbList->topRB();
	//announce that the debugger is stopping because the conditional
	//break occured
	ost << endl << "Condition:" << endl;
	char* name = _current->get_name();
	ost << name << "::";
	format_output(key_number, rep);
	ost << "evaluated to true." << endl << endl;
	doPrintOST();
	breakStep=1; //this says stop as soon as you get the chance
	delete [] rep;
	delete [] name;
      }
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: setToExec
// Parameters: (NonType* toChange, char* name, char* key)
// Returns: NonType*
//
// This method sets the global variable toExec when a COKO transformation
// is about to call a sub-transformation on a branch of the tree (toExec stands
// for to execute). toExec points to this branch of the tree. In addition, the
// variable num of the root of this branch is set to be negative so that an
// arrow can be drawn in the visual window to indicate that a subroutine is
// about to be called. This branch with the new variable num is returned. In
// addition, the global variable "execName" is set so that the visual window
// can use this as the label of the arrow pointing to the tree branch.
//
//////////////////////////////////////////////////////////////////////////////

NonType* DDebugController::setToExec(NonType* toChange, char* name, char* key)
{
  toExec = toChange;
  if (toExec) {
    int x;
    //set the variable number to negative, but don't lose its magnitude
    if ((x = toExec->getVariableNum()) > 0)
      toExec->setVariableNum(x*-1);
    else 
      toExec->setVariableNum(-2); //if the variable num was 0, just make it -2
  }
  if (execName)
    delete[] execName; //delete the old execName
  execName = name;
  if (execKey)
    delete[] execKey; //delete the old execKey
  if (key) {
    execKey = new char[strlen(key) + 1];
    strcpy(execKey, key);
  }
  else
    execKey = NULL;
  return toExec;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: resetOldVariableNum
// Parameters: (NonType* toExec, char* key, CState* s)
// Returns: NonType*
//
// This method takes a NonType (pointer to a KOLA tree) and resets all of the
// variable-nums of each node in the tree back to being a positive number (ie
// if that node was marked (the mark is done by giving the varibale-num of that
// node a negative number) as the node upon which a sub-transformation was
// about to be called, then we now remove this "mark"). This method is
// currently called from the PCRuleInvokeStmt class after the debugger has
// returned control to this class and just before the transformation call
// actually occurs. The char* key paramater is the name of the variable that
// is bound to the branch of the tree that the sub-transformation is being
// called upon. For this method, I only care whether or not key is NULL. If
// it is not NULL, then s->Store has to be rebuilt to reflect these changes in
// variableNum - otherwise the change was only to the root and the tree does
// not have to be rebuilt. The function returns back the toExec tree with
// the updated variableNums.
//
//////////////////////////////////////////////////////////////////////////////

NonType* DDebugController::resetOldVariableNum(NonType* toExec, char* key, CState* s)
{
  if (toExec) {
    int x = toExec->getVariableNum();

    // if the variable num was negative, but not == -2 then simply multiply
    // by -1 to make it positive. If it was == -2 then make the new variable
    // num 0
    if (x < -2)
      toExec->setVariableNum(x*-1);
    else if (x == -2)
      toExec->setVariableNum(0);
    else if (x == -1)
      toExec->setVariableNum(1);

    // if key is not NULL then rebuild s->Store to reflect these changes in
    // variableNum
    if (key) {
      s = tempStmt->updateDEP(key, s, s->IEnv()->getData(), 1);
      if (s->CurrentPattern())
	s = s->CurrentPattern()->buildCurrent(s);
    }
  }
  return toExec;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: check_for_break
// Parameters: (int key_number)
// Returns: bool
//
// This method checks to see if the debugger should break and take control
// before executing the next COKO statement. It is called at the beginning of
// every ExecDebug method of each COKO statement class. This method takes an 
// int (key_number) and finds if there is a break on that key (statement of 
// COKO code) in the current transformation (the transformation that is 
// currently executing). Alternatively, it checks if the debugger should break
// for other reasons (such as if the user typed "step" or next". If any of
// these reasons are valid, it returns true indicating that the debugger
// should break. Otherwise it returns false.
//
//////////////////////////////////////////////////////////////////////////////

int DDebugController::check_for_break(int key_number) {
  int temp = findCurrent();
  //if we should stop because "step" was indicated
  if (breakStep) {
    breakStep=0;
    return 1;
  }
  //if we should stop because "next" was indicated
  else if (_curr_rbList->topBN()) {
    int temp = _curr_rbList->set_topBN(0);
    if (temp == 0) exit(1);
    return 1;
  }
  //if we should stop because the user placed a break on this statement
  else if ((temp) && ((_break_rbList->iteratorBA())[key_number-1])) {
    return 1;
  }
  else  //we shouldn't stop
    return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: findCurrWhich
// Parameters: (NonType* currNT)
// Returns: int
//
// This method looks through its KOLA tree parameter (currNT) and finds the
// parent of the node whose variableNum is not even (which signifies that
// that is the "current" node of the tree and should be drawn with the color
// green on the visual display of that tree). After finding the parent of this
// node, it returns which child of the parent this node is (ie it returns 1
// if it is the first child of the parent, 2 if it is the second child, etc).
// This information is used in the breakLoop method to replace this branch of
// the tree (from this special node downwards) with s->Store. But in order to
// call the replaceChild method on the parent, the second parameter needs to
// know which child - hence this method is called.
//
//////////////////////////////////////////////////////////////////////////////

int DDebugController::findCurrWhich(NonType* currNT) {
    NonType* child;
    int currChild = 1;
    int temp = 1;
    //get ready to loop through the children of the current node
    currNT->initCurrChild();
    //loop through each child
    while (!((child=currNT->nextChild()) == NULL)) {
      //if this child is not even, then return which child it is
      if (child->getVariableNum() % 2 != 0)
	return currChild;
      else //otherwise recurse down the tree
	temp = findCurrWhich(child);
      if (temp) //if we found a value, then return it upwards
	return temp;
      currChild++; //otherwise move on to the next child
    }
    return 0; //no more children to check - this branch of the tree does not contain the node whose variableNum is not even
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: findCurr
// Parameters: (NonType* currNT)
// Returns: NonType
//
// This method looks through its KOLA tree parameter (currNT) and finds the
// parent of the node whose variableNum is not even (which signifies that
// that is the "current" node of the tree and should be drawn with the color
// green on the visual display of that tree). It returns a pointer to this
// parent.
//
//////////////////////////////////////////////////////////////////////////////

NonType* DDebugController::findCurr(NonType* currNT) {
    NonType* child;
    NonType* temp;
    //get ready to loop through the children of the current node
    currNT->initCurrChild();

    //loop through each child
    while (!((child=currNT->nextChild()) == NULL)) {
      //if child is not even then return the parent
      if (child->getVariableNum() % 2 != 0)
	return currNT;
      else //recuse down the tree
	temp = findCurr(child);
      if (temp) //if we found the parent, then return it
	return temp;
      //else go on to the next child
    }
    return NULL;//no more children to check - this branch of the tree does not contain the node whose variableNum is not even
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: scaleVariableNums
// Parameters: (NonType* currNT)
// Returns: void
//
// This method takes a KOLA tree input (currNT) and scales the variableNum
// of each node in the tree to a number between [-2, 1]. In so doing, it
// retains the sign of the variableNum (if it was negative before it will
// remain negative and if it was positive before it will remain positive or 0)
// and whether or not it was divisible by 2 (odd numbers remain odd, even
// numbers remain even). Thus the mapping is:
// negative and odd  --> -1
// negative and even --> -2
// positive and odd  --> 1
// positive and even --> 0
//
// By scaling variableNums to be within [-2, 1] this method takes away
// magnitude information from the variableNums where each variable number
// corresponds to a variable in the environment. This magnitude information is
// set in the method visMarkVariables, which is usually called after the
// the variableNums have been scaled in this method.
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::scaleVariableNums(NonType* currNT) {
  if (!currNT)
    return;
  int x = currNT->getVariableNum();
  //scale the current variableNum to [-2, 1]
  if (x % 2 == 0) {
    if (x < 0)
      x = -2;
    else
      x = 0;
  }
  else {
    if (x < 0)
      x = -1;
    else
      x = 1;
  }
  currNT->setVariableNum(x);
  NonType* child;
  //recurse for each child of the tree
  currNT->initCurrChild();
  while (!((child=currNT->nextChild()) == NULL))
    scaleVariableNums(child);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: visMarkVariables
// Parameters: (CState *s)
// Returns: void
//
// This method takes a State (s) and goes through the environment of that
// state. For each variable that is found in the environment, it assigns a
// variableNum to that node starting from 4, and then counting by twos upwards
// (4, 6, 8, 10, etc). Before setting the variableNum of the NonType (KOLA
// tree) stored in that variable, it checks to see what it used to be. If it
// was negative (meaning that a COKO transformation is about to be called
// on this node) this information is preserved by assigning it a new
// variableNum that is -1 times what it would have otherwise given it (ie
// instead of giving it 4, it gives it -4). Once the variableNum has been
// changed for each KOLA tree pointed to by each variable, s->Store is rebuilt
// to reflect these changes in variableNums (this is because unfortunately,
// the COKO system was built without the debugger in mind, and for this reason
// the KOLA trees that are pointed to by each variable in the environment are
// only copies of the branches of the corresponding current tree, so any
// changes made to these copies must also be made to the current KOLA tree
// in s->Store).
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::visMarkVariables(CState *s) {
  //delete all of the nodes in _symList - we will recreate them later in this method
  _symList->delete_all_nodes();
  varCounter = 4; //start counting from 4
  CIEnvNodeType* currNode = NULL;
  CIEnvValueType* currValue = NULL;
  CIdent* currIdent = NULL;
  NonType* currNT = NULL;
  const char* tempKey;
  char* copiedKey;
  CPattern* currPattern = NULL;
  int currVarNum;
  //get ready to explore the environment
  CIEnvStackDataType* startingPt = s->IEnv()->getData();
  //loop through each element in the environment
  while (startingPt) {
    currNode = startingPt->_item->firstNode();
    while (currNode) {
      currValue = currNode->GetData();
      if (currValue) { //this means that we found a variable
	currIdent = currValue->GetContent(); //get the content of the variable
	tempKey = currValue->GetKey(); //get the name of the variable

	//copy the variable name
	copiedKey = new char[strlen(tempKey) + 1];
	strcpy(copiedKey, tempKey);

	if (currIdent) { //if the content of the variable is non-NULL

	  //find the KOLA tree to which the variable is bound
	  currNT = currIdent->Child_Look_Up();

	  if (currNT) {

	    //check to see if the current variableNum of this tree is negative
	    currVarNum = currNT->getVariableNum();
	    if (currVarNum < 0) //if so, preserve this information
	      currVarNum = varCounter * -1;
	    else
	      currVarNum = varCounter; //otherwise just use the current counter
	    currNT->setVariableNum(currVarNum);

	    //make sure that to make sure that the Pattern for this variable also gets changed (if an not sure if this is necessary, but just being safe)
	    currPattern = currValue->GetPattern();
	    if (currPattern)
	      currPattern->updateNTVarNum(currVarNum);

	    //add this variable into the symbol list with its variable counter
	    _symList->insert(tempKey, varCounter);
	   
	    varCounter+=2; //update the variable counter

	    //rebuild s->Store to reflect the above change in one of the variableNums
	    s = tempStmt->updateDEP(copiedKey, s, s->IEnv()->getData(), 1);
	    if (s->CurrentPattern())
	      s = s->CurrentPattern()->buildCurrent(s);
	  }
	}
	delete[] copiedKey;
      }
      currNode = currNode->_next; //go on to the next node in the environment
    }
    startingPt = startingPt->_next; //go on to the next element
  }    
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: updatePaintWindow
// Parameters: (CState *s)
// Returns: void
//
// This method tells the visual display window (vis) to update itself, sending
// the variable symbol list (_symList), and the current tree (s->Store()) 
// which is found using the State parameter to this method (s).
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::updatePaintWindow(CState* s) {
  vis->updatePaintWindows(s->Store(), s->Store(),_symList, NULL);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doBreak
// Parameters: (CEqn* eqn, CState *s)
// Returns: void
//
// This method is called from any COKO statement class that inherits from
// CEqn. It is called if its previous call to check_for_break returned true.
// If so, then it passes along a pointer to itself (eqn) and a pointer to
// the current state (s). The key number of the equation inside the current
// transformation and the rep_string of the equation are found from eqn and 
// this information, along with s, is passed along to the breakLoop method
// which takes control and allows the user to indicate his/her wishes while
// the program execution has paused.
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::doBreak(CEqn* eqn, CState *s) {
  int kn = eqn->get_key_number();
  char* rep = eqn->rep_string();
  breakLoop(kn, rep, s);
  delete[] rep;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: breakLoop
// Parameters: (int kn, char* rep, CState *s)
// Returns: void
//
// This is the central and most important method in this class! If the visual
// option is turned on, then this method gathers all of the current
// information: the transformation code, the global tree, the current tree,
// and all trees bound by variables, and sends this information to vis (the
// visual coko display controller) which displays this information to the user.
// In addition, this method sets the variableNums of all of the nodes in these
// trees so that the tree drawer can display arrows pointing to all bound
// variables and so that the current node can be painted green. If the visual
// option is not turned on, then the only information displayed is the current
// statement upon which the COKO transformation has stopped. Further
// information must be explicitly requested by the user.
//
// Once the information has been display, the method loops while the user
// types commands that do not return control back to program execution (such as
// break x, display y, print z, etc) - performing those commands and waiting
// for the next command. Otherwise (if the user typed step, next, or continue)
// global variables are set and control is returned back to the COKO program
// execution.
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::breakLoop(int kn, char* rep, CState * s) {
  NonType* treeToDisplay;
  _current = _curr_rbList->topRB();

  if (!visual) {
    //If this is the command line version of the debugger, then simply output
    //to the screen the statement that the debugger has stopped on. The format
    // is <transformation name>::<key number>|<statement representation> for
    // example: "CNFAux::3|dist1"
    ost << endl << "Debugger stopped on:" << endl;
    char* name = _current->get_name();
    ost << name << "::";
    format_output(kn, rep);
    ost <<  endl;
    delete [] name;
  }
  string x;

  //prepare the parsedInput array to accept user input
  char** parsedInput;
  parsedInput = new char*[maxParameters];
  for (int i = 0; i < maxParameters; i++)
    parsedInput[i] = NULL;

  //check each variable in the symbol list to see if they exist in the current
  //environment. If they don't then delete them from the list, if they do, then
  //textually display them.
  displayAndDeleteList(_curr_rbList->topSym(), s);

  ost << endl;
  ost << "COKODB> ";
  doPrintOST();

  if (visual) {
    //if visual mode is on, then we have to do the following:

    if (execKey) {
      //if execKey is non-NULL then a transformation is about to be called
      //on one of the nodes in the tree. To find which node, we have to rebuild
      //s->Store so that the variable nums of the KOLA tree pointed to by the
      //variable that this transformation is about to be called upon, will be
      //updated in s->Store
      s = tempStmt->updateDEP(execKey, s, s->IEnv()->getData(), 1);
      if (s->CurrentPattern())
    	s = s->CurrentPattern()->buildCurrent(s);
    }

    //find out what the contents of the current transformation is so that it
    //can be displayed in the main Visual COKO window
    char* contents = _current->representation_with_debugging_info(this);

    //The root of the s->Store is the "current node" and should be colored
    //green in the visual representation of the KOLA tree. This is indicated
    //by making its variableNum odd (not even).
    s->Store()->setVariableNum(s->Store()->getVariableNum() + 1);

    //If s->Store is not the global root of the query, then we will have to
    //rebuild the global root using the histUnravel method (treeToDisplay is
    //a pointer to this global root)
    if (_histList->topVal())
      treeToDisplay = histUnravel(s);
    else //otherwise the global root is the current root
      treeToDisplay = s->Store()->Copy_kTree();

    //remove any magnitude information from the variableNums in treeToDisplay
    //and s->Store. Then reset these magnitudes (where magnitude is a counter
    //where each variable in the environment has a corresponding variable
    //number) in visMarkVariables.
    scaleVariableNums(treeToDisplay);
    scaleVariableNums(s->Store());
    visMarkVariables(s);

    //if the visMarkVaribales method call above caused the variable num in
    //s->Store to become even again, reset it to being odd
    if (s->Store()->getVariableNum() % 2 == 0)
      s->Store()->setVariableNum(s->Store()->getVariableNum() + 1);

    //if we changed the root with the above line then the global root is the 
    //root in s->Store, so we just have to display s->Store() and can delete
    //treeToDisplay
    if (treeToDisplay->getVariableNum() % 2 != 0) {
      delete treeToDisplay;
      treeToDisplay = s->Store();
    }
    //Otherwise, these roots are not the same and we are going to have to find
    //where the root of s->Store is located inside treeToDisplay. We then have
    //to replace this branch with s->Store itself (because changes may have
    //been made)
    else {
      NonType* tempNT = findCurr(treeToDisplay);
      tempNT->replaceChild(s->Store(), findCurrWhich(treeToDisplay));
    }  

    //refresh all of the variable display windows
    reaffirmAndDeleteList(s);

    //finally, send all of the information about the current ruleblock, the
    //global and current trees, and display information that we have learned so
    //far to vis (the visual display controller class) where this information
    //will be used to display the contents of the current transfomration and
    //various additional display windows of KOLA trees
    vis->displayRBInfo(contents, kn, currLCArray, currNumStmts, treeToDisplay, s->Store(),_symList, execName, s);
    delete[] contents;
    readyToGo = 0;
    //readyToGo will be set to true by vis when it is ready to return control to the debugger
    while (!readyToGo) 
      a->processOneEvent();
    parsedInput = vis->getParsedArray();
    displayRootTree=vis->getDisplayRT();
    displayCurrTree=vis->getDisplayCT();
  }
  else { //not Visual COKO, but command line COKO
    getInput(&x); //get command line input
    const char* charArrayToParse = x.c_str(); //convert string to char array so that we can access individual characters
    parseInput(charArrayToParse, parsedInput, maxParameters);
  }
  int keepGoing = analyzeInput(parsedInput, kn, s); //analyze the input

  //retain control which keepGoing is true- this is the main loop of the method
  while (keepGoing) {
    //clean parsedInput for reuse
    for (int i = 0; i < maxParameters; i++)
      if (parsedInput[i]) {
	delete[] parsedInput[i];
	parsedInput[i] = NULL;
      }
    ost << "COKODB> ";
    doPrintOST();

    if (visual) {
      reaffirmAndDeleteList(s); //refresh all of the variable display windows 
      vis->setCurrInformation(_symList, execName, treeToDisplay, s);
      readyToGo = 0;
      while (!readyToGo)
	a->processOneEvent();
      parsedInput = vis->getParsedArray();

      //set visual global variables
      displayRootTree=vis->getDisplayRT();
      displayCurrTree=vis->getDisplayCT();
    }
    else {
      getInput(&x);
      const char* charArrayToParse = x.c_str(); //convert string to char array so that we can access individual characters
      parseInput(charArrayToParse, parsedInput, maxParameters);
    }
    keepGoing = analyzeInput(parsedInput, kn, s);
  }


  //Do I need to delete x?
  //delete &x;

  //clean parsedInput
  for (int i = 0; i < maxParameters; i++)
    if (parsedInput[i]) {
      delete[] parsedInput[i];
      parsedInput[i] = NULL;
    }
  if (visual)
    {
      //clean variableNums
      s->Store()->setVariableNum(s->Store()->getVariableNum() - 1);
      scaleVariableNums(treeToDisplay);

      //set visual global variables
      displayRootTree=vis->getDisplayRT();
      displayCurrTree=vis->getDisplayCT();
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doBreak
// Parameters: (CStmt* stmt, CState *s)
// Returns: void
//
// This method is called from any COKO statement class that inherits from
// CStmt. It is called if its previous call to check_for_break returned true.
// If so, then it passes along a pointer to itself (stmt) and a pointer to
// the current state (s). The key number of the COKO statment inside the
// current transformation and the rep_string of the statement are found from
// stmt and this information, along with s, is passed along to the breakLoop
// method which takes control and allows the user to indicate his/her wishes
// while the program execution has paused.
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::doBreak(CStmt* stmt, CState* s) {
  int kn = stmt->get_key_number();
  char* rep = stmt->rep_string();
  breakLoop(kn, rep, s);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: start
// Parameters: (CRuleBlock* current)
// Returns: void
//
// This method is called to start the debugging process on the root
// transformation. Two options are possible: this is the first time through
// this transformation (the user just started up Visual COKO or Command Line
// COKO) or this is not the first time through this transfomration (the user
// restarted the debugger). In the former case, the current transformation is
// added to the _curr and _break_rbList and a welcome line is printed to the
// screen. In the latter case, the same thing occurs but the _break_rbList
// is not reinitialized (so that the breaks that the user placed on various
// statements will be remembered even though execution got restarted).
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::start(CRuleBlock* current) {
  //if the program is running for the first time (the debugger just started)
  if (!currentlyRunning) {
    currentlyRunning=1;
    _root = current;
    _current = current;

    //add the current transformation to the _curr and _break _rbLists
    _curr_rbList->insert(current);
    _curr_rbList->set_topBN(1);
    _break_rbList->insert(current, current->get_num_stmts());

    if (!visual)
      ost << "Welcome to Command Line COKODB!" << endl;
    doPrintOST();
    //breakStep = 1;
  }
  else { //the user restarted execution
    _current=current;
    if (_curr_rbList->topRB()) //not sure if this is ever true?
      _curr_rbList->insert(current);
    else {
      _curr_rbList->insert(current);
      _curr_rbList->set_topBN(1);
      }
    int temp = findCurrent();
    if (!(temp)) { //not sure if this is ever true?
      _break_rbList->insert(current, current->get_num_stmts());
    }
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: stop
// Parameters: ()
// Returns: void
//
// This method is called when the debugger is finished executing the root
// transformation. This root transfomration (rule block) is simply popped off
// the _curr_rbList stack.
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::stop() {
  int temp = _curr_rbList->pop();
  if (temp == 0) exit(1);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: format_output
// Parameters: (char* first, char* second)
// Returns: void
//
// This method outputs to the textual display window the two parameters
// separated by a '|' character.
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::format_output(char* first, char* second)
{
  ost << first;
  ost << "|";
  ost << second << endl;
  doPrintOST();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: format_output
// Parameters: (int first, char* second)
// Returns: void
//
// This method outputs to the textual display window the two parameters
// separated by a '|' character.
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::format_output(int first, char* second)
{
  ost << first;
  ost << "|";
  ost << second << endl;
  doPrintOST();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: setReadyToGo
// Parameters: ()
// Returns: void
//
// This method sets the readyToGo member variable to equal 1.
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::setReadyToGo()
{
  readyToGo = 1;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: repInitialize
// Parameters: (int line, int col, int num_stmts)
// Returns: void
//
// This method takes textual display information about the current
// transformation including the number of lines (line) columns (col) and
// statements (num_stmts) and sets corresponding member variables to equal
// these parameter values. In addition, this method initializes an array
// (currLCArray) of size num_stmts that will eventually contain the line and
// column number of the beginning and end of each statement in the textual
// represenation of the current transformation.
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::repInitialize(int line, int col, int num_stmts)
{
  currLine = line;
  currCol = col;
  currNumStmts = num_stmts;
  currLCArray = new LCSubstring[num_stmts];
  for (int i = 0; i < num_stmts; i++) {
    currLCArray[i].startLine = 0;
    currLCArray[i].startCol = 0;
    currLCArray[i].endLine = 0;
    currLCArray[i].endCol = 0;
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: updateLine
// Parameters: (int sL, int sC)
// Returns: void
//
// This method is called if CMulStmtCon or CMulStmtDis added a '{' to the
// textual representation. If that is the case then every statement on that
// line must be moved over by one character to account for this '{'.
//
//////////////////////////////////////////////////////////////////////////////

void DDebugController::updateLine(int sL, int sC)
{
  //loop through every statement
  for (int i = 0; i < currNumStmts; i++) {
    if ((currLCArray[i].startLine == sL) && (currLCArray[i].startCol >= sC)) {
      currLCArray[i].startCol++;
      if (currLCArray[i].endLine == sL)
	currLCArray[i].endCol++;
    }
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: updateArray
// Parameters:	(int kn, int sL, int sC, int eL, int eC)
// Returns: void
//
// This method is called from the representationWDI mehtod of various COKO
// breakable statement classes. This method will update the currLCArray with
// input statment number (kn) the exact location (in terms of characters)
// inside the QT textarea window of this statement (using the start line (sL),
// end line (eL), start column (sC) and end column (eC)). This method also
// updates where we are in the text area using the end line and end column.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::updateArray(int kn, int sL, int sC, int eL, int eC)
{
  currLine = eL;
  currCol = eC;
  currLCArray[kn].startLine = sL;
  currLCArray[kn].startCol = sC;
  currLCArray[kn].endLine = eL;
  currLCArray[kn].endCol = eC;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getLine
// Parameters:	()
// Returns: int
//
// Returns the current line number in the text area that is being used to
// display the contents of the current transformation.
//         								    
//////////////////////////////////////////////////////////////////////////////

int DDebugController::getLine()
{
  return currLine;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: setLine
// Parameters:	(int)
// Returns: void
//
// Sets the current line number in the text area that is being used to
// display the contents of the current transformation.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::setLine(int line)
{
  currLine = line;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getCol
// Parameters:	()
// Returns: int
//
// Returns the current column number in the text area that is being used to
// display the contents of the current transformation.
//         								    
//////////////////////////////////////////////////////////////////////////////

int DDebugController::getCol()
{
  return currCol;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: setCol
// Parameters:	(int)
// Returns: void
//
// Sets the current column number in the text area that is being used to
// display the contents of the current transformation.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::setCol(int col)
{
  currCol = col;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: outputDBArray
// Parameters:	()
// Returns: void
//
// This method is for debugging purposed only. It displays to cout the current
// contents of currLCArray and currLine and currCol.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::outputDBArray()
{
  cout << "CURRENT ARRAY:" << endl;
  cout << "currLine = " << currLine << endl;
  cout << "currCol = " << currCol << endl;
  for (int i = 0; i < currNumStmts; i++) {
    cout << currLCArray[i].startLine << " " << currLCArray[i].startCol << " ";
    cout << currLCArray[i].endLine << " " << currLCArray[i].endCol << endl;
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doPrintOST
// Parameters:	()
// Returns: void
//
// This method displays to the screen whatever is stored so far in the member
// variable ost. If visual mode is turned on, then this text is printed to the
// console window. Otherwise, it is printed directly to the screen (cout).
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::doPrintOST()
{
  if (visual)
    vis->insertConsoleText(ost.str().c_str());
  else
    cout << ost.str();
  ost.str("");
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: printInRightPlace
// Parameters:	(char* toPrint)
// Returns: void
//
// This method prints whatever is stored in the Char* parameter (toPrint) in
// the correct location. If visual mode is turned on, then this text is
// printed to the console window. Otherwise, it is printed directly to the
// screen (cout).
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::printInRightPlace(char* toPrint)
{
  if (visual)
    vis->insertConsoleText(toPrint);
  else
    cout << toPrint;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: histPush
// Parameters:	(NonType* tree, int location, int which, char* v_name, CIEnvStack* eStack, NonType* currPattern)
// Returns: void
//
// This method simply passes along all of its parameters to the push method of
// member variable _histList (which is an instance of HistoryList and is used
// to reconstruct the global KOLA tree from the current KOLA tree and the
// environment stack stored in this _histList variable).
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::histPush(NonType* tree, int location, int which, char* v_name, CIEnvStack* eStack, NonType* currPattern)
{  
  _histList->push(tree, location, which, v_name, eStack, currPattern);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: histPop
// Parameters:	()
// Returns: void
//
// This method simply calls the pop() method of the _histList member variable.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::histPop()
{  
  _histList->pop();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: findNodeFromNumHelper
// Parameters:	(NonType* tree, int num)
// Returns: NonType*
//
// This method initializes the member variable histCounter to 0 and then calls
// the findNodeFromNum method which finds the "num"th element in a preorder
// traversal of the parameter: tree.
//         								    
//////////////////////////////////////////////////////////////////////////////

NonType* DDebugController::findNodeFromNumHelper(NonType* tree, int num)
{
  histCounter = 0; 
  return findNodeFromNum(tree, num);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: findNodeFromNum
// Parameters:	(NonType* tree, int num)
// Returns: NonType*
//
// This method recurses through the KOLA tree parameter (tree) counting each
// node that it visits. Once it has visited num nodes, it returns the tree
// that is rooted at the current node.
//         								    
//////////////////////////////////////////////////////////////////////////////

NonType* DDebugController::findNodeFromNum(NonType* tree, int num)
{
  //add one to the global (member) variable histCounter
  histCounter++;

  //if we have seen num nodes, then return the current tree
  if (histCounter == num)
    return tree;

  //otherwise continue to visit nodes in the tree recursively
  else {
    NonType* child;
    NonType* temp;
    tree->initCurrChild();
    while (!((child=tree->nextChild()) == NULL)) {
      temp = findNodeFromNum(child, num);
      if (temp)
	return temp;
    }
    return NULL;
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                           
// Function Name: histUnravel
// Parameters:	(CState*)
// Returns: NonType*
//
// This method takes a pointer to the current state (s) and unravels the 
// histList stack from the current state to the root of the stack. Each
// element in this histList stack is the current environment when a
// transformation was called on a branch of the tree resulting in a new
// environment to be initialized that contains only information pertaining
// to this new "zoomed in" portion of the tree. Thus this method can be seen
// as unraveling all of these environments to "zoom out" to get the original
// KOLA tree as it now stands. Since changes may have been made in each
// "zoomed in" frame, care must be take to make sure that this new version of
// this branch of the KOLA tree replaces the old branch in the older
// envirment in the stack frame above.
//
// This method is called when the user types: "display root" in the command
// line debugger, and in the root tree display in the visual debugger
//         								     
//////////////////////////////////////////////////////////////////////////////

NonType* DDebugController::histUnravel(CState *s)
{
  //copy the cuurent tree to currState variable
  NonType* currState = s->Store()->Copy_kTree();

  //initialze a new CState variable which will hold ...
  CState* new_s = new CState(currState, NULL, 0);

  _histList->initIterator(); //prepare to iterate through the _histList stack

  //get the KOLA tree pointed to by the top value in _histList
  NonType* tempVal = _histList->topVal()->Copy_kTree(); 
  int currLoc;
  int currWhich;
  char* curr_v_name;
  NonType* currPattern;
  CIEnvStack* curreStack;
  if (tempVal) {
    //if this tree is non-NULL then get location, child, variable name, environment, and pattern information about this tree from _histList
    currLoc = _histList->iteratorLocation();
    currWhich = _histList->iteratorWhich();
    curr_v_name = _histList->iteratorName();
    curreStack = _histList->iteratorStack();
    currPattern = _histList->iteratorPattern();

    //if the current tree is the result of a transformation call on a branch
    //of the KOLA tree in this stack element pointed to by a variable
    if (curr_v_name) {
      //I'm not really sure how the following code works because I copied it
      //from PCRuleInvokeStmt.C when it is trying to reconstuct new tree
      //when part of the tree pointed to by _second was potentially changed
      //because a COKO transformation was called on it. Here, curr_v_name is
      //the same as _second in PCRuleInvokeStmt.C. I figured that since
      //this method is doing the same this as PCRuleInvokeStmt.C when it is
      //reconsturcting the environment, it was ok to copy that code without
      //totally understanding it :)
      new_s->SetIEnv(curreStack);
      new_s->IModifyFromHere(curr_v_name, 
			 new_s->Store(), 
			 currWhich, new_s->IEnv()->getData());
      new_s = tempStmt->updateDEP(curr_v_name, new_s, NULL, 1);
      if (currPattern) {
	CPattern* tempPattern = new CPattern(currPattern->Copy_kTree());
	new_s->SetCurrentPattern(tempPattern);
	new_s = new_s->CurrentPattern()->buildFromHere(new_s, new_s->IEnv()->getData());
	delete tempPattern;
      }
    }

    //the transformation call that resulted in a new stack element was not on a branch of the tree pointed to by a variable
    else 
      { 
	//find where in the tree stored in this stack element the current tree
	//is located (using the currLoc counter variable) and replace this
	//branch of the tree with the current tree
	findNodeFromNumHelper(tempVal, currLoc)->replaceChild(currState, currWhich);
	//set the new current tree to be the result of this replacement
	new_s->SetStore(tempVal->Copy_kTree());
      }

    //now do the same thing as above, for every element in the _histList stack
    while (tempVal = _histList->getNextValue()) {
      tempVal = tempVal->Copy_kTree();
      currLoc = _histList->iteratorLocation();
      currWhich = _histList->iteratorWhich();
      curr_v_name = _histList->iteratorName();
      curreStack = _histList->iteratorStack();
      currPattern = _histList->iteratorPattern();
      
      if (curr_v_name) {
	new_s->SetIEnv(curreStack);
	new_s->IModifyFromHere(curr_v_name, 
			   new_s->Store(), 
			   currWhich, new_s->IEnv()->getData());
	new_s = tempStmt->updateDEP(curr_v_name, new_s, NULL, 1);
	if (currPattern) {
	  CPattern* tempPattern = new CPattern(currPattern->Copy_kTree());
	  new_s->SetCurrentPattern(tempPattern);
	  new_s = new_s->CurrentPattern()->buildFromHere(new_s, new_s->IEnv()->getData());
	  delete tempPattern;
	}
      }
      else {
	findNodeFromNumHelper(tempVal, currLoc)->replaceChild(new_s->Store(), currWhich);
        new_s->SetStore(tempVal->Copy_kTree());
      }
    }
    
  }
  //finally return the Store() of the root State. This will hold the current
  //root KOLA query
  return new_s->Store();
  //DELETE new_s
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: setTempStmt
// Parameters:	(CStmt* stmt)
// Returns: void
//
// This method sets member variable tempStmt to CStmt* parameter stmt. This is
// done because unfortunately, updateDEP was written so that it is a member
// method of the CStmt class, and sometimes the debugger needs to call this
// method and doesn't have an instance of a CStmt around to call it on. Since
// updateDEP doesn't actually affect the stmt that it was called on, it
// doesn't matter what statement is sent to this method - as long as it will
// always be non-NUll while the debugger is running. Really, updateDEP should
// have been declared static when it was originally written.
//         								    
//////////////////////////////////////////////////////////////////////////////
 
void DDebugController::setTempStmt(CStmt* stmt)
{
  if (!tempStmt)
    tempStmt = stmt;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: resetVariables()
// Parameters:	()
// Returns: void
//
// This method is called whenever the debugger is restarted. Right now, all
// it does is set tempStmt back to NULL, but my guess is that certain other
// variables must be reset as well, and that is why the program crashes
// sometimes when the user restarts debugger execution (by pressing the run
// button). This method should be revisited.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DDebugController::resetVariables() {
  tempStmt = NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getCurrLCArray()
// Parameters:	()
// Returns: LCSubstring*
//
// This method returns currLCArray (the variable that contains the location
// of each statement in the text area that is displaying the current
// transformation).
//         								    
//////////////////////////////////////////////////////////////////////////////

LCSubstring* DDebugController::getCurrLCArray()
{
  return currLCArray;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getNumStmts()
// Parameters:	()
// Returns: int
//
// This method returns the number of breakable statements in the current
// transformation.
//         								    
//////////////////////////////////////////////////////////////////////////////

int DDebugController::getNumStmts()
{
  return currNumStmts;
}
