//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Daniel Abadi 
//  FILE: HistoryList.C
//  PROJ: COKO DEBUGGER
//  DATE: SUMMER 2001                                                      
//
// SYNOPSIS
//  Classes for History Stack.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef _HISTORYLIST_C_
#define _HISTORYLIST_C_

#include "DEHistoryList.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: HistoryNode
// Parameters:	(NonType*, int, int, char*, CIEnvStack*, NonType*)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

HistoryNode::HistoryNode(NonType* value, int location, int which, char* v_name, CIEnvStack* eStack, NonType* pattern):	
  _next(NULL),
  _environmentStack(NULL),
  _variableName(NULL),
  _pattern(NULL),
  _value(NULL)
{
  _location = location;
  _which = which;
  _environmentStack = eStack;
  if (v_name) {
    _variableName = new char[strlen(v_name)+1];
    strcpy(_variableName, v_name);
  }
  if (pattern)
    _pattern = pattern->Copy_kTree();
  _value = value->Copy_kTree();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~HistoryNode
// Parameters:	(void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////
  
HistoryNode::~HistoryNode(void) 
{
  if (_value) { delete _value; _value = NULL; }
  _next = NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: display
// Parameters:	(void)
//
// For debugging purposes only
//         								    
//////////////////////////////////////////////////////////////////////////////
  
void HistoryNode::display(void) 
{
  cout << "Location: " << _location << " Which: " << _which << " ";
  //if (_value)
  //  cout <<  _value  << endl;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: push
// Parameters: (NonType*, int, int, char*, CIEnvStack*)	
//
// Push a new node with _value = a and _location = num to the list.
//         								    
//////////////////////////////////////////////////////////////////////////////

void HistoryList::push(NonType* a, int num, int which, char* v_name, CIEnvStack* eStack, NonType* pattern) 
{
  if (!_head)
	_head = _tail = new HistoryNode(a, num, which, v_name, eStack, pattern); 
  else 
	{
	  HistoryNode * tmp = new HistoryNode(a, num, which, v_name, eStack, pattern);
	  tmp->_next = _head;
	  _head = tmp;
	  tmp = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: pop
// Parameters: (void)
//
// Pop off a front node from the list.
// Return success value of the pop.
//         								    
//////////////////////////////////////////////////////////////////////////////

int HistoryList::pop(void)
{
  if (!_head)
	return 0;

  HistoryNode * tmpnode = _head;

  if (_head == _tail)
	_head = _tail = NULL;
  else
	_head = _head->_next;

  if (tmpnode->_value) 
	{ delete tmpnode; tmpnode = NULL; }
  else
	{
	  cout << "Error @ HistoryList : pop " << endl;
	  cout << "node has no content error " << endl;
	  exit(1);
	  return 0;
	}

  return 1;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: topVal
// Parameters: (void)
//
// Return _value of the top node.
//         								    
//////////////////////////////////////////////////////////////////////////////

NonType* HistoryList::topVal(void)
{
  if (!_head)
	return NULL;

  if (_head->_value) 
	return _head->_value;
  else
	{
	  cout << "Error @ HistoryList : topVal " << endl;
	  cout << "node has no content error " << endl;
	  exit(1);
	  return NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: topWhich
// Parameters: (void)
//
// Return _which of the top node.
//         								    
//////////////////////////////////////////////////////////////////////////////

int HistoryList::topWhich(void)
{
  if (!_head)
    return 0;

  else
    return _head->_which;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: topLocation
// Parameters: (void)
//
// Return _location of the top node.
//         								    
//////////////////////////////////////////////////////////////////////////////

int HistoryList::topLocation(void)
{
  if (!_head)
    return 0;

  else
    return _head->_location;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: empty
// Parameters: (void)
//
// Is the list empty?
//         								    
//////////////////////////////////////////////////////////////////////////////

int HistoryList::empty(void) 
{ 
  return (!_head); 
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: initIterator 
// Parameters: (void)
//
// Initialize _iterator.
// This method is used with getValue method.
//         								    
//////////////////////////////////////////////////////////////////////////////

void HistoryList::initIterator(void)
{
  _iterator = _head;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorValue
// Parameters:	(void)
//
// Returns _value of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

NonType* HistoryList::iteratorValue(void)
{
  if (_iterator)
	return _iterator->_value;
  else
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorLocation
// Parameters:	(void)
//
// Returns _location of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

int HistoryList::iteratorLocation(void)
{
  if (_iterator)
	return _iterator->_location;
  else
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorWhich
// Parameters:	(void)
//
// Returns _which of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

int HistoryList::iteratorWhich(void)
{
  if (_iterator)
	return _iterator->_which;
  else
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorStack
// Parameters:	(void)
//
// Returns _environmentStack of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

CIEnvStack* HistoryList::iteratorStack(void)
{
  if (_iterator)
	return _iterator->_environmentStack;
  else
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorName
// Parameters:	(void)
//
// Returns _variableName of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

char* HistoryList::iteratorName(void)
{
  if (_iterator)
	return _iterator->_variableName;
  else
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorPattern
// Parameters:	(void)
//
// Returns _pattern of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

NonType* HistoryList::iteratorPattern(void)
{
  if (_iterator)
	return _iterator->_pattern;
  else
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getNextValue
// Parameters:	(void)
//
// Point _iterator to the next node and returns the _value of that node.
// One can call this methods repeatly to get the _value of all the nodes.
// However, initIterator has to be called before the initial call to this 
// method.  
//         								    
//////////////////////////////////////////////////////////////////////////////

NonType* HistoryList::getNextValue()
{
  NonType* return_val;
  if ((_iterator) && (_iterator->_next))
	{
	   _iterator = _iterator->_next;
	  return_val = _iterator->_value;
	}
  else
	return_val = NULL;
  return return_val;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: HistoryList
// Parameters: (void)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

HistoryList::HistoryList():
  _head(NULL),
  _tail(NULL),
  _iterator(NULL)
{
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:~HistoryList
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

HistoryList::~HistoryList()
{
  HistoryNode * tmp = _head;
  _tail = NULL;
  while (_head)
   	{
 	  _head = _head->_next;
 	  delete tmp;
 	  tmp = _head;
 	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: display
// Parameters:	(void)
//
// For debugging purposes only. Displays the contents of the entire list.
//         								    
//////////////////////////////////////////////////////////////////////////////
void HistoryList::display()
{
  cout << "DISPLAYING LIST" << endl;
  initIterator();
  if (_iterator)
    _iterator->display();
  
  while(getNextValue())
      _iterator->display();
}

#endif

