#include "DEPaintWindow.moc"

DPaintWindow::DPaintWindow(QPopupMenu* view = 0, int index = 0, MyMainWindow* win, DScrollWidget* grandpa=0, const char *name=0):QWidget(grandpa, name)
{
  _name = name;
  _grandpaWid = grandpa;
  int x_width = size().width();
  int y_height = size().height();
  if (_grandpaWid) {
    x_width = _grandpaWid->size().width();
    y_height = _grandpaWid->size().height();
    setGeometry(5, 15,x_width, y_height);
  }
  //setAlignment(AlignHCenter | AlignVCenter);
  paint = new QPainter;
  treeDesc = new int[100];
  for (int i = 0; i < 100; i++)
    treeDesc[i] = 0;
  setBackgroundColor(Qt::white);
  _view = view;
  _index = index;
  nodes = NULL;
  numNodes = 0;
  vis = win;
  _zoomFactor = 1;
}

const char* DPaintWindow::getName()
{
  return _name;
}

void DPaintWindow::setCurrent(NonType* curr)
{
  current = curr;
}

void DPaintWindow::setImportantValues(SymbolList* sym, char* name)
{
  _sym = sym;
  execName = name;
}

void DPaintWindow::generateTreeDesc(NonType* curr, int level)
{
  NonType * child;
  if (!curr)
    return;
  curr->initCurrChild();
  while (!((child=curr->nextChild()) == NULL))
    {
      treeDesc[level]++;
      generateTreeDesc(child, level+1);
    }
}

int DPaintWindow::findNumNodes(NonType* curr)
{
  NonType * child;
  if (!curr)
    return 0;
  curr->initCurrChild();
  int num_children = 0;
  int num_so_far = 0;
  while (!((child=curr->nextChild()) == NULL))
    {
      num_children++;
      num_so_far+=findNumNodes(child);
    }
  if (num_children == 0)
    return 1;
  else
    return num_so_far+1;
}

int DPaintWindow::findNumExternalNodes(NonType* curr)
{
  NonType * child;
  if (!curr)
    return 0;
  curr->initCurrChild();
  int num_children = 0;
  int num_so_far = 0;
  while (!((child=curr->nextChild()) == NULL))
    {
      num_children++;
      num_so_far+=findNumExternalNodes(child);
    }
  if (num_children == 0) {
    int count = 1;
    int varNum = curr->getVariableNum();
    if (varNum < 0) 
      count++;
    if ((varNum < -2) || (varNum > 2)) 
      count++;
    return count;
  }
  else
    return num_so_far;
}

char* DPaintWindow::shortenString(char* inputStr)
{
  char* tempStr;
  int count = 0;
  while ((count < strlen(inputStr)) && (inputStr[count] != '('))
    count ++;
  if (count != strlen (inputStr)) {
    tempStr = new char[count+1];
    int i = 0;
    while (i <= count-1) {
      tempStr[i] = inputStr[i];
      i++;
    }
    tempStr[count] = '\0';
    return tempStr;
  }
  else if (strlen(inputStr) > 7) {
    char* tempStr = new char[10];
    tempStr[0] = inputStr[0];
    tempStr[1] = inputStr[1];
    tempStr[2] = inputStr[2];
    tempStr[3] = inputStr[3];
    tempStr[4] = inputStr[4];
    tempStr[5] = inputStr[5];
    tempStr[6] = '.';
    tempStr[7] = '.';
    tempStr[8] = '.';
    tempStr[9] = '\0';
    return tempStr;
  }
  else
    return inputStr;
}

int DPaintWindow::drawNode(NonType* curr, int y, int radius)
{
  NonType * child;
  bool drawnLeft = false;
  int middle;
  char* textToWrite;
  int w1; //The width of drawth text
  int minW = paint->fontMetrics().width("XX");
  int h1 = paint->fontMetrics().height(); //The height of drawn text
        
  curr->initCurrChild();
  int* x_loc = new int[3];
  for (int i = 0; i < 3; i++)
    x_loc[i] = 0;
  int num_children = 0;
  int temp = int(radius / 1.41);
  int temp_return_val;
  while (!((child=curr->nextChild()) == NULL))
    {
      temp_return_val = drawNode(child, y + radius*4, radius);
      x_loc[num_children] = temp_return_val;
      num_children++;
    }
  int currNum = curr->getVariableNum();
  int scaledCurr = currNum;
  if (currNum % 2 != 0)
    scaledCurr--;
  if (scaledCurr < 0)
    scaledCurr = scaledCurr * -1;
  if (num_children == 0) {
     if ((currNum < 0) || (currNum > 2)) 
       middle = curr_x + x_inc;
     else
       middle = curr_x;
     curr_x += x_inc;
     if ((currNum < -2) || (currNum > 2)) 
       curr_x += x_inc;
     if (currNum < 0)
       curr_x += x_inc;
  }
  else {
    int sum = 0;
    for (int i = 0; i < 3; i++)
      sum += x_loc[i];
    middle = sum / num_children;
  }
  if ((currNum < -2) || (currNum > 2)) {
    textToWrite = _sym->findCorrespondingSym(scaledCurr);
    textToWrite = shortenString(textToWrite);
    w1 = paint->fontMetrics().width(textToWrite);
    if (w1 < minW)
      w1 = minW;
    paint->drawText(middle-radius-w1,y-(3*h1/4),w1,h1,Qt::AlignCenter,textToWrite);
    paint->drawLine(middle-radius-w1,y+(h1/2),middle-radius,y+(h1/2));
    paint->drawLine(middle-radius-(w1/3),y+(h1/4),middle-radius,y+(h1/2));
    paint->drawLine(middle-radius-(w1/3),y+(3*h1/4),middle-radius,y+(h1/2));
    if (strlen(textToWrite) > 7)
      {
	delete[] textToWrite;
      }
    drawnLeft = true;
  }
  if (execName) {
    if (curr->getVariableNum() < 0) {
      textToWrite = shortenString(execName);
      w1 = paint->fontMetrics().width(textToWrite);
      if (w1 < minW)
	w1 = minW;
      if (drawnLeft) {
	paint->drawText(middle+radius,y-(3*h1/4),w1,h1,Qt::AlignCenter,textToWrite);
	paint->drawLine(middle+radius+w1,y+(h1/2),middle+radius,y+(h1/2));
	paint->drawLine(middle+radius+(w1/3),y+(h1/4),middle+radius,y+(h1/2));
	paint->drawLine(middle+radius+(w1/3),y+(3*h1/4),middle+radius,y+(h1/2));
      }
      else {
	paint->drawText(middle-radius-w1,y-(3*h1/4),w1,h1,Qt::AlignCenter,textToWrite);
	paint->drawLine(middle-radius-w1,y+(h1/2),middle-radius,y+(h1/2));
	paint->drawLine(middle-radius-(w1/3),y+(h1/4),middle-radius,y+(h1/2));
	paint->drawLine(middle-radius-(w1/3),y+(3*h1/4),middle-radius,y+(h1/2));
      }
      if (strlen(textToWrite) > 7)
	{
	  delete[] textToWrite;
	}
      
    }
  }
  if (currNum % 2 != 0)
    paint->setPen(darkGreen);
  paint->drawEllipse(middle-radius,y-radius,radius*2, radius*2);
  nodes[nodeCounter] = new DVisNodeInfo(middle-radius, y-radius, middle+radius, y+radius);
  nodeCounter++;
  w1 = paint->fontMetrics().width(curr->Name());
  int origSize = paint->font().pointSize();
  int currSize = origSize;
  QFont oldFont;
  while ((w1 > 2*radius) && (currSize > 2)) {
    oldFont = paint->font();
    oldFont.setPointSize(currSize--);
    paint->setFont(oldFont);
    w1 = paint->fontMetrics().width(curr->Name());
    h1 = paint->fontMetrics().height();
  }
  paint->drawText(middle-(w1/2),y-(h1/2),w1,h1,Qt::AlignCenter,curr->Name());
  paint->setPen(black);
  oldFont = paint->font();
  oldFont.setPointSize(origSize);
  paint->setFont(oldFont);
  h1 = paint->fontMetrics().height();

  if (num_children == 1) {
    paint->drawLine(middle, y+radius, x_loc[0], y + 3*radius);
  }
  else if (num_children == 2) {
    paint->drawLine(middle-temp, y+temp, x_loc[0], y + 3*radius);
    paint->drawLine(middle+temp, y+temp, x_loc[1], y + 3*radius);
  }
  else if (num_children == 3) {
    paint->drawLine(middle-temp, y+temp, x_loc[0], y + 3*radius);
    paint->drawLine(middle, y+radius, x_loc[1], y + 3*radius);
    paint->drawLine(middle+temp, y+temp, x_loc[2], y + 3*radius);
  }
  return middle;
}

int DPaintWindow::twoToThe(int exp)
{
  if (exp < 1)
    return 1;
  else
    return 2 * twoToThe(exp-1);
}

void DPaintWindow::paintEvent(QPaintEvent*)
{
  if (nodes) {
    for (int i = 0; i < numNodes; i++)
      if (nodes[i]) {
	delete nodes[i];
	nodes[i] = NULL;
      }
    delete[] nodes;
    nodes = NULL;
  }
  NonType * curr = current;
  NonType * child;
  int x_width = size().width();
  int y_height = size().height();
  //if (_parent)
  //  x_width = _parent->size().width()-10;
  //  y_height = _parent->size().height()-20;
  //  setGeometry(5, 15, x_width, y_height);
  //}
  //int x_width = size().width();
  //int y_height = size().height();
  //cout << x_width << " " << y_height << endl;
  int x = 30;
  paint->begin(this);
  treeDesc[0] = 1;
  generateTreeDesc(curr, 1);
  int depth = 0;
  char* temp;
  while (!(treeDesc[depth] == 0))
    depth++;
  
  int numExNodes = findNumExternalNodes(curr);
  if (numExNodes == 0) {
    paint->drawText(50,50,"Nothing To Display");
  }
  else {
    numNodes = findNumNodes(curr);
    nodes = new DVisNodeInfo* [numNodes];
    x_inc = (x_width-30) / numExNodes;
    curr_x = x_inc / 2 + 20;
    int rad1 = (((y_height-50) / depth) / 4);
    int rad2 = ((2*x_inc) / 5);
    int rad3;
    if (rad1 < rad2)
      rad3 = rad1;
    else
      rad3 = rad2;
    nodeCounter = 0;
    int w1 = paint->fontMetrics().width(_name);
    int h1 = paint->fontMetrics().height();
    paint->drawText(x_width/2-w1/2,5,w1,h1,Qt::AlignCenter,_name);
    drawNode(curr, 50+2*rad3, rad3);
  }
  paint->end();
  for (int i = 0; i <= depth; i++)
    treeDesc[i] = 0;
  
}

bool DPaintWindow::close( bool alsoDelete ) {
  if (_view->isItemChecked(_index)) 
    _view->setItemChecked(_index, false);
  return QWidget::close(alsoDelete);
}

int DPaintWindow::findCondtainingIndex(int x, int y) {
  if (!nodes)
    return -1;
  for (int i = 0; i < numNodes; i++)
    if ((nodes[i]->_left <=x) && (nodes[i]->_right >=x) && (nodes[i]->_top <=y) && (nodes[i]->_bottom >=y))
      return i;
  return -1;
}

void DPaintWindow::mousePressEvent(QMouseEvent * e)
{
  if ( e->button() == RightButton ) {
    mouse_x = e->pos().x();
    mouse_y = e->pos().y();
    QMouseEvent e_new(QEvent::MouseButtonPress, e->pos(),LeftButton,LeftButton);
    int index = findCondtainingIndex(mouse_x,mouse_y);
    

    QPopupMenu *popup = new QPopupMenu( this );
    int id[3];
    id[0] = popup->insertItem("ZOOM IN", this, SLOT(zoom_in()));
    id[1] = popup->insertItem("ZOOM OUT", this, SLOT(zoom_out()));
    id[2] = popup->insertItem("ISOLATE", this, SLOT(isolate()));
    if (index != -1) {
      indexToIsolate = index;
      popup->setItemEnabled(id[2],true);
    }
    else
      popup->setItemEnabled(id[2],false);
    if (_zoomFactor <= 1)
      popup->setItemEnabled(id[1],false);
    else
      popup->setItemEnabled(id[1],true);
    popup->setItemEnabled(id[0],true);
      
    int r = popup->exec( e->globalPos() );
  }
  
}

NonType* DPaintWindow::findNodeFromIndex(int index, NonType* curr, DWhichList* wl) {
  NonType* tempNT;
  NonType * child;
  if (!curr) 
    return NULL;
  curr->initCurrChild();
  int numChild = 1;
  while (!((child=curr->nextChild()) == NULL))
    {
      tempNT = findNodeFromIndex(index, child, wl);
      if (tempNT) {
	wl->push(numChild);
	return tempNT;
      }
      numChild++;    }
  
  if (globalCounter == index) {
    return curr;
  }
  else {
    globalCounter++;
    return NULL;
  }
}

void DPaintWindow::updateWhichList(DWhichList* curr, NonType* root, int index) {
  globalCounter = 0;
  NonType* node = findNodeFromIndex(index, root, curr);
}

void DPaintWindow::isolate() {
  DWhichList* temp = new DWhichList();
  updateWhichList(temp,current,indexToIsolate);
  DScrollWidget* tempGB = new DScrollWidget(NULL, 0, NULL, "ISOLATION", NULL);
  //$$//tempGB->setTitle("BRANCH ISOLATION");
  DPaintWindow* tempPW = new DPaintWindow(NULL, 0, vis, tempGB, "BRANCH ISOLATION");
  //$$//DScrollWidget* tempScroll = new DScrollWidget(tempGB, NULL, tempPW);
  //scaleVariableNums(myNode->Child_Look_Up());
  tempPW->setCurrent(vis->doIsolate(current,temp));
  tempGB->setSon(tempPW);
  //$$//tempGB->setGrandson(tempPW);
  int tempIndex = vis->insertIntoView("ISOLATION");
  tempGB->setViewVals(vis->getView(), tempIndex);
  tempPW->setImportantValues(_sym, execName);
  vis->insertIntoSVL(NULL,temp,0,tempGB,tempPW,tempIndex);
  tempPW->repaint(true);
  tempGB->show();
}

void DPaintWindow::zoom_in() {
  float ratio_x = float(mouse_x)/float(size().width());
  float ratio_y = float(mouse_y)/float(size().height());
  _zoomFactor = _zoomFactor*5/4;
  _grandpaWid->redoGeometryCalculations();
  _grandpaWid->moveScrollBars(ratio_x, ratio_y);
}

void DPaintWindow::zoom_out() {
  float ratio_x = float(mouse_x)/float(size().width());
  float ratio_y = float(mouse_y)/float(size().height());
  _zoomFactor = _zoomFactor*4/5;
  if (_zoomFactor < 1)
    _zoomFactor = 1;
  _grandpaWid->redoGeometryCalculations();
  _grandpaWid->moveScrollBars(ratio_x, ratio_y);
}

float DPaintWindow::getZoomFactor() {
  return _zoomFactor;
}
