//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Daniel Abadi
//  FILE: DESymbolVisList.H
//  PROJ: COKO DEBBUGGER
//  DATE: WINTER 2002                                                    
//
// SYNOPSIS
//  Classes for Symbol Visual Display List.
//  
//////////////////////////////////////////////////////////////////////////////


#ifndef _DSYMBOLVISLIST_H_
#define _DSYMBOLVISLIST_H_

#include <stdlib.h>
#include <string.h>
#include "DEWhichList.H"
#include "DEPaintWindow.H"
#include "DEScrollWidget.H"
#include <iostream.h>

class DPaintWindow;
class DScrollWidget;

class DSymbolVisNode
{
public:
  char * _value;
  DWhichList* _wl;
  int _num;
  int _viewIndex;
  DScrollWidget* _gb;
  DPaintWindow* _pw;
  DSymbolVisNode * _next;

  DSymbolVisNode(const char * c, DWhichList* wl, int num, DScrollWidget* gb, DPaintWindow* pw, int index);
  // Constructor

  ~DSymbolVisNode();
  // Destructor

  void display();
  //for debugging purposes only

};

class DSymbolVisList
{
private:
  DSymbolVisNode * _iterator;
  DSymbolVisNode * _head;
  DSymbolVisNode * _tail;

public:

  DSymbolVisList();
  // Constructor
  
  ~DSymbolVisList();
  // Destructor

  void insert(const char* a, DWhichList* wl, int num, DScrollWidget* gb, DPaintWindow* pw, int index);
  //  Insert a new node to the list

  int pop();
  // Pop off a front node from the list
  // Return success value of the pop.

  char* top();
  // Returns _value of the top node.

  int empty();
  // Is the list empty?

  void initIterator();
  // Initialize _iterator
  // This method is used with getValue method

  char* iteratorValue();
  // Returns _value of the node that _iterator is pointing to.

  char* iteratorNextValue();
  // Returns _value of the next node that _iterator is pointing to.

  int iteratorNum();
  // Returns _num of the node that _iterator is pointing to.

  int iteratorNextNum();
  // Returns _num of the next node that _iterator is pointing to.

  void setIteratorNum(int x);
  // Set _num of the node that _iterator is pointing to to x.

  void setIteratorNextNum(int x);
  // Set _num of the next node that _iterator is pointing to x.

  DScrollWidget* iteratorGB();
  // Returns _gb of the node that _iterator is pointing to.

  DPaintWindow* iteratorPW();
  // Returns _pw of the node that _iterator is pointing to.

  int iteratorViewIndex();
  // Returns _viewIndex of the node that _iterator is pointing to.

  int iteratorNextViewIndex();
  // Returns _viewIndex of the node that _iterator is pointing to.
  
  DWhichList* iteratorWhichList();
  // Returns _wl of the node that _iterator is pointing to.

  void moveIterator();
  // make _iterator point to the next value

  int deleteNext();
  // Deletes the next node that the _iterator is pointing to.

  int find(const char*);
  // Points _iterator to the previous node from the node that has the given
  // value, and returns 1 if that value was found.
  // If the node that has the given value is the _head of the list, the it
  // points _iterator to the head and returns 2.
  // If the given value is not in the list, _iterator points to NULL and
  // 0 is returned.

  char* getNextValue();
  // Point _iterator to the next node and returns the _value of the next node
  // after that.
  // One can call this methods repeatly to get the _value of all the nodes.
  // However, initIterator has to be called before the initial call to this 
  // method.  

  void display();
  //for debugging purposes only

  void delete_marked_nodes();
  //delete all nodes with _num of -1 in list

  void delete_all_nodes();
  //delete all nodes in list

  char* findCorrespondingSym(int);
  // Returns the symbol with corresponding to the integer input x
  // If the list was maintained correctly, there should be only one such symbol
  // Otherwise, the first such symbol is returned
  

};

#endif

