//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Daniel Abadi 
//  FILE: DETransformationList.C
//  PROJ: COKO DEBUGGER
//  DATE: WINTER 2002                                             
//
// SYNOPSIS
//  Classes for Symbol Visual DIsplay List.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef _DTRANSFORMATIONLIST_C_
#define _DTRANSFORMATIONLIST_C_

#include "DETransformationList.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: DTransformationNode
// Parameters:	(RuleBlockNode*, MyMultiLineEdit *, char*, int)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

DTransformationNode::DTransformationNode(RuleBlockNode* rbn, MyMultiLineEdit *widget, char* contents, int viewIndex):	
  _next(NULL),
  _contents(NULL),
  _widget(NULL),
  _value(NULL)
{
  _value = rbn;
  _contents = contents;
  _widget = widget;
  _viewIndex = viewIndex;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~DTransformationNode
// Parameters:	(void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////
  
DTransformationNode::~DTransformationNode(void) 
{
  _next = NULL;
  if (_contents) { delete[] _contents; _contents = NULL;}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: display
// Parameters:	(void)
//
// For debugging purposes only
//         								    
//////////////////////////////////////////////////////////////////////////////
  
void DTransformationNode::display(void) 
{
  if (_value)
    _value->display();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: insert
// Parameters: (RuleBlockNode*, MyMultiLineEdit*, char*, int)    
//
// Insert a new node with _value = a and _num = num to the list.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DTransformationList::insert(RuleBlockNode* rbn, MyMultiLineEdit *widget, char* contents, int viewIndex) 
{
  if (!_head)
	_head = _tail = new DTransformationNode(rbn, widget, contents, viewIndex); 
  else 
    {
      DTransformationNode * tmp = new DTransformationNode(rbn, widget, contents, viewIndex);
      _tail->_next = tmp;
      _tail = tmp;
      tmp = NULL;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: pop
// Parameters: (void)
//
// Pop off a front node from the list.
// Return success value of the pop.
//         								    
//////////////////////////////////////////////////////////////////////////////

int DTransformationList::pop(void)
{
  if (!_head)
	return 0;

  DTransformationNode * tmpnode = _head;

  if (_head == _tail)
	_head = _tail = NULL;
  else
	_head = _head->_next;

  delete tmpnode; tmpnode = NULL;
  
  return 1;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: top
// Parameters: (void)
//
// Return _value of the top node.
//         								    
//////////////////////////////////////////////////////////////////////////////

RuleBlockNode* DTransformationList::top(void)
{
  if (!_head)
	return NULL;

  if (_head->_value) 
	return _head->_value;
  else
	{
	  cout << "Error @ DTransformationList : top " << endl;
	  cout << "node has no content error " << endl;
	  exit(1);
	  return NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: empty
// Parameters: (void)
//
// Is the list empty?
//         								    
//////////////////////////////////////////////////////////////////////////////

int DTransformationList::empty(void) 
{ 
  return (!_head); 
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: initIterator 
// Parameters: (void)
//
// Initialize _iterator.
// This method is used with getValue method.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DTransformationList::initIterator(void)
{
  _iterator = _head;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorValue
// Parameters:	(void)
//
// Returns _value of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

RuleBlockNode* DTransformationList::iteratorValue(void)
{
  if (_iterator)
	return _iterator->_value;
  else
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorWindow
// Parameters:	(void)
//
// Returns _widget of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

MyMultiLineEdit* DTransformationList::iteratorWindow(void)
{
  if (_iterator)
	return _iterator->_widget;
  else
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorNextValue
// Parameters:	(void)
//
// Returns _value of the next node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

RuleBlockNode* DTransformationList::iteratorNextValue(void)
{
  if ((_iterator) && (_iterator->_next))
	return _iterator->_next->_value;
  else
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorNextWindow
// Parameters:	(void)
//
// Returns _widget of the next node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

MyMultiLineEdit* DTransformationList::iteratorNextWindow(void)
{
  if ((_iterator) && (_iterator->_next))
	return _iterator->_next->_widget;
  else
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorViewIndex
// Parameters:	(void)
//
// Returns _viewIndex of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

int DTransformationList::iteratorViewIndex(void)
{
  if (_iterator)
	return _iterator->_viewIndex;
  else
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorNextViewIndex
// Parameters:	(void)
//
// Returns _viewIndex of the next node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

int DTransformationList::iteratorNextViewIndex(void)
{
  if ((_iterator) && (_iterator->_next))
	return _iterator->_next->_viewIndex;
  else
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: deleteNext
// Parameters:	(void)
//
// Deletes the next node that the _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

int DTransformationList::deleteNext(void)
{
  if ((_iterator) && (_iterator->_next))
    {
      DTransformationNode * tmpnode = _iterator->_next;
      _iterator->_next=tmpnode->_next;
      if (!(tmpnode->_next))
	_tail = _iterator;
      delete tmpnode; tmpnode = NULL; 
    }
  else
    return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: moveIterator
// Parameters:	(void)
//
// make _iterator point to the next value
//         								    
//////////////////////////////////////////////////////////////////////////////

void DTransformationList::moveIterator()
{
  if (_iterator)
    _iterator = _iterator->_next;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getNextValue
// Parameters:	(void)
//
// Point _iterator to the next node and returns the _value of the next node
// after that.
// One can call this methods repeatly to get the _value of all the nodes.
// However, initIterator has to be called before the initial call to this 
// method.  
//         								    
//////////////////////////////////////////////////////////////////////////////

RuleBlockNode* DTransformationList::getNextValue()
{
  RuleBlockNode* return_val;
  if ((_iterator) && (_iterator->_next) && (_iterator->_next->_next))
	{
	   _iterator = _iterator->_next;
	  return_val = _iterator->_next->_value;
	}
  else
	return_val = NULL;
  return return_val;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: DTransformationList
// Parameters: (void)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

DTransformationList::DTransformationList():
  _head(NULL),
  _tail(NULL),
  _iterator(NULL)
{
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:~DTransformationList
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

DTransformationList::~DTransformationList()
{
  DTransformationNode * tmp = _head;
  _tail = NULL;
  while (_head)
   	{
 	  _head = _head->_next;
 	  delete tmp;
 	  tmp = _head;
 	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: display
// Parameters:	(void)
//
// For debugging purposes only. Displays the contents of the entire list.
//         								    
//////////////////////////////////////////////////////////////////////////////
void DTransformationList::display()
{
  cout << "DISPLAYING LIST" << endl;
  initIterator();
  if (_iterator)
    _iterator->display();
  //if (_iterator->_next)
  //  _iterator->_next->display();
  
  for(;;)
    if (getNextValue())
      _iterator->display();
    else
      break;
  
  if ((_iterator) && (_iterator->_next))
    _iterator->_next->display();
} 

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: find
// Parameters:	(const char*)
//
// Points _iterator to the previous node from the node that has the given
// value, and returns 1 if that value was found.
// If the node that has the given value is the _head of the list, the it
// points _iterator to the head and returns 2.
// If the given value is not in the list, _iterator points to NULL and
// 0 is returned.
//         								    
//////////////////////////////////////////////////////////////////////////////

int DTransformationList::find(const char* name)
{
  initIterator();
  RuleBlockNode* temp;
  if (_iterator)
    if (strcmp(name,iteratorValue()->_name) == 0)
      return 2;
    else {
      if (_iterator->_next) {
	if (strcmp(name,iteratorNextValue()->_name) == 0)
	  return 1;
	else {
	  while (temp = getNextValue())
	    if (strcmp(name,temp->_name) == 0)
	      return 1;
	  return 0;
	}
      }
      else 
	return 0;
    }
  else
    return 0;
}

#endif

