//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon Suk Lee (jl) 
//  FILE: DEPList.C
//  PROJ: COKO
//  DATE: 10/96                                                              
//
// SYNOPSIS
//  Classes for Dependancy List.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef _DEPLIST_C_
#define _DEPLIST_C_

#include "DEPList.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: DEPNode
// Parameters:	(char*)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

DEPNode::DEPNode(char * a):	
  _next(NULL),
  _value(NULL)
{
  if (a)
  	{
  	  _value = new char[strlen(a)+1];
  	  strcpy(_value, a);
  	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~DEPNode
// Parameters:	(void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////
  
DEPNode::~DEPNode(void) 
{
  if (_value) { delete [] _value; _value = NULL; }
  _next = NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: find
// Parameters:	(const char*)
//
// find a node with _value = a.
//         								    
//////////////////////////////////////////////////////////////////////////////

DEPNode * DEPList::find(const char* a) 
{
  DEPNode * iterator = _head;
  while (iterator)
	{
	  if (strcmp(iterator->_value, a) == 0)
		return iterator;
	  else
		iterator = iterator->_next;
	}
  return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: insert
// Parameters: (char*)	
//
// Insert a new node with _value = a to the list.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DEPList::insert(char* a) 
{
  if (!_head)
	_head = new DEPNode(a); 
  else 
	{
	  DEPNode * tmp = new DEPNode(a);
	  tmp->_next = _head;
	  _head = tmp;
	  tmp = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: pop
// Parameters: (void)
//
// Pop off a front node from the list.
// Return a copy of _value.
// Must make a copy of _value, since the poped node should be deleted.
//         								    
//////////////////////////////////////////////////////////////////////////////

char* DEPList::pop(void)
{
  if (!_head)
	return NULL;

  DEPNode * tmpnode = _head;

  _head = _head->_next;

  if (tmpnode->_value) 
	{
	  char * return_char = new char[strlen(tmpnode->_value) + 1];
	  strcpy(return_char, tmpnode->_value);
	  delete tmpnode; tmpnode = NULL;
	  return return_char;
	}
  else
	{
	  cout << "Error @ DEPList : pop " << endl;
	  cout << "node has no content error " << endl;
	  exit(1);
	  return NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: empty
// Parameters: (void)
//
// Is the list empty?
//         								    
//////////////////////////////////////////////////////////////////////////////

int DEPList::empty(void) 
{ 
  return (!_head); 
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: initIterator 
// Parameters: (void)
//
// Initialize _iterator.
// This method is used with getValue method.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DEPList::initIterator(void)
{
  _iterator = _head;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getValue
// Parameters:	(void)
//
// Returns _value of the node that _iterator is pointing to.
// After a call to this method, _iterator will point to the next node.
// One can call this methods repeatly to get the _value of all the nodes.
// However, initIterator has to be called before the initial call to this 
// method.  
//         								    
//////////////////////////////////////////////////////////////////////////////

char* DEPList::getValue()
{
  char* return_char;
  if (_iterator)
	{
	  return_char = _iterator->_value;
	  _iterator = _iterator->_next;
	}
  else
	return_char = NULL;
  return return_char;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: DEPList
// Parameters: (void)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

DEPList::DEPList():
  _head(NULL),
  _iterator(NULL)
{
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:~DEPList
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

DEPList::~DEPList()
{
    DEPNode * tmp = _head;
    while (_head)
  	{
	  _head = _head->_next;
	  delete tmp;
	  tmp = _head;
	}
}

#endif

