//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon S. Lee (jl), Kee-Eung Kim (kek)
//  FILE: PCIEnvStack.C
//  PROJ: COKO
//  DATE: 5/5/97                                                         
//
// SYNOPSIS
//  Method definitions for CIEnvStack class.
//  
//////////////////////////////////////////////////////////////////////////////

#include "PCIEnvStack.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CIEnvStackDataType
// Parameters:	(CIEnv*)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CIEnvStackDataType::CIEnvStackDataType(CIEnv * item)
{
  _item = item;
  _next = NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~CIEnvStackDataType
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CIEnvStackDataType::~CIEnvStackDataType(void)
{
  if (_item) { delete _item; _item = NULL; }
  if (_next) { delete _next; _next = NULL; }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CIEnvStack
// Parameters: (void)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CIEnvStack::CIEnvStack(void)
{
  _data = NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~CIEnvStack
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CIEnvStack::~CIEnvStack(void)
{
  if (_data) { delete _data; _data = NULL; }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getData
// Parameters:	(void) 
//
// Returns _data.
//         								    
//////////////////////////////////////////////////////////////////////////////

CIEnvStackDataType* CIEnvStack::getData(void) 
{
  return _data; 
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: push
// Parameters:	(CIEnv*)
//
// The "Push" operation on the stack.
//         								    
//////////////////////////////////////////////////////////////////////////////

void CIEnvStack::push(CIEnv * item)
{
  CIEnvStackDataType * temp = new CIEnvStackDataType(item);
  temp->_next = _data;
  _data = temp;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: pop
// Parameters: (void)
//
// The "Pop" operation on the stack.
//         								    
//////////////////////////////////////////////////////////////////////////////

void CIEnvStack::pop(void)
{
  CIEnvStackDataType * temp = _data;
  if(temp) 
	{
	  _data = _data->_next;
	  temp->_next = NULL;
	  delete temp;
	  temp = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: top
// Parameters:	(void)
//
// Returns the top element.
//         								    
//////////////////////////////////////////////////////////////////////////////

CIEnv * CIEnvStack::top(void)
{
  if(_data) return _data->_item;
  else return NULL;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: findNodeDeep
// Parameters: (const char*)
//
// find a node.
// search for the entire stack starting from the top.
//
//////////////////////////////////////////////////////////////////////////////

CIEnvNodeType *CIEnvStack::findNodeDeep(const char *key)
{
  if (this->top() == NULL)
 	{
 	  cout << "Error @ CIEnvStack : findNodeDeep" << endl;
 	  cout << "Stack is empty error" << endl;
	  exit(1);
 	  return NULL;
 	}
  else
    {
      CIEnvStackDataType* tmp;
      CIEnvNodeType * tmpNode = NULL;
      tmp = _data;
      while (tmp != NULL)
	{
	  tmpNode = tmp->_item->_findNode(key);
	  if (tmpNode != NULL)
	    tmp = NULL;
	  else
	    tmp = tmp->_next;
	}
      if (tmpNode == NULL)
	{
	  cout << "Error @ CIEnvStack : findNodeDeep" << endl;
	  cout << "Node not found for key " << key << endl;
	  cout << key << " is not bound to anything yet." << endl;
	  exit(1);
	}
      return tmpNode;
    }
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: findNodeDeepDB
// Parameters: (const char*)
//
// Finds a node.
// Searches the entire stack starting from the top.
// BUT THIS TIME ALLOWS NULL TO BE RETURNED IF THE NODE IS NOT FOUND
// ADDED BY DJA WINTER 2002
//
//////////////////////////////////////////////////////////////////////////////

CIEnvNodeType *CIEnvStack::findNodeDeepDB(const char *key)
{
  if (this->top() == NULL)
 	{
 	  cout << "Error @ CIEnvStack : findNodeDeep" << endl;
 	  cout << "Stack is empty error" << endl;
	  exit(1);
 	  return NULL;
 	}
  else
    {
      CIEnvStackDataType* tmp;
      CIEnvNodeType * tmpNode = NULL;
      tmp = _data;
      while (tmp != NULL)
	{
	  tmpNode = tmp->_item->_findNode(key);
	  if (tmpNode != NULL)
	    tmp = NULL;
	  else
	    tmp = tmp->_next;
	}
      return tmpNode;
    }
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: deleteNodeDeep
// Parameters:	(const char*)
//
// find a node and delete it.
// search for entire stack starting from the top.
//
//////////////////////////////////////////////////////////////////////////////

void CIEnvStack::deleteNodeDeep(const char *key)
{
  if (this->top() == NULL)
 	{
 	  cout << "Error @ CIEnvStack : deleteNodeDeep" << endl;
 	  cout << "Stack is empty error" << endl;
	  exit(1);
 	}
  else
 	{
	  int found = 0;
  	  CIEnvStackDataType* tmp;
  	  CIEnvNodeType * tmpNode = NULL;
  	  tmp = _data;
  	  while ((found == 0) && (tmp != NULL))
  		{
		  tmpNode = tmp->_item->_findNode(key);
  		  if (tmpNode != NULL)
  			found = 1;
  		  else
  			tmp = tmp->_next;
  		}
		if (found == 0)
		  {
			cout << "Error @ CIEnvStack : deleteNodeDeep" << endl;
			cout << "Node not found for key " << key << endl;
			cout << key << "is not bound to anything yet." << endl;
			exit(1);
		  }
		else 
		  tmp->_item->erase(key);
 	}
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name: findNodeFromHere
// Parameters:	(const char*, CIEnvStackDataType*)
//
// search the stack from the given starting point.
//
//////////////////////////////////////////////////////////////////////////////

CIEnvNodeType *CIEnvStack::findNodeFromHere(const char *key, CIEnvStackDataType* startingPt)
{
  if (this->top() == NULL)
 	{
 	  cout << "Error @ CIEnvStack : findNodeFromHere" << endl;
 	  cout << "Stack is empty error" << endl;
	  exit(1);
 	  return NULL;
 	}
  else
 	{
  	  CIEnvStackDataType* tmp;
  	  CIEnvNodeType * tmpNode = NULL;
	  
	  tmp = startingPt;
	  
  	  while (tmp)
  		{
		  tmpNode = tmp->_item->_findNode(key);
		  
  		  if (tmpNode)
  			tmp = NULL;
  		  else
			tmp = tmp->_next;
  		}
	  if (!tmpNode)
		{
		  cout << "Error @ CIEnvStack : findNodeFromHere" << endl;
		  cout << "Node not found for key " << key << endl;
		  cout << key << " is not bound to anything yet." << endl;
		  exit(1);
		}
	  return tmpNode;
	}
}
