//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Kee-Eung Kim (kek)                                             
//  FILE: PCMulStmtSeq.C
//  PROJ: COKO
//  DATE: 8/27/96                                                              
//
// SYNOPSIS
//  Method definitions for CMulStmtSeq class.
//  
//////////////////////////////////////////////////////////////////////////////

#include <assert.h>
#include "PCState.H"
#include "PCStmt.H"
#include "PCMulStmtSeq.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CMulStmtSeq
// Parameters:	(CStmt *first, CStmt *second)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CMulStmtSeq::CMulStmtSeq(CStmt *first, CStmt *second)
{
  _first = first;
  _second = second;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~CMulStmtSeq
// Parameters:	(void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CMulStmtSeq::~CMulStmtSeq(void)
{
  if (_first) { delete _first; _first = NULL; }
  if (_second) { delete _second; _second = NULL; }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: Exec
// Parameters:	(CState*)
//
// What to do when executed.
//         								    
//////////////////////////////////////////////////////////////////////////////

CState *CMulStmtSeq::Exec(CState *s)
{
  if(_debug_)
	cout << "-- MulStmtSeq Begins" << endl;

  s = _first->Exec(s);   // Execute the former statement(s) first.
  int success = s->ISuccess();
  
  s = _second->Exec(s);  // Then execute the next statement.

  s->SetSuccess(s->ISuccess() || success);
  // The MulStmtSeq is true when either _first or _second succeed

  if(_debug_)
	cout << "-- MulStmtSeq Ends" << endl;


  return s;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ExecDebug
// Parameters:	(CState*, DDebugController*)
//
// What to do when executed.
//         								    
//////////////////////////////////////////////////////////////////////////////

CState *CMulStmtSeq::ExecDebug(CState *s, DDebugController *db)
{
  if (db->really_execute()) {
    s = _first->ExecDebug(s, db);   // Execute the former statement(s) first.
    int success = s->ISuccess();
    
    if (db->check_for_break(key_number))
      db->doBreak(this, s);
    
    s = _second->ExecDebug(s, db);  // Then execute the next statement.
    
    s->SetSuccess(s->ISuccess() || success);
    // The MulStmtSeq is true when either _first or _second succeed
    
    db->check_for_condition(s, rep_string(), key_number);
  }
  return s;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: rep_string
// Parameters:	(void)
//
// Returns the string representation of only the current 
// node of the COKO tree
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CMulStmtSeq::rep_string ()
{
  char *res = new char [strlen (";") + 1];
  sprintf (res, ";");
  return res; 
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Returns string representation of COKO tree starting from this statement
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CMulStmtSeq::representation (int indent)
{
  char *left = _first -> representation (indent);
  char *right = _second -> representation (indent);
  char *indent_s = get_indent_string(indent);
  char *rep = rep_string();
  char *res = new char [strlen(indent_s) + strlen (left) + strlen (right) + strlen (rep) + strlen ("\n") + 1];
  sprintf (res, "%s%s\n%s%s", left, rep, indent_s, right);
  delete [] left;
  delete [] right;
  delete [] rep;
  delete [] indent_s;
  return res; 
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representationWDI
// Parameters:	(int, DDebugController *)
//
// ADDED DJA SUMMER 2001
// returns the string representation of this COKO RuleBlock, but
// at the same time updates an array in db that locates sub-strings for
// each key number
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CMulStmtSeq::representationWDI (int indent, DDebugController *db)
{
  char *left = _first -> representationWDI (indent, db);
  char *rep = rep_string();
  int currLine = db->getLine();
  int currCol = db->getCol();
  db->updateArray(key_number-1, currLine, currCol, currLine, currCol+strlen(rep));
  db->setLine(currLine+1);
  db->setCol(indent);
  char *right = _second -> representationWDI (indent, db);
  char *indent_s = get_indent_string(indent);
  char *res = new char [strlen(indent_s) + strlen (left) + strlen (right) + strlen (rep) + strlen ("\n") + 1];
  sprintf (res, "%s%s\n%s%s", left, rep, indent_s, right);
  delete [] left;
  delete [] right;
  delete [] rep;
  delete [] indent_s;
  return res; 
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: number_self
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Uses start parameter to assign key_number data_member, calls number_self 
// on any sub_statements and returns current key-number
//         								    
//////////////////////////////////////////////////////////////////////////////

int CMulStmtSeq::number_self (int start)
{
  int temp = _first -> number_self(start);
  key_number=temp;
  return (_second -> number_self(temp+1));
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: output_self
// Parameters:	(DDebugController *)
//
// ADDED DJA SUMMER 2001
// Outputs current statement in list format (for the debugger) if it is a 
// breakable statement; otherwise just calls output_self on any sub-statements
//         								    
//////////////////////////////////////////////////////////////////////////////

void CMulStmtSeq::output_self (DDebugController *db)
{
  _first -> output_self(db);
  char* rep = rep_string();
  db->format_output(key_number, rep);
  delete[] rep;
  _second -> output_self(db);
}

int CMulStmtSeq::is_leaf()
//returns whether current statement is a leaf in the COKO tree
{
  return 0;
}
