//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon Suk  Lee (jl)                                             
//  FILE: PCPrintStmt.C
//  PROJ: COKO
//  DATE: 9/96                                                   
//
// SYNOPSIS
// 
//  This class is a derived class from CStmt. It is used for coko language 
//  debugging purposes.
//  
//////////////////////////////////////////////////////////////////////////////

#include <assert.h>
#include "PCState.H"
#include "PCStmt.H"
#include "PCPrintStmt.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CPrintStmt
// Parameters: (int, char *)       
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CPrintStmt::CPrintStmt(int which_print, char *iname):
  _iname(NULL)
{
  _which_print = which_print;
  if (iname)
  	{
  	  _iname = new char[strlen(iname)+1];
  	  strcpy(_iname, iname);
  	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~CPrintStmt
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CPrintStmt::~CPrintStmt(void)
{
  if (_iname) { delete [] _iname; _iname = NULL; }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: Exec
// Parameters: (CState*)
//
// What to do when executed.
//         								    
//////////////////////////////////////////////////////////////////////////////

CState *CPrintStmt::Exec(CState *s)
{
  if (_which_print == 1)
    {
      CIdent * myNode;
      if (_iname) 
	{ 
	  char * res = NULL;;
	  myNode = s->IFind(_iname);
	  cout << (res = (myNode->Child_Look_Up())->representation()) << endl;
	  if(res) { delete [] res; res = NULL; }		  
	}
      else
	{
	  cout << "Warning @ PCPrintStmt : Exec" << endl;
	  cout << "Nothing to print out warning" << endl;
	}
    }
  else if (_which_print == 2) 
    {
      cout << _iname << endl;
    }
  else if (_which_print == 3)
    {
      char * res = NULL;
      cout << (res = (s->Store())->representation()) << endl;
      if (res) {delete [] res; res = NULL; }
    }
  
  s->SetSuccess(1);
  return s;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ExecDebug
// Parameters: (CState*, DDebugController*)
//
// What to do when executed.
//         								    
//////////////////////////////////////////////////////////////////////////////

CState *CPrintStmt::ExecDebug(CState *s, DDebugController *db)
{
  if (db->really_execute()) {
    if (db->check_for_break(key_number))
      db->doBreak(this, s);
    if (db->really_execute()) {
      if (_which_print == 1)
	{
	  CIdent * myNode;
	  if (_iname) 
	    { 
	      char * res = NULL;;
	      myNode = s->IFind(_iname);
	      db->printInRightPlace(res = (myNode->Child_Look_Up())->representation());
	      db->printInRightPlace("\n");
	      if(res) { delete [] res; res = NULL; }		  
	    }
	  else
	    {
	      db->printInRightPlace("Warning @ PCPrintStmt : ExecDebug");
	      db->printInRightPlace("\n");
	      db->printInRightPlace("Nothing to print out warning");
	      db->printInRightPlace("\n");
	    }
	}
      else if (_which_print == 2)
	{
	  db->printInRightPlace(_iname);
	  db->printInRightPlace("\n");
	}
      else if (_which_print == 3)
	{
	  char * res = NULL;
	  db->printInRightPlace(res = (s->Store())->representation());
	  db->printInRightPlace("\n");
	  if (res) {delete [] res; res = NULL; }
	}
      
      s->SetSuccess(1);
      db->check_for_condition(s, rep_string(), key_number);
    }
  }
  return s;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: rep_string
// Parameters:	(void)
//
// Returns the string representation of only the current 
// node of the COKO tree
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CPrintStmt::rep_string ()
{ 
  if (_which_print == 1) {
    char *res = new char [strlen (_iname) + strlen ("PRINT ") + 1];
    sprintf (res, "PRINT %s", _iname);
    return res;
  } 
  else if (_which_print == 2) {
    //The following code converts all carriage returns to the two characters '\' and 'n' since this is how the string probably originally indicated a carriage return
    int temp_length = strlen(_iname);
    int count = 0;
    for (int i=0; i < temp_length; i++) {
      if (_iname[i] == '\n') {
	count ++;
      }
    }
    int i = 0;
    int j = 0;
    int total = temp_length + count;
    char* new_iname = new char [total + 1];
    for (int i = 0; i < total; i++)
      new_iname[i] = ' ';
    while (i < temp_length) {
      if (_iname[i] != '\n') {
	new_iname[j] = _iname[i];
	i++;
	j++;
      }
      else {
	new_iname[j] = '\\';
	new_iname[j+1] = 'n';
	j+=2;
	i++;
      }
    }
    new_iname[total]='\0';
    char *res = new char [strlen (new_iname) + strlen ("PRINT \"\"") + 1];
    sprintf (res, "PRINT \"%s\"", new_iname);
    delete [] new_iname;
    return res;
  } 
  else if (_which_print == 3) {
    char *res = new char [strlen ("PRINT CURRTREE") + 1];
    sprintf (res, "PRINT CURRTREE");
    return res;
  }
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Returns string representation of COKO tree starting from this statement
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CPrintStmt::representation (int indent)
{
  return rep_string();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representationWDI
// Parameters:	(int, DDebugController *)
//
// ADDED DJA SUMMER 2001
// returns the string representation of this COKO RuleBlock, but
// at the same time updates an array in db that locates sub-strings for
// each key number
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CPrintStmt::representationWDI (int indent, DDebugController *db)
{
  char* temp = rep_string();
  int currLine = db->getLine();
  int currCol = db->getCol();
  db->updateArray(key_number-1, currLine, currCol, currLine, currCol + strlen(temp));
  return temp;
}

int CPrintStmt::is_leaf()
  //returns whether current statement is a leaf in the COKO tree
{
  return 1;
}
