//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon S. Lee (jl)
//  FILE: PCRepeatStmt.C
//  PROJ: COKO
//  DATE: 6/15/97
//
// SYNOPSIS
//  Method definitions for CRepeatStmt class.
//  
//////////////////////////////////////////////////////////////////////////////

#include "PCRepeatStmt.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CRepeatStmt
// Parameters: (CStmt *)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CRepeatStmt::CRepeatStmt(CStmt * stmt)
{
  _stmt = stmt;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~CRepeatStmt
// Parameters:	(void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CRepeatStmt::~CRepeatStmt(void)
{
  if(_stmt) { delete _stmt; _stmt = NULL; }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: Exec
// Parameters:	(CState*)
//
// What to do when executed.
// Keep Executing _stmt until it fails
//         								    
//////////////////////////////////////////////////////////////////////////////

CState *CRepeatStmt::Exec(CState *s)
{
  if(_debug_)
	cout << "-- RepeatStmt Begins" << endl;

  s = _stmt->Exec(s);
  int success = s->ISuccess();

  while (s->ISuccess())
	{
	  s = _stmt->Exec(s);
	}

  s->SetSuccess(success);

  if(_debug_)
	cout << "-- RepeatStmt Ends" << endl;


  return s;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ExecDebug
// Parameters:	(CState*, DDebugController*)
//
// What to do when executed.
// Keep Executing _stmt until it fails
//         								    
//////////////////////////////////////////////////////////////////////////////

CState *CRepeatStmt::ExecDebug(CState *s, DDebugController *db)
{

  if (db->really_execute()) {
    s = _stmt->ExecDebug(s, db);
    int success = s->ISuccess();
    
    while (s->ISuccess())
      {
	s = _stmt->ExecDebug(s, db);
      }
    
    s->SetSuccess(success);
  }
    return s;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: rep_string
// Parameters:	(void)
//
// Returns the string representation of only the current 
// node of the COKO tree
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CRepeatStmt::rep_string ()
{
  char *res = new char [strlen ("REPEAT") + 1];
  sprintf (res, "REPEAT");
  return res; 
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Returns string representation of COKO tree starting from this statement
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CRepeatStmt::representation (int indent)
{
  char *stmt;
  if (_stmt -> is_leaf())
    stmt = _stmt -> representation (indent);
  else {
    char *stmt_orig = _stmt -> representation (indent);
    stmt = new char[strlen(stmt_orig) + strlen("{}") + 1];
    sprintf (stmt, "{%s}", stmt_orig);
    delete[] stmt_orig;
  }
  char *rep = rep_string();
  char *res = new char [strlen (stmt) + strlen (rep) + strlen (" ") + 1];
  sprintf (res, "%s %s", rep, stmt);
  delete [] stmt;
  delete [] rep;
  return res; 
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representationWDI
// Parameters:	(int, DDebugController *)
//
// ADDED DJA SUMMER 2001
// returns the string representation of this COKO RuleBlock, but
// at the same time updates an array in db that locates sub-strings for
// each key number
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CRepeatStmt::representationWDI (int indent, DDebugController *db)
{
  char *rep = rep_string();
  int currCol = db->getCol();
  db->setCol(currCol + strlen(rep) + 1);
  char *stmt;
  if (_stmt -> is_leaf())
    stmt = _stmt -> representationWDI (indent,db);
  else {
    db->setCol(currCol+strlen(rep)+2);
    char *stmt_orig = _stmt -> representationWDI (indent,db);
    stmt = new char[strlen(stmt_orig) + strlen("{}") + 1];
    sprintf (stmt, "{%s}", stmt_orig);
    delete[] stmt_orig;
    currCol = db->getCol();
    db->setCol(currCol+1);
  }
  char *res = new char [strlen (stmt) + strlen (rep) + strlen (" ") + 1];
  sprintf (res, "%s %s", rep, stmt);
  delete [] stmt;
  delete [] rep;
  return res; 
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: number_self
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Uses start parameter to assign key_number data_member, calls number_self 
// on any sub_statements and returns current key-number
//         								    
//////////////////////////////////////////////////////////////////////////////

int CRepeatStmt::number_self (int start)
{
  key_number=0; //REPEAT not a breakable statement
  return (_stmt -> number_self(start));
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: output_self
// Parameters:	(DDebugController *)
//
// ADDED DJA SUMMER 2001
// Outputs current statement in list format (for the debugger) if it is a 
// breakable statement; otherwise just calls output_self on any sub-statements
//         								    
//////////////////////////////////////////////////////////////////////////////

void CRepeatStmt::output_self (DDebugController *db)
{
  //REPEAT is not a breakable stamtent
  _stmt -> output_self(db);
}

int CRepeatStmt::is_leaf()
//returns whether current statement is a leaf in the COKO tree
{
  //I know that REPEAT isn't really a leaf, but for the purposes of the
  //representation method which uses is_leaf, REPEAT is just one statement
  //like a leaf
  return 1;
}
