//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon Suk  Lee (jl)                                             
//  FILE: PCSQPrintStmt.C
//  PROJ: COKO
//  DATE: 7/98                                                   
//
// SYNOPSIS
// 
//  This class is a derived class from CStmt. It is used for coko language 
//  debugging purposes.
//  
//////////////////////////////////////////////////////////////////////////////

#include "PCSQPrintStmt.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CSQPrintStmt
// Parameters: (int, char *)       
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CSQPrintStmt::CSQPrintStmt(int which_print, char *iname):
  _iname(NULL)
{
  _which_print = which_print;
  if (iname)
  	{
  	  _iname = new char[strlen(iname)+1];
  	  strcpy(_iname, iname);
  	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~CSQPrintStmt
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CSQPrintStmt::~CSQPrintStmt(void)
{
  if (_iname) { delete [] _iname; _iname = NULL; }
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: Exec
// Parameters: (CState*)
//
// What to do when executed.
//         								    
//////////////////////////////////////////////////////////////////////////////

CState *CSQPrintStmt::Exec(CState *s)
{
  initstats ();
  if (_which_print == 1)
	{
	  CIdent * myNode;
	  if (_iname) 
		{ 
		  char * res = NULL;;
		  myNode = s->IFind(_iname);
		  koObject* tree = ((koObject*)(myNode->Child_Look_Up()))->obj();
		  res = tree->sql();
		  cout << res << endl;
		  if (tree) { delete tree; tree = NULL; }
		  if (res) { delete [] res; res = NULL; }		  
		}
	  else
		{
		  cout << "Warning @ PCSQPrintStmt : Exec" << endl;
		  cout << "Nothing to print out warning" << endl;
		}
	}
  else if (_which_print == 2)
	{
	  CIdent * myNode;
	  if (_iname) 
		{ 
		  char * res = NULL;;
		  myNode = s->IFind(_iname);
		  koBool* tree = ((koBool*)(myNode->Child_Look_Up()))->boo();
		  res = tree->sql();
		  cout << res << endl;
		  if (tree) { delete tree; tree = NULL; }
		  if (res) { delete [] res; res = NULL; }		  
		}
	  else
		{
		  cout << "Warning @ PCSQPrintStmt : Exec" << endl;
		  cout << "Nothing to print out warning" << endl;
		}
	}
  else if (_which_print == 3)
	{
	  NonType* tree = s->Store();
	  NameUnion* name_ = tree->Myname();
	  if (name_->tag == BNAME )
		{
		  char * res = NULL;;
		  koBool* tree2 = ((koBool*)tree)->boo();
		  res = tree2->sql();
		  cout << res << endl;
		  if (tree2) { delete tree2; tree2 = NULL; }
		  if (res) { delete [] res; res = NULL; }			  
		}
	  else if (name_->tag == ONAME)
		{
		  char * res = NULL;;
		  koObject* tree2 = ((koObject*)tree)->obj();
		  res = tree2->sql();
		  cout << res << endl;
		  if (tree2) { delete tree2; tree2 = NULL; }
		  if (res) { delete [] res; res = NULL; }	
		}
	  else
		cout << "Error @ PCSQPrintStmt.C : Current tree must be either object or bool" << endl;
	  delete name_;
	}

  s->SetSuccess(1);
  return s;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ExecDebug
// Parameters: (CState*, DDebugController*)
//
// What to do when executed.
//         								    
//////////////////////////////////////////////////////////////////////////////

CState *CSQPrintStmt::ExecDebug(CState *s, DDebugController *db)
{
  if (db->really_execute()) {
    if (db->check_for_break(key_number))
      db->doBreak(this, s);
    if (db->really_execute()) {
      initstats ();
      if (_which_print == 1)
	{
	  CIdent * myNode;
	  if (_iname) 
	    { 
	      char * res = NULL;;
	      myNode = s->IFind(_iname);
	      koObject* tree = ((koObject*)(myNode->Child_Look_Up()))->obj();
	      res = tree->sql();
	      db->printInRightPlace(res);
	      db->printInRightPlace("\n");
	      if (tree) { delete tree; tree = NULL; }
	      if (res) { delete [] res; res = NULL; }		  
	    }
	  else
	    {
	      db->printInRightPlace("Warning @ PCSQPrintStmt : ExecDebug");
	      db->printInRightPlace("\n");
	      db->printInRightPlace("Nothing to print out warning");
	      db->printInRightPlace("\n");
	    }
	}
      else if (_which_print == 2)
	{
	  CIdent * myNode;
	  if (_iname) 
	    { 
	      char * res = NULL;;
	      myNode = s->IFind(_iname);
	      koBool* tree = ((koBool*)(myNode->Child_Look_Up()))->boo();
	      res = tree->sql();
	      db->printInRightPlace(res);
	      db->printInRightPlace("\n");
	      if (tree) { delete tree; tree = NULL; }
	      if (res) { delete [] res; res = NULL; }		  
	    }
	  else
	    {
	      db->printInRightPlace("Warning @ PCSQPrintStmt : ExecDebug");
	      db->printInRightPlace("\n");
	      db->printInRightPlace("Nothing to print out warning");
	      db->printInRightPlace("\n");
	    }
	}
      else if (_which_print == 3)
	{
	  NonType* tree = s->Store();
	  NameUnion* name_ = tree->Myname();
	  if (name_->tag == BNAME )
	    {
	      char * res = NULL;;
	      koBool* tree2 = ((koBool*)tree)->boo();
	      res = tree2->sql();
	      db->printInRightPlace(res);
	      db->printInRightPlace("\n");
	      if (tree2) { delete tree2; tree2 = NULL; }
	      if (res) { delete [] res; res = NULL; }			  
	    }
	  else if (name_->tag == ONAME)
	    {
	      char * res = NULL;;
	      koObject* tree2 = ((koObject*)tree)->obj();
	      res = tree2->sql();
	      db->printInRightPlace(res);
	      db->printInRightPlace("\n");
	      if (tree2) { delete tree2; tree2 = NULL; }
	      if (res) { delete [] res; res = NULL; }	
	    }
	  else
	    {
	      db->printInRightPlace("Error @ PCSQPrintStmt.C : Current tree must be either object or bool");
	      db->printInRightPlace("\n");
	    }
	  delete name_;
	}
      
      s->SetSuccess(1);
      db->check_for_condition(s, rep_string(), key_number);
    }
  }
  return s;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: rep_string
// Parameters:	(void)
//
// Returns the string representation of only the current 
// node of the COKO tree
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CSQPrintStmt::rep_string ()
{ 
  if ((_which_print == 1) || (_which_print == 2)){
    char *res = new char [strlen (_iname) + strlen ("PrintSQL ") + 1];
    sprintf (res, "PrintSQL %s", _iname);
    return res;
  } 
  else if (_which_print == 3) {
    char *res = new char [strlen ("PrintSQL CURRTREE") + 1];
    sprintf (res, "PrintSQL CURRTREE");
    return res;
  }
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Returns string representation of COKO tree starting from this statement
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CSQPrintStmt::representation (int indent) {
  return rep_string();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representationWDI
// Parameters:	(int, DDebugController *)
//
// ADDED DJA SUMMER 2001
// returns the string representation of this COKO RuleBlock, but
// at the same time updates an array in db that locates sub-strings for
// each key number
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CSQPrintStmt::representationWDI (int indent, DDebugController *db)
{
  char* temp = rep_string();
  int currLine = db->getLine();
  int currCol = db->getCol();
  db->updateArray(key_number-1, currLine, currCol, currLine, currCol + strlen(temp));
  return temp;
}

int CSQPrintStmt::is_leaf()
  //returns whether current statement is a leaf in the COKO tree
{
  return 1;
}
