#include "PAParser.h"

//Siginificant modifications have been made to this file by DJA in SUMMER 2001

PARuleBlock::PARuleBlock(string name, PACode include, PACode dat_mem, PACode uses_code, PACode declarations, PACode body, PACode pbody)
{
  //DJA SUMMER 2001: changed method to accept dat_mem and uses_code from Coko.y
  _name = name;
  _includeCode = include;
  _dat_memCode = dat_mem;
  _uses_codeCode = uses_code;
  _declarationsCode = declarations;
  _bodyCode = body;
  _prologCode = pbody;
 
}

const char *PARuleBlock::Name(void)
{
  return _name.chars();
}

// Summer 2000: foutPR removed
void PARuleBlock::Emit(ofstream & foutH, ofstream & foutC, ofstream & foutSLC, ofstream & foutMainC, ofstream & foutMainH, int c_type, int doDebug)
{
  PACode::iterator i;

  if (c_type == 2)
	{
	  // Emit directives + class declaration -> Goes into .H
	  foutH << "#ifndef _CORB_" << _name << "_H_" << endl;
	  foutH << "#define _CORB_" << _name << "_H_" << endl;
	  foutH << "#include \"COKO.H\"" << endl;
	  foutH << "#include \"UsesList.H\"" << endl;
	  foutH << "#include \"KOFVar.H\"" << endl;
	  foutH << "#include \"KOPVar.H\"" << endl;
	  foutH << "#include \"KOOVar.H\"" << endl;
	  foutH << "#include \"KOBVar.H\"" << endl;
	  foutH << endl;

	  foutH << "class CORB" << _name << " : public CRuleBlock " << endl;
	  foutH << "{" << endl;
	  foutH << "public:" << endl;
	  
	  //DJA SUMMER 2001 added constructor and destructor
	  foutH << "\tCORB" << _name << "();" << endl;
	  foutH << "\t~CORB" << _name << "();" << endl;
	  foutH << "\tCState *Exec(CState *);" << endl;
	  
	  //DJA SUMMER 2001 added ExecDebug, construct_stmt, and get_name methods
	  //Exec_Debug: What to do if executed in debug mode
	  foutH << "\tCState *ExecDebug(CState*, DDebugController*);" << endl;
	  //construct_stmt will initialize the _rStmt_ that Exec (or ExecDebug) will work on
	  foutH << "\tvoid construct_stmt();" << endl;
	  //get_name will return a copy of the name of the RuleBlock
	  foutH << "\tchar *get_name();" << endl;
	  foutH << "private:" << endl;
	  //DJA SUMMER 2001 added rb_name
	  //this variable will store the name of the RuleBlock
	  foutH << "\tchar* rb_name;" << endl;
	  
	  //Added DJA SUMMER 2001: make each CRule and CRuleBlock data members
	  for(i = _dat_memCode.begin(); i != _dat_memCode.end(); i++) { 
	    foutH << "\t" << *i << endl;
	  }

	  foutH << "};" << endl;
	  foutH << "#endif" << endl;

	  // Emit Exec Body -> Goes into .cc
	  foutC << endl;
	  foutC << "#include \"" << _name << ".h\"" << endl;
	  for(i = _includeCode.begin(); i != _includeCode.end(); i++) { 	  
	    foutC << *i << endl;
	     
	  }

	  foutC << endl;

	  //Constructor added DJA SUMMER 2001
	  //initialze key data members
	  //UseList is a linked list that stores pointers to each used CRule and CRuleBlock
	  //constructed_stmt stores whether construct_stmt has been called yet
	  //numbered_self stores whether number_self has been called yet
	  //and rb_name is set to the name of the RuleBlock
	  foutC << "CORB" << _name << "::CORB" << _name << "()" << endl;
	  foutC << "{" << endl;
	  foutC << "\t_useList = NULL;" << endl;
	  foutC << "\tconstructed_stmt=0;" << endl;
	  foutC << "\tnumbered_self=0;" << endl;
	  foutC << "\trb_name = new char [strlen (\"" << _name << "\") + 1];" << endl;
	  foutC << "\tsprintf (rb_name, \"" << _name << "\");" << endl;
	  foutC << "}" << endl;
	  foutC << endl;

	  //Destrctor added DJA SUMMER 2001
	  foutC << "CORB" << _name << "::~CORB" << _name << "()" << endl;
	  foutC << "{" << endl;
	  foutC << "\tif (rb_name) { delete [] rb_name; rb_name = NULL;}" << endl;
	  foutC << "\tif (_useList) {delete _useList; _useList = NULL;}" << endl;
	  foutC << "}" << endl;
	  foutC << endl;

	  //construct_stmt added DJA SUMMER 2001
	  //uses code generated from Coko.y
	  foutC << "void CORB" << _name << "::construct_stmt()" << endl;
	  foutC << "{" << endl;
	  foutC << "\tif (_useList) {delete _useList; _useList = NULL;}" << endl;
	  foutC << "\t_useList = new UsesList();" << endl;
	  for (i = _declarationsCode.begin(); i!= _declarationsCode.end(); i++) {
	    foutC << "\t" << *i << endl;
	  }
	  for(i = _uses_codeCode.begin(); i != _uses_codeCode.end(); i++) {
	    foutC << "\t" << *i << endl;
	  }
	  foutC << "\tconstructed_stmt=1;" << endl;
	  foutC << "\tnum_stmts = number_self(1) - 1;" << endl;
	  foutC << "}" << endl;
	  foutC << endl;

	  //get_name added DJA SUMMER 2001
	  //returns a copy of the name of the RuleBlock
	  foutC << "char * CORB" << _name << "::get_name()" << endl;
	  foutC << "{" << endl;
	  foutC << "\tchar* copy = new char [strlen (rb_name) + 1];" << endl;
	  foutC << "\tstrcpy (copy, rb_name);" << endl;
	  foutC << "\treturn copy;" << endl;
	  foutC << "}" << endl;
	  foutC << endl;

	  foutC << "CState * CORB" << _name << "::Exec(CState *s)" << endl;
	  foutC << "{" << endl;
	  foutC << "\tif (!constructed_stmt)" << endl;
	  foutC << "\t\tconstruct_stmt();" << endl;
	  foutC << "\tconstructed_stmt=0;" << endl;
	  for(i = _bodyCode.begin(); i != _bodyCode.end(); i++) {
	    foutC << "\t" << *i << endl;
	  }
	  foutC << "}" << endl;
	  foutC << endl;

	  //ExecDebug added DJA SUMMER 2001
	  //Just like Exec except it calls db->start(), db->stop
	  //and the ExecDebug method of the stored statement
	  foutC << "CState * CORB" << _name << "::ExecDebug(CState *s, DDebugController *db)" << endl;
	  foutC << "{" << endl;
	  foutC << "\tif (!constructed_stmt)" << endl;
	  foutC << "\t\tconstruct_stmt();" << endl;
	  foutC << "\tconstructed_stmt=0;" << endl;
	  for(i = _bodyCode.begin(); i != _bodyCode.end(); i++) {
	    
	    //all of this code simply changes the line that says:
	    //_rstmt_->Exec(s) to _rStmt_->ExecDebug(s,db)
	    //and stores this new line of code in newExecLine
	    const char* temp= (*i).chars();
	    int tempSize = i->size();
	    int index = 0;
	    for (int j = 5; j < tempSize; j++)
	      if ((temp[j-5] == 'E') && (temp[j-4] == 'x') && (temp[j-3] == 'e') && (temp[j-2] == 'c') && (temp[j-1] == '(') && (temp[j] == 's'))
		index = j-2;
	    if (index != 0) {
	      char newExecLine[tempSize + 8];
	      int m = 0;
	      int n = 0;
	      while (m <= index) {
		newExecLine[n] = temp[m];
		n++;
		m++;
	      }
	      newExecLine[n] = 'D';
	      newExecLine[n+1] = 'e';
	      newExecLine[n+2] = 'b';
	      newExecLine[n+3] = 'u';
	      newExecLine[n+4] = 'g';
	      newExecLine[n+5] = '(';
	      newExecLine[n+6] = 's';
	      newExecLine[n+7] = ',';
	      newExecLine[n+8] = 'd';
	      newExecLine[n+9] = 'b';
	      n+=10;
	      m+=2;
	      while (m < tempSize) {
		newExecLine[n] = temp[m];
		n++;
		m++;
	      }
	      //let the debugger know that this is the current transformation
	      foutC << "\tdb->start(this);" << endl;
	      foutC << "\t" << newExecLine << endl;
	      //let the debugger know that we are done with this transformation
	      foutC << "\tdb->stop();" << endl;
	    }
	    else
	      foutC << "\t" << *i << endl;
	  }
	  foutC << "}" << endl;
	  
	  if(strcmp(_name.chars(), "Main") == 0) {
		foutC << endl;
		foutC << "#include <stdio.h>" << endl;
		foutC << "#include <stdlib.h>" << endl;
		foutC << "#include <iostream.h>" << endl;
		foutC << "#include \"PAParser.H\"" << endl;
		foutC << "#include \"QLParser.H\"" << endl;
		foutC << "#include \"KONonType.H\"" << endl << endl;
		foutC << "int main(int argc, char *argv[])" << endl;
		foutC << "{" << endl;
		foutC << "\tNonType * qTree = NULL;" << endl;
		foutC << "\tif(argc < 1) qTree = PAParser().parse();" << endl;
		foutC << "\telse if(argc == 2) {" << endl;

		foutC << "\t\tif(strcmp(\"-q\", argv[1]) == 0)" << endl;
		foutC << "\t\t\tqTree = PAParser().parse();" << endl;
		foutC << "\telse" << endl;

		foutC << "\t\tif(strcmp(\"-ddebug\", argv[1]) == 0)" << endl;
		foutC << "\t\t\t_debug_ = 1, qTree = PAParser().parse();" << endl;

		//DJA SUMMER 2001 added -cldbg and -clcokodb
		//-cldbg cause outputting of rule information when run
		//-clcokodb causes the launching of command line COKODB
		//I added this code to the (if (c_type==0)) code below, however
		//I noticed this code mirrors that, so I added it here as well
		//even though I don't know when this code actualy gets executed
		foutC << "\t\t\telse if(strcmp(\"-cldbg\", argv[1]) == 0)" << endl;
		foutC << "\t\t\t\t_cldbg_ = 1, qTree = PAParser().parse();" << endl;
		foutC << "\t\t\telse if(strcmp(\"-clcokodb\", argv[1]) == 0)" << endl;
		foutC << "\t\t\t\t_clcokodb_ = 1, qTree = PAParser().parse();" << endl;
		foutC << "\t\telse qTree = PAParser(argv[1]).parse();" << endl;
		foutC << "\t}" << endl;
		foutC << "\telse if (argc == 3) {" << endl;

		foutC << "\t\tif(strcmp(\"-q\", argv[1]) == 0)" << endl;
		foutC << "\t\t\tqTree = QLParser(argv[2]).parse();" << endl;
		foutC << "\t\telse if(strcmp(\"-q\", argv[2]) == 0) {" << endl;
		foutC << "\t\t\tqTree = QLParser(argv[1]).parse();" << endl;
		foutC << "\t\t}" << endl;
		foutC << "\t\telse" << endl;
		
		foutC << "\t\tif(strcmp(\"-ddebug\", argv[1]) == 0) {" << endl;
		foutC << "\t\t\t_debug_ = 1, qTree = PAParser(argv[2]).parse();" << endl;
		foutC << "\t\t}" << endl;
		foutC << "\t\telse if(strcmp(\"-ddebug\", argv[2]) == 0) {" << endl;
		foutC << "\t\t\t_debug_ = 1, qTree = PAParser(argv[1]).parse();" << endl;
		foutC << "\t\t}" << endl;
		foutC << "\t\telse if(strcmp(\"-cldbg\", argv[1]) == 0) {" << endl;
		foutC << "\t\t\t_cldbg_ = 1, qTree = PAParser(argv[2]).parse();" << endl;
		foutC << "\t\t}" << endl;
		foutC << "\t\telse if(strcmp(\"-cldbg\", argv[2]) == 0) {" << endl;
		foutC << "\t\t\t_cldbg_ = 1, qTree = PAParser(argv[1]).parse();" << endl;
		foutC << "\t\t}" << endl;
		foutC << "\t\telse if(strcmp(\"-clcokodb\", argv[1]) == 0) {" << endl;
		foutC << "\t\t\t_clcokodb_ = 1, qTree = PAParser(argv[2]).parse();" << endl;
		foutC << "\t\t}" << endl;
		foutC << "\t\telse if(strcmp(\"-clcokodb\", argv[2]) == 0) {" << endl;
		foutC << "\t\t\t_clcokodb_ = 1, qTree = PAParser(argv[1]).parse();" << endl;
		foutC << "\t\t}" << endl;
		foutC << "\t\telse if(strcmp(\"-vcokodb\", argv[1]) == 0) {" << endl;
		foutC << "\t\t\t_vcokodb_ = 1, qTree = PAParser(argv[2]).parse();" << endl;
		foutC << "\t\t}" << endl;
		foutC << "\t\telse if(strcmp(\"-vcokodb\", argv[2]) == 0) {" << endl;
		foutC << "\t\t\t_vcokodb_ = 1, qTree = PAParser(argv[1]).parse();" << endl;
		foutC << "\t\t}" << endl;
		foutC << "\t}" << endl;
		
		foutC << "\tif(!qTree) exit(1);" << endl;
		
		foutC << "\tCRuleBlock * MainRB = new CORB" << _name << ";" << endl;
		if (doDebug) {
		  //added DJA SUMMER 2001
		  //Add debug code only if the user compiled with the debug flag
		  foutC << "\tif (_clcokodb_){" << endl;
		  foutC << "\t\tDDebugController *db = new DDebugController(0);" << endl;
		  foutC << "\t\twhile (db->do_restart()) {" << endl;
		  foutC << "\t\t\tCState *s;" << endl;
		  foutC << "\t\t\tNonType * qTreeCopy = qTree->Copy_kTree();" << endl;
		  foutC << "\t\t\ts = MainRB->ExecDebug(new CState(qTreeCopy,new CIEnvStack(),0), db);" << endl; 
		  foutC << "\t\t\tdb->updatePaintWindow(s);" << endl;
		  foutC << "\t\t\tdb->testRunAgain();" << endl;
		  foutC << "\t\t\tdelete s;" << endl;
		  foutC << "\t\t}" << endl;
		  // foutC << "\tchar *result = s->Store()->representation();" << endl;
		  // foutC << "\tcout << result << endl;" << endl;
		  foutC << "\t\tdelete MainRB; delete db; delete qTree;" << endl;
		  foutC << "\t}" << endl;
		  foutC << "\telse if (_vcokodb_){" << endl;
		  foutC << "\t\tDDebugController *db = new DDebugController(1);" << endl;
		  foutC << "\t\tQApplication a(argc, argv);" << endl;
		  foutC << "\t\tMyMainWindow *w = new MyMainWindow(qTree, MainRB, db);" << endl;
		  foutC << "\t\ta.setMainWidget(w);" << endl; 
		  foutC << "\t\tw->show();" << endl;
		  foutC << "\t\ta.exec();" << endl;
		  // foutC << "\tchar *result = s->Store()->representation();" << endl;
		  // foutC << "\tcout << result << endl;" << endl;
		  foutC << "\t\tdelete MainRB; delete db; delete qTree;" << endl;
		  foutC << "\t}" << endl;
		  foutC << "\telse {" << endl;
		  foutC << "\t\tCState *s = MainRB->Exec(new CState(qTree,new CIEnvStack(),0));" << endl;
		  foutC << "\t\tdelete MainRB; delete s;" << endl;
		  foutC << "\t}" << endl;
		  
		}
		else {
		  foutC << "\tCState *s = MainRB->Exec(new CState(qTree,new CIEnvStack(),0));" 
			<< endl;
		  // foutC << "\tchar *result = s->Store()->representation();" << endl;
		  // foutC << "\tcout << result << endl;" << endl;
		  foutC << "\tdelete MainRB; delete s; " << endl;
		  // delete [] result;" << endl;
		  foutC << "}" << endl;
		}
	  }
	  
	}
  // Summer 2000: Shared library support temporarily removed
  // if (c_type == 1)
  //	{  
  //	  // added by jl on July 23 1997
  //	  foutSLC <<  "#include \"" << _name << ".h\"" << endl;
  //	  foutSLC << endl << endl <<  "extern \"C\" {" << endl;
  //	  foutSLC << "\tNonType* callOpt (NonType* arg) { " << endl;
  //	  foutSLC << "\t\tCRuleBlock* tmp = new CORB" << _name <<"();" << endl;
  //	  foutSLC << "\t\tCState* s = new CState(arg,new CIEnvStack(), 0);" << endl;
  //	  foutSLC << "\t\ts = tmp->Exec(s);" << endl;
  //	  //foutSLC << "\t\tNonType* return_val = s->Store();" << endl;
  //	  foutSLC << "\t\tNonType* return_val = s->Store()->Copy_kTree();" << endl;
  //	  // foutSLC << "\t\t/* char* result = s->Store()->representation(); */" << endl;
  //	  // foutSLC << "\t\t/* cout << result << endl; */" << endl;
  //	  foutSLC << "\t\tdelete tmp; delete s;" << endl;
  //	  // /* delete [] result; */" << endl;
  //	  foutSLC << "\t\treturn return_val;" << endl;
  //	  foutSLC << "\t\t}" << endl;
  //	  foutSLC << "\t}" << endl << endl;
  //	  // added by jl on July 23 1997
  //	}

  if (c_type == 0) // Executable
	{
	  foutMainC << "#include \"" << _name << "_main.h\"" << endl;
	  foutMainC << "#include <" << _name << ".h>" << endl;
	  foutMainC << endl;
	  foutMainC << "CState * CORB" << _name << "_main::Exec(CState *s)" << endl;
	  foutMainC << "{" << endl;
	  foutMainC << "CRuleBlock * " << _name << "= new CORB" << _name << ";" <<endl;
	  foutMainC << "CStmt * _rStmt_ =" << endl;
	  foutMainC << "new CRuleInvokeStmt(" << _name << ", NULL);" << endl;
	  foutMainC << "CState * new_s = _rStmt_->Exec(s);" << endl;
	  foutMainC << "delete _rStmt_;" << endl << "delete " << _name << ";" << endl;
	  foutMainC << "return new_s;" << endl << "}" << endl;
	  foutMainC << endl;
	  foutMainC << "CState * CORB" << _name << "_main::ExecDebug(CState *s, DDebugController *db)" << endl;
	  foutMainC << "{" << endl;
	  foutMainC << "CRuleBlock * " << _name << "= new CORB" << _name << ";" <<endl;
	  foutMainC << "CStmt * _rStmt_ =" << endl;
	  foutMainC << "new CRuleInvokeStmt(" << _name << ", NULL);" << endl;
	  foutMainC << "CState * new_s = _rStmt_->ExecDebug(s, db);" << endl;
	  foutMainC << "delete _rStmt_;" << endl << "delete " << _name << ";" << endl;
	  foutMainC << "return new_s;" << endl << "}" << endl;


	  foutMainH << "#ifndef _CORB_" << _name << "_main_H_" << endl;
	  foutMainH << "#define _CORB_" << _name << "_main_H_" << endl;
	  foutMainH << "#include \"COKO.H\"" << endl;
	  foutMainH << "#include \"KOFVar.H\"" << endl;
	  foutMainH << "#include \"KOPVar.H\"" << endl;
	  foutMainH << "#include \"KOOVar.H\"" << endl;
	  foutMainH << "#include \"KOBVar.H\"" << endl;	  
	  foutMainH << endl;
	  foutMainH << "class CORB" << _name << "_main : public CRuleBlock" << endl;
	  foutMainH << "{" << endl;
	  foutMainH << "public:" << endl;
	  foutMainH << "\tCState *Exec(CState *);" << endl;
	  foutMainH << endl << "\tCState *ExecDebug(CState *, DDebugController *);" << endl;
	  foutMainH << "};" << endl << "#endif" << endl;


	  foutMainC << endl;
	  foutMainC << "#include <stdio.h>" << endl;
	  foutMainC << "#include <stdlib.h>" << endl;
	  foutMainC << "#include <iostream.h>" << endl;
	  foutMainC << "#include \"PAParser.H\"" << endl;
	  foutMainC << "#include \"KONonType.H\"" << endl << endl;
	  foutMainC << "#include \"QLParser.H\"" << endl;
	  foutMainC << "int main(int argc, char *argv[])" << endl;
	  foutMainC << "{" << endl;
	  foutMainC << "\tNonType * qTree = NULL;" << endl;
	  foutMainC << "\tif(argc < 1) qTree = PAParser().parse();" << endl;
	  foutMainC << "\telse if(argc == 2) {" << endl;

	  foutMainC << "\t\tif(strcmp(\"-q\", argv[1]) == 0)" << endl;
	  foutMainC << "\t\t\tqTree = PAParser().parse();" << endl;
	  foutMainC << "\t\telse" << endl;

	  foutMainC << "\t\t\tif(strcmp(\"-ddebug\", argv[1]) == 0)" << endl;
	  foutMainC << "\t\t\t\t_debug_ = 1, qTree = PAParser().parse();" << endl;

	  //DJA SUMMER 2001 added -cldbg and -clcokodb and -vcokodb
	  //-cldbg cause outputting of rule information when run
	  //-clcokodb causes the launching of command line COKODB
	  //-vcokodb causes the launching of visual COKODB
	  foutMainC << "\t\t\telse if(strcmp(\"-cldbg\", argv[1]) == 0)" << endl;
	  foutMainC << "\t\t\t\t_cldbg_ = 1, qTree = PAParser().parse();" << endl;
	  foutMainC << "\t\t\telse if(strcmp(\"-clcokodb\", argv[1]) == 0)" << endl;
	  foutMainC << "\t\t\t\t_clcokodb_ = 1, qTree = PAParser().parse();" << endl;
	  foutMainC << "\t\t\telse if(strcmp(\"-vcokodb\", argv[1]) == 0)" << endl;
	  foutMainC << "\t\t\t\t_vcokodb_ = 1, qTree = PAParser().parse();" << endl;
	  foutMainC << "\t\t\telse qTree = PAParser(argv[1]).parse();" << endl;
	  foutMainC << "\t}" << endl;
	  foutMainC << "\telse if (argc == 3) {" << endl;

	  foutMainC << "\t\tif(strcmp(\"-q\", argv[1]) == 0)" << endl;
	  foutMainC << "\t\t\tqTree = QLParser(argv[2]).parse();" << endl;
	  foutMainC << "\t\telse if(strcmp(\"-q\", argv[2]) == 0) {" << endl;
	  foutMainC << "\t\t\tqTree = QLParser(argv[1]).parse();" << endl;
	  foutMainC << "\t\t}" << endl;
	  foutMainC << "\t\telse" << endl;

	  foutMainC << "\t\tif(strcmp(\"-ddebug\", argv[1]) == 0) {" << endl;
	  foutMainC << "\t\t\t_debug_ = 1, qTree = PAParser(argv[2]).parse();" << endl;
	  foutMainC << "\t\t}" << endl;
	  foutMainC << "\t\telse if(strcmp(\"-ddebug\", argv[2]) == 0) {" << endl;
	  foutMainC << "\t\t\t_debug_ = 1, qTree = PAParser(argv[1]).parse();" << endl;
	  foutMainC << "\t\t}" << endl;
	  foutMainC << "\t\telse if(strcmp(\"-cldbg\", argv[1]) == 0) {" << endl;
	  foutMainC << "\t\t\t_cldbg_ = 1, qTree = PAParser(argv[2]).parse();" << endl;
	  foutMainC << "\t\t}" << endl;
	  foutMainC << "\t\telse if(strcmp(\"-cldbg\", argv[2]) == 0) {" << endl;
	  foutMainC << "\t\t\t_cldbg_ = 1, qTree = PAParser(argv[1]).parse();" << endl;
	  foutMainC << "\t\t}" << endl;
	  foutMainC << "\t\telse if(strcmp(\"-clcokodb\", argv[1]) == 0) {" << endl;
	  foutMainC << "\t\t\t_clcokodb_ = 1, qTree = PAParser(argv[2]).parse();" << endl;
	  foutMainC << "\t\t}" << endl;
	  foutMainC << "\t\telse if(strcmp(\"-clcokodb\", argv[2]) == 0) {" << endl;
	  foutMainC << "\t\t\t_clcokodb_ = 1, qTree = PAParser(argv[1]).parse();" << endl;
	  foutMainC << "\t\t}" << endl;
	  foutMainC << "\t\telse if(strcmp(\"-vcokodb\", argv[1]) == 0) {" << endl;
	  foutMainC << "\t\t\t_vcokodb_ = 1, qTree = PAParser(argv[2]).parse();" << endl;
	  foutMainC << "\t\t}" << endl;
	  foutMainC << "\t\telse if(strcmp(\"-vcokodb\", argv[2]) == 0) {" << endl;
	  foutMainC << "\t\t\t_vcokodb_ = 1, qTree = PAParser(argv[1]).parse();" << endl;
	  foutMainC << "\t\t}" << endl;
	  foutMainC << "\t}" << endl;
	  foutMainC << "\tif(!qTree) exit(1);" << endl;
	  foutMainC << "\tCRuleBlock * MainRB = new CORB" << _name << "_main;" << endl;
	  if (doDebug) {
	    //added DJA SUMMER 2001
	    //Add debug code only if the user compiled with the debug flag
	    foutMainC << "\tif ((_clcokodb_) || (_vcokodb_)){" << endl;
	    foutMainC << "\t\tDDebugController *db = new DDebugController(_vcokodb_, argc, argv);" << endl;
	    foutMainC << "\t\twhile (db->do_restart()) {" << endl;
	    foutMainC << "\t\t\tCState *s;" << endl;
	    foutMainC << "\t\t\tNonType * qTreeCopy = qTree->Copy_kTree();" << endl;
	    foutMainC << "\t\t\ts = MainRB->ExecDebug(new CState(qTreeCopy,new CIEnvStack(),0), db);" << endl; 
	    foutMainC << "\t\t\tdb->updatePaintWindow(s);" << endl;
	    foutMainC << "\t\t\tdb->testRunAgain();" << endl;
	    foutMainC << "\t\t\tdelete s;" << endl;
	    foutMainC << "\t\t}" << endl;
	    // foutMainC << "\tchar *result = s->Store()->representation();" << endl;
	    // foutMainC << "\tcout << result << endl;" << endl;
	    foutMainC << "\t\tdelete MainRB; delete db; delete qTree;" << endl;
	    foutMainC << "\t}" << endl;
	    /*foutMainC << "\telse if (_vcokodb_){" << endl;
	    foutMainC << "\t\tDDebugController *db = new DDebugController(1);" << endl;
	    foutMainC << "\t\tQApplication a(argc, argv);" << endl;
	    foutMainC << "\t\tMyMainWindow *w = new MyMainWindow(qTree, MainRB, db);" << endl;
	    foutMainC << "\t\ta.setMainWidget(w);" << endl; 
	    foutMainC << "\t\tw->show();" << endl;
	    foutMainC << "\t\ta.exec();" << endl;
	    // foutMainC << "\tchar *result = s->Store()->representation();" << endl;
	    // foutMainC << "\tcout << result << endl;" << endl;
	    foutMainC << "\t\tdelete MainRB; delete db; delete qTree;" << endl;
	    foutMainC << "\t}" << endl;*/
	    foutMainC << "\telse {" << endl;
	    foutMainC << "\t\tCState *s = MainRB->Exec(new CState(qTree,new CIEnvStack(),0));" << endl;
	    foutMainC << "\t\tdelete MainRB; delete s;" << endl;
	    foutMainC << "\t}" << endl;

	  }
	  else {
	    foutMainC << "\tCState *s = MainRB->Exec(new CState(qTree,new CIEnvStack(),0));" << endl; 
	    // foutMainC << "\tchar *result = s->Store()->representation();" << endl;
	    // foutMainC << "\tcout << result << endl;" << endl;
	    foutMainC << "\tdelete MainRB; delete s;" << endl;
	  }
	  // delete [] result;" << endl;
	  foutMainC << "}" << endl;
	}
}
