//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Marina Zlatkina
//  FILE: BoundProperty.H
//  PROJ: COKO
//  DATE: 8/2001                                                            
//
// SYNOPSIS
//  Template class used when a property is defined in terms of itself or
//  other properties. If there is a rule: Inj(f) /\ Inj(g) => Inj(f o g),
//  it is necessary to pass to the recursive call of Inj the value of 
//  variable "f". This class contains a pattern NonTypeArgs, to represent
//  the what we wish to pass to the recursive call - the isTrue method, at 
//  runtime, substitutes values for variables, and does the recursive call.
//  This class is also used when a property makes use of another property.
//  
//////////////////////////////////////////////////////////////////////////////
#ifndef _BOUNDPROPERTY_H
#define _BOUNDPROPERTY_H

#include "Property.H"
#include "PCState.H"
#include "KONonType.H"

template<typename T>
class BoundProperty : public Property{

public:
  BoundProperty( NonTypeArgs *nt ):pattern(nt) {}
  // constructor

  NonTypeArgs *isTrue(CState *s, NonTypeArgs *a){  
    
    // create a new CState contaning a copy of the current NonType
    // and a pointer to the current variable stack
    CState *state = new CState( s->Store()->Copy_kTree(), s->IEnv(), 1 );
    
    // set b to the NonTypeArgs object created from substituting all
    // variables with their values
    // call the class of which this is a template's isTrue method, giving
    // as arguments the new CState and the built NonTypeArgs
    NonTypeArgs *b = pattern->build(state);
   
   
    return T::instance()->isTrue(state, b);
  }

private:
  NonTypeArgs *pattern;
  // the pattern of the arguments we wish to pass to the recursive call, or to
  // the call of another property
};

#endif 
