/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

// Environment::Environment (Environment &other);

#include "TYTTup.H"

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)					 	 
// FILE: ENEnvEntry.C
// PROJ: KOLA Parser, AQUA Translator
// DATE: 5/95
//
//////////////////////////////////////////////////////////////////////////////

#include "ENEnvEntry.H"
#include "ENEnvironment.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  EnvEntry
// Parameters:	()
//
//  Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

EnvEntry::EnvEntry ()
{

    thevariable = NULL;
    thefunction = NULL;
    thetype = NULL;
    next = NULL;
    
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  EnvEntry
// Parameters:	(char *var, Function *f)							  
//
//  Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

EnvEntry::EnvEntry (char *var, Function *f, TObj *t, EnvEntry *n)
{

    thevariable = new char [strlen (var) + 1];
    sprintf (thevariable, "%s", var);
    thefunction = f;
    thetype = t;
    next = n;
    
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  EnvEntry
// Parameters:	(EnvEntry& other)							  
//
//  Copy Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

EnvEntry::EnvEntry (EnvEntry& other)
{

    if (other . Variable () == NULL)
      thevariable = NULL;
    else {
       thevariable = new char [strlen (other. Variable ()) + 1];
       sprintf (thevariable, "%s", other. Variable ());
     };

    if (other . Funct () == NULL)
      thefunction = NULL;
    else
      thefunction = (other . Funct ()) -> Copy ();

    if (other . Type () == NULL)
	thetype = NULL;
    else
	thetype = (other . Type ()) -> Copy ();

    if (other . Next () == NULL)
	next = NULL;
    else
	next = new EnvEntry (*(other . Next ()));
    
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~EnvEntry
// Parameters:	()							  
//
//  Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

EnvEntry::~EnvEntry ()
{

  if (thevariable) { delete [] thevariable; thevariable = NULL; }
  if (thefunction) { delete thefunction; thefunction = NULL; }
  if (thetype) { delete thetype; thetype = NULL; }
  if (next) { delete next; next = NULL;}
    
}



//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  show
// Parameters:	()							  
//
//  Return textual representation of this.
//         								    
//////////////////////////////////////////////////////////////////////////////

char *EnvEntry::show ()
{

    char *others;
    char *fr;
    char *er;
    char *vr;
    
    if (thevariable == NULL) {
	vr = new char [1];
	sprintf (vr, "");
    } else
	vr = thevariable;
    
    if (thetype != NULL) 
	er = thetype -> representation ();
    else {
	er = new char [1];
	sprintf (er, "");
    };
    
    if (thefunction != NULL) 
	fr = thefunction -> representation ();
    else {
	fr = new char [1];
	sprintf (fr, "");
    };
    
    if (next != NULL) 
	others = next -> show ();
    else {
	others = new char [1];
	sprintf (others, "");
    };
    
	
    char *res = new char [strlen (others) + strlen (vr) + strlen (fr) + strlen (er) + strlen ("[, , ] -> ") + 1];
    sprintf (res, "[%s, %s, %s] -> %s", vr, fr, er, others);

    delete [] others;
    delete [] fr;
    delete er;
    return res;
    
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Add_one ()
// Parameters:	()							  
//
// Modifies function of this entry by adding "o _1".  Then does the same
// for rest of entries in chain
//         								    
//////////////////////////////////////////////////////////////////////////////

void EnvEntry::Add_one ()
{

    if (thefunction == NULL) ; 
    else if (thefunction -> Identity () != FUN_ID) 
       thefunction = new FCompose (thefunction, new F1 ());
    else {
       delete thefunction;
       thefunction = new F1 ();
    };
     
    if (next != NULL)
	next -> Add_one ();
    
}



//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  GetFunc ()
// Parameters:	(char *var)							  
//
// Get function associated with var in chain, or NULL if none
//         								    
//////////////////////////////////////////////////////////////////////////////

Function *EnvEntry::GetFunc (char *var)
{

    if (thevariable != NULL && strcmp (thevariable, var) == 0)
	return thefunction;
    else if (next == NULL)
	return NULL;
    else return next -> GetFunc (var);
	        
}



//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  GetType ()
// Parameters:	(char *var)							  
//
// Get type associated with var in chain, or NULL if none
//         								    
//////////////////////////////////////////////////////////////////////////////

TObj *EnvEntry::GetType (char *var)
{

    if (thevariable == NULL)
        return NULL;
    else if (strcmp (thevariable, var) == 0)
	return thetype;
    else if (next == NULL)
	return NULL;
    else return next -> GetType (var);
	        
}




//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reversed ()
// Parameters:	()
//
// Return chain of copies of EnvEntries in chain buit in reversed order
//         								    
//////////////////////////////////////////////////////////////////////////////

EnvEntry *EnvEntry::reversed ()
{

    char *newv;
    Function *newf;
    TObj *newt;
    EnvEntry *newn;

    newv = thevariable;
    
    if (thefunction == NULL)
      newf = NULL;
    else
      newf = thefunction -> Copy ();

    if (thetype == NULL)
	newt = NULL;
    else if (thetype -> asTup () != NULL) {
        Environment *env = (thetype -> asTup ()) -> Env ();
	newt = new TTup ((new Environment ()) -> insert ((env -> Top ()) -> reversed ()));
    } else 
        newt = thetype -> Copy ();

    if (next == NULL) 
	return new EnvEntry (newv, newf, newt, NULL);
    else {
	newn = next -> reversed ();
        newn -> append (new EnvEntry (newv, newf, newt, NULL));
        return newn;
    }
}
  
//  if (next == NULL) return new EnvEntry (*this);
//  else {
//
//     EnvEntry *e = next -> reversed ();
//     if (thetype -> asTup () != NULL) {
//       Environment *en = new Environment () -> insert (thetype -> asTup () -> Env () -> reversed ());
//       e -> append (new EnvEntry (thevariable, thefunction -> Copy (), new TTup (en)));
//
//     } else
//       e -> append (new EnvEntry (thevariable, thefunction -> Copy (), thetype -> Copy ()));
//	 
//     return e;
//  };



//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  append ()
// Parameters:	(EnvEntry* ent)
//
// Manually adds (var, ID, typ) to end of EnvEntry chain
//         								    
//////////////////////////////////////////////////////////////////////////////

EnvEntry *EnvEntry::append (EnvEntry *ent)
{

  if (next == NULL)
      next = ent;
  else {      
      EnvEntry *lst = next;
      while (lst -> Next () != NULL) 
        lst = lst -> Next ();

      lst -> Next (ent);
  };	

  return this;
        
}











