/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Joon S. Lee (jl@cs.brown.edu)
//
// FILE: KOBVar.C
// PROJ: COKO Compiler
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOBVar.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  BVar
// Parameters:	(char* n)							  
//
//  Constructor. n is the name of the variable.
//         								    
//////////////////////////////////////////////////////////////////////////////

BVar::BVar (char* v, TBool* t) : koBool(t, v)
{
  if (v == NULL) val = NULL;
  else {
    val = new char [strlen(v) + 1];
    sprintf (val, "%s", v);
  }
  variableNum = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  BVar
// Parameters:	(BVar& other)							  
//
//  Copy Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

BVar::BVar (const BVar& other) : koBool ((koBool &) other)
{
  if (other.val == NULL) val = NULL;
  else {
    val = new char [strlen(other.val) + 1];
    sprintf (val, "%s", other.val);
  }
  variableNum = other.getVariableNum();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~BVar
// Parameters:	()							  
//
//  Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

BVar::~BVar ()
{
 if (val) {delete [] val; val = NULL;}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	()							  
//  Return 
//    new new BResID (namestring)
//
/////////////////////////////////////////////////////////////////////////////

//BResult* BVar::reduce ()
//{
//  return new BResID (val);
//}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  "BVar: " + name
//         								    
//////////////////////////////////////////////////////////////////////////////

char* BVar::representation ()
{
  //Commented out by DJA: SUMMER 2002 - the B: was ugly
  //char *res = new char [strlen (val) + strlen ("(B: )") + 1];
  //sprintf (res, "(B: %s)", val);

  //Added by DJA: SUMMER 2002
  char* res;
  if ((strlen(val) > 1) && (val[0] == '_') && (val[1] == 'B')) {
    res = new char [strlen ("_B") + 1];
    sprintf (res, "_B");
  }
  else {
    char *res = new char [strlen (val) + 1];
    sprintf (res, "%s", val);
  }
  return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy
// Parameters:	()							  
//
//  return copy of this as koBool*
//         								    
//////////////////////////////////////////////////////////////////////////////

koBool* BVar::Copy () const
{
  BVar* temp = new BVar (*this);
  temp->setVariableNum(variableNum);
  return temp;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  if there is a entry for this variable name in the stack
//  return the corresponding NonType else return NULL
//  Start search from top of the stack.
//         								    
//////////////////////////////////////////////////////////////////////////////

NonType* BVar::transform(CState* a)
{
  if ((a->IFind(val)) != NULL)
	{
	  a->SetSuccess(T);
	  return (((koBool*)((a->IFind(val))->Child())));
	}
  else
	{
	  a->SetSuccess(F);
	  return NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transformDeep
// Parameters:	(CState*, CIEnvStackDataType*)							  
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//  The difference between this method and transform method is a search
//  method used in VAR classes. (Take a look at transform and transformDeep
//  methods in BVar, FVar, PVar, or OVar.)
//  Transform method searches the top frame only, 
//  while TransformDeep methods searches entire stack starting from 
//  a given point. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* BVar::transformDeep(CState* a, CIEnvStackDataType* startingPt)
{
  if ((a->IFindFromHere(val, startingPt)) != NULL)
	{
	  a->SetSuccess(T);
	  return (((koBool*)((a->IFindFromHere(val, startingPt))->Child())));
	}
  else
	{
	  a->SetSuccess(F);
	  return NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  compares
// Parameters:	(NonType*, CState*)							  
//
//  For variables, I only need to check whether the corresponding
//  NonType Class is as same type as current NonType.
//
//////////////////////////////////////////////////////////////////////////////

int BVar::compares(NonType* nt, CState* a)
{
  // Please check - is this correct?

  NameUnion* tmp_name =  nt->Myname();
  // if ((tmp_name->tag == BNAME) && (tmp_name->bn >= BOOL_AND) && (tmp_name->bn <= BOOL_PRIM))
  if ((tmp_name->tag == BNAME) && (tmp_name->bn != BOOL_VAR) && (tmp_name->bn != BOOL_DC))
	{
	  delete tmp_name;
	  return T;
	}
  else
	{
	  delete tmp_name;
	  return F;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 5 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////

NameUnion* BVar::Myname()
{
	NameUnion* a = new NameUnion(BNAME, BOOL_VAR);
	return a;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: directStore
// Parameters:	(CState*)							  
//
//  In most cases, unification of two trees (pattern tree and query tree) 
//  is handled by compares methods (in a case of a rule) or comparesSTM methods
//  (in a case of statement). However, there is one special case that needs 
//  to be handled differently. 
//
//  For example, 
//               match f ! _O  is a usual case that handled by a comparesSTM method.
//
//  However, the following case is a special one.
//          
//               match f
//
//  This time a top node of a pattern tree is a variable and comparesSTM method
//  cannot handle it correctly.
//
//  For this reason, we need this method which can do the unification job for 
//  the special case.
//
//////////////////////////////////////////////////////////////////////////////

CState* BVar::directStore(CState* a)
{
  if (a->Store() == NULL)
	{
	  cout << "EOR -------- EOR ---- Store is NULL :: BVar" << endl;
	  return NULL;
	}
  
  NameUnion* tmp_name =  (a->Store())->Myname();
  
  // if ((tmp_name->tag == BNAME) && (tmp_name->bn >= BOOL_AND) && (tmp_name->bn <= BOOL_PRIM))
  if ((tmp_name->tag == BNAME) && (tmp_name->bn != BOOL_VAR) && (tmp_name->bn != BOOL_DC))
	{
	  
	  if (((a->IFind(val))) == NULL)
		{
		  a->IInsert(val,  new CIdent( NULL, a->Store(), 0));
		  a->SetSuccess(T);
		}
	  else if (((a->IFind(val))) != NULL)
		{
		  if (!((a->IFind(val))->Child_Look_Up())->deepeq (a->Store()))
			a->SetSuccess(F);
		  else
			a->SetSuccess(T);
		}
	}
  else a->SetSuccess(F);
  
  delete tmp_name; 
  return a;
}      

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  setDEPnInvDEP
// Parameters:  (char*, CState*)
//
// set dependency list (ancestor list) and inverse dependency list (descendant list)
//
//////////////////////////////////////////////////////////////////////////////

void BVar::setDEPnInvDEP(char* vn, CState* s)
{
  if (vn == NULL)
	  s->CInsertInvDep(val);
  else
	{
	  s->IInsertInvDep(vn, val);
	  s->IInsertDep(val, vn);
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  undoSetInvDEP
// Parameters:  (char*, CState*)
//
// undo changes made on descendant list (inverse dependency list).
// undo for dependency list (ancestor list) is not necessary, since 
// any changes on dependency list is undone by poping up
// top frame from the environment stack.
//
//////////////////////////////////////////////////////////////////////////////

void BVar::undoSetInvDEP(char* vn, CState* s)
{
  s->IDeleteInvDep(val);
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void BVar::replaceChild(NonType* child, int which_child)
{
}
  
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* BVar::prolog ()
{
  char *res = new char [strlen (val) + strlen ("V") + 1];
  sprintf (res, "V%s", val);
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref BVar::sp_prolog ()
{
  cout << "Error @BVar::sp_prlog" << endl;
  cout << "query input should not have variable inside error!" << endl;
  return NULL;
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Val
// Parameters:	()
//
// Selector - return a copy of val
//         								    
//////////////////////////////////////////////////////////////////////////////

char* BVar::Val ()
{
  char *res = new char [strlen (val) + 1];
  sprintf (res, "%s", val);
  return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	()
//
//  Return the SQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* BVar::sql ()
{
  char *res = new char [strlen (val) + 1];
  sprintf (res, "%s", val);
  return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	()
//
//  Return the OQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* BVar::oql ()
{
  char *res = new char [strlen (val) + 1];
  sprintf (res, "%s", val);
  return res;
}
