/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)
//        Joon S. Lee (jl)					 	    
//  FILE: KOBool.H 
//  PROJ: COKO, KOLA
//  DATE: 5/97	                                                    
//
// SYNOPSIS
//  The type of Boolean objects.  These are objects 
//  that are used to form conditional expressions (objects)
//  and determine the semantics of predicates.  Bools
//  are not first class objects in that they cannot be 
//  contained in sets.
//  
//  This class and its subclasses should not
//  be confused with the BResult family.  Objects of
//  this family form nodes in a KOLA tree while objects
//  in the latter form nodes in an expression tree.  The former
//  kind of tree is used for optimization, while
//  the latter has the specific use of producing
//  a string denoting the result of the query.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_BOOL_HEADER
#define KOLA_BOOL_HEADER

#include "KGlobals.H"
#include "KONameUnion.H"
#include "KONonType.H"
#include "TYTBool.H"

#ifdef _OSTORE_
#include "OSBool.H"
#endif //_OSTORE_

#include "PCState.H"

class koBool : public NonType
{
public:
  koBool (TBool *t = NULL, char * nm = NULL);
  // Constructor.

  koBool (const koBool& other);
  // Copy Constructor.

  virtual ~koBool ();
  // Destructor.

  virtual koBool* Copy () const = 0;
  // Virtual Copy Constructor.

  NonType* Copy_kTree() const { return Copy(); } 
    
  TBool* Type () const {return this->type;}
  // Selector.
    
  //  virtual BResult* reduce () =0;
  //  Return string denting the "meaning" of this object.
  //  Virtually defined in subclasses.

  virtual char *oql ();
  // Return the OQL string equivalent of this object

  virtual char *sql ();
  // Return the SQL string equivalent of this object

  virtual koBool *boo ();
  // Symbolic simplification of this.  Eventually,
  // this method will replace "bool" presently
  // only defined with _OSTORE_ flag set

#ifdef _OSTORE_

  virtual koBool* bool() = 0;
  // Returns the koBool equivalent of the object.

  virtual OSBool* result() = 0;
  // Returns the OSBool equivalent of the object.

#endif // _OSTORE_

 protected:
  TBool* type;
  //  koBools are always of type TBool
 
  /*********************************************************/
  // right now "myname" is not necessary. Myname() methods does not
  // use this data field. Myname() methods directly manipulates
  // BoolName for each classes.
  // 
  //  BoolName myname;
  //  What kind of bool are you?
  //*******************************************************/

  char * namestring;
  // string representation of the bool name.
};


#endif



