/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)	
//        Joon S. Lee (jl)				 	    
//  FILE: KOFDif.H 
//  PROJ: COKO, KOLA
//  DATE: 5/97	                                                    
//
// SYNOPSIS
//  Class of Difference Function former.  Instantiated
//  with equivalence eq, we get:
//  
//    dif (eq) ! [A, B] = 
//      {a IN A | forall b IN B * not (a eq b)}
//    
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_FDIF_HEADER
#define KOLA_FDIF_HEADER 

#include "KGlobals.H"
#include "KOFunction.H"
#include "KOPredicate.H"
#include "KONameUnion.H"
#include "KONonType.H"
#include "KOOInvoke.H"
#include "KOPCFA.H"
#include "KOPNotEqual.H"
#include "KOFID.H"

#ifdef _OSTORE_
#include "LSJoinSieve.H"
#endif

#include "PCState.H"
#include "KOPVar.H"

#include <string.h>

class FDif : public Function
{
public:
  FDif (Predicate* eq, TFunc * ftype = NULL);
  //  Constructor.  eq an equivalence.

  int compares(NonType* nt, CState* a);
  // Checks for match between NonType and NonType.
  // Stores a variable and CIdent pair onto the stack.
  // This method is for rule matching only.
  // Rule matching is different from statement matching.
  // Duplicated variable entry is allowed here. 

  int comparesSTM(NonType* nt, CState* a);
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This match method is used for matching statement.
  // No duplicated variable entry is allowed.
  
  NonType* transform(CState* s);
  //  Return coresponding Nontype object.
  //  CState is only used in VAR classes.
  //  No other classes need CState.
 
  void setDEPnInvDEP(char*, CState*);
  // set dependency list and inverse dependency list

  void undoSetInvDEP(char*, CState*);
  // undo changes made on inverse dependency list.
  // undo for dependency list is not necessary, since 
  // any changes on dependency list is undone by poping up
  // top frame from the environment stack.

  NonType* transformDeep(CState* s, CIEnvStackDataType* startingPt);
  // difference between transform and 
  // transformDeep is that the former uses
  // IFind method and later uses IFindDeep 
  // method. 

  void replaceChild(NonType* child, int which_child);
  // Replace a child of given tree.

  
  FDif (const FDif& other);
  //  Copy Constructor. 
  
  ~FDif ();
  //  Destructor. 

  Function *Copy () const;
  // Virtual Copy Constructor
    
  int deepeq(NonType * other);
  // Value-based equality

  NameUnion* Myname();
  // returns myname wrapped in side of a class NameUnion.

  int numChildren() const { return 1; }
  //  Number of arguments.

  Predicate* One () const {return one;}
  //  What is the predicate that is the first argument?
  
  //  OResult* reduce (OResult* a);
  //  v1 = new OResVar ();
  //  v2 = new OResVar ();
  //  
  //  return 
  //    new OResSet 
  //      (v,
  //       new BResCOM
  //         (new BResINF (v1, "IN", a -> fst -> reduce ()),
  //          "/\",
  //          new BResNOT
  //  	  (new BResEX
  //              (v2, 
  //               new BResCOM 
  //  	       (new BResINF (v2, "IN", a -> snd -> reduce ()),
  //  	       "/\",
  //  		one -> reduce (new OResPAIR
  //  		  (v1, v2)..)
	      
  char* prolog ();
  // return string representation in prolog term

  char* representation ();
  //  "FFO: DIF (" + one -> representation () + ")"

  const char * Name() const { return namestring; }
  // Return the text representation of the name.

  void SetOne(NonType* a) { delete one; one = (Predicate*)a; }
  // replace existing child to new one.

  void initCurrChild() {currChild = 0;}
  // set (reset) currChild to 0.
  // This needs to be called before using nextChild() for first time.

  NonType* nextChild();
  // returns next child. (next Child = current Child + 1) 
  //If current child is the last child return NULL.

  char *oql (koObject *o);
  // Return the OQL string equivalent of invoking this on o

  char *sql (koObject *o);
  // Return the SQL string equivalent of invoking this on o

#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
    // convert kola tree into prolog term and return it.
#endif // _PROLOG_ 

#ifdef _OSTORE_
  koObject* invoke(koObject* x) { return new OInvoke(this,x); }
  //  This should call the optimizer when call has been implemented.

  OSObject* exec(OSObject* x);
  //  Returns an Iterator
#endif // _OSTORE_

protected:
  Predicate* one;
  //  What is the Predicate that is the 1st argument?

  int currChild;
};


# endif
