/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)	
//        Xiaoyu Wang (wangxy)
//  FILE: KOFEle.H 
//  PROJ: COKO, KOLA
//  DATE: 10/00                                                    
//
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_FELE_HEADER
#define KOLA_FELE_HEADER 

#include "KGlobals.H"
#include "KOObject.H"
#include "KOFunction.H"
#include "KONameUnion.H"
#include "KONonType.H"
#include "KOTag.H"

#include "PCState.H"
#include "KOOVar.H"
#include "KOOInvoke.H"

#include <string.h>

class FEle : public Function
{
public:
  FEle (Tag* t);
  //  Constructor.  o is object argument to former.

  int compares(NonType* nt, CState* a);
  // Checks for match between NonType and NonType.
  // Stores a variable and CIdent pair onto the stack.
  // This method is for rule matching only.
  // Rule matching is different from statement matching.
  // Duplicated variable entry is allowed here. 

  int comparesSTM(NonType* nt, CState* a);
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This match method is used for matching statement.
  // No duplicated variable entry is allowed.
  
  NonType* transform(CState* s);
  //  Return coresponding Nontype object.
  //  CState is only used in VAR classes.
  //  No other classes need CState.
 
  void setDEPnInvDEP(char*, CState*);
  // set dependency list and inverse dependency list

  void undoSetInvDEP(char*, CState*);
  // undo changes made on inverse dependency list.
  // undo for dependency list is not necessary, since 
  // any changes on dependency list is undone by poping up
  // top frame from the environment stack.

  NonType* transformDeep(CState* s, CIEnvStackDataType* startingPt);
  // difference between transform and 
  // transformDeep is that the former uses
  // IFind method and later uses IFindDeep 
  // method. 

  void replaceChild(NonType* child, int which_child);
  // Replace a child of given tree.

  FEle (const FEle& other);
  //  Copy Constructor. 
  
  ~FEle ();
  //  Destructor. 

  Function *Copy () const;
  // Virtual Copy Constructor
    
  int deepeq(NonType * other);
  // Value-based equality

  NameUnion* Myname();
  // returns myname wrapped in side of a class NameUnion.

  int numChildren() const { return 1; }
  //  Number of arguments.

  Tag* One () const {return one;}
  
  char* prolog ();
  // return string representation in prolog term
  
  char* representation ();
  // ELE (% ID)

  const char * Name() const { return namestring; }
  // Text representation of the constant

  void SetOne(NonType* a) { delete one; one = (Tag*)a; }
  // replace existing child to new one.

  void initCurrChild() {currChild = 0;}
  
  NonType* nextChild();
  // returns next child. (next Child = current Child + 1) 
  //If current child is the last child return NULL.

  koObject* inv (koObject *o);
  // Symbolic invocation of this on o.  Eventually,
  // this method will replace "invoke" presently
  // only defined with _OSTORE_ flag set

#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
    // convert kola tree into prolog term and return it.
#endif // _PROLOG_ 

#ifdef _OSTORE_
  koObject* invoke(koObject* x) { return NULL; }

  OSObject* exec(OSObject* x) { return NULL; }
#endif // _OSTORE_
 
protected:
  Tag* one;
  int currChild;
};


# endif


