/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack(mfc)
//        Xiaoyu Wang (wangxy)		
//  FILE: KOFExtract.H 
//  PROJ: COKO, KOLA
//  DATE: 9/00    
//
// SYNOPSIS
//  The extract function former.  Given tagexp
//  we have:
//
//  Extract(tagexp)  
//    
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_FEXTRACT_HEADER
#define KOLA_FEXTRACT_HEADER 

#include "KGlobals.H"
#include "KOFunction.H"
#include "KOPredicate.H"
#include "KONameUnion.H"
#include "KONonType.H"
#include "KOOInvoke.H"
#include "KOBPrimitive.H"
#include "KOPConst.H"

#ifdef _OSTORE_
#include "SODA_GLOBALS.H"
#include "Sieve.H"
#include "MapSieve.H"
#include "FilterSieve.H"
#include <dlfcn.h>
#endif // _OSTORE_

typedef NonType* (*callOptPtr)(NonType*);

#include "PCState.H"
#include "KOTagExp.H"
#include "KOPVar.H"
#include "KOFVar.H"
#include <string.h>

class FExtract : public Function
{
public:
  FExtract (TagExp* te, TFunc * ftype = NULL);
  //  Constructor. extract nodes that matches te  

  int compares(NonType* nt, CState* a);
  // Checks for match between NonType and NonType.
  // Stores a variable and CIdent pair onto the stack.
  // This method is for rule matching only.
  // Rule matching is different from statement matching.
  // Duplicated variable entry is allowed here. 

  int comparesSTM(NonType* nt, CState* a);
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This match method is used for matching statement.
  // No duplicated variable entry is allowed.
  
  NonType* transform(CState* s);
  //  Return coresponding Nontype object.
  //  CState is only used in VAR classes.
  //  No other classes need CState.
 
  void setDEPnInvDEP(char*, CState*);
  // set dependency list and inverse dependency list

  void undoSetInvDEP(char*, CState*);
  // undo changes made on inverse dependency list.
  // undo for dependency list is not necessary, since 
  // any changes on dependency list is undone by poping up
  // top frame from the environment stack.

  NonType* transformDeep(CState* s, CIEnvStackDataType* startingPt);
  // difference between transform and 
  // transformDeep is that the former uses
  // IFind method and later uses IFindDeep 
  // method. 

  void replaceChild(NonType* child, int which_child);
  // Replace a child of given tree.

  FExtract (const FExtract& other);
  //  Copy Constructor. 
  
  ~FExtract ();
  //  Destructor. 

  Function *Copy () const;
  // Virtual Copy Constructor
    
  int deepeq(NonType * other);
  // Value-based equality

  NameUnion* Myname();
  // returns myname wrapped in side of a class NameUnion.

  int numChildren() const { return 1; }
  //  Number of arguments.

  TagExp* One () const {return one;}
  //  What is the predicate that is the first argument?
    
  char* prolog ();
  // return string representation in prolog term

  char* representation ();
  
  const char * Name() const { return namestring; }
  // Return the text representation of the name.

  void SetOne(NonType* a) { delete one; one = (TagExp*)a; }
  // replace existing child to new one.

  void initCurrChild() {currChild = 0;}
  // set (reset) currChild to 0.
  // This needs to be called before using nextChild() for first time.

  NonType* nextChild();
  // returns next child. (next Child = current Child + 1) 
  //If current child is the last child return NULL.

  char *sql (koObject *o);
  // Return the SQL string equivalent of invoking this on o

  char *oql (koObject *o);
  // Return the OQL string equivalent of invoking this on o

#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
    // convert kola tree into prolog term and return it.
#endif // _PROLOG_ 

#ifdef _OSTORE_
  koObject* invoke(koObject* x);
  //  This should call the optimizer when call has been implemented.

  OSObject* exec(OSObject* x);
  //  Returns a Sieve.
#endif // _OSTORE_

protected:
  TagExp* one;
  //  What is the tagexp that is the first argument?

  int currChild;

};


# endif

