/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)	
//       Joon S. Lee (jl@cs.brown.edu)				 	 
// FILE: KOFNJoin.C
// PROJ: COKO Compiler, KOLA Parser
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOFNJoin.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  FNJoin
// Parameters:	(Predicate *p, Function *f, Function *g)
//
//  Constructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

FNJoin::FNJoin (Predicate *p, Function *f, Function *g, TFunc * ftype)
                                  : Function (ftype, FUN_NJOIN,"NJOIN")
{
  one = p;
  two = f;
  three = g;
  variableNum = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  FNJoin
// Parameters:	(FNJoin& other)							  
//
//  Copy Constructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

FNJoin::FNJoin (const FNJoin& other) : Function ((Function &) other)
{
  one = (other . One ()) -> Copy ();
  two = (other . Two ()) -> Copy ();
  three = (other . Three ()) -> Copy ();
  variableNum = other.getVariableNum();
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~FNJoin
// Parameters:	()							  
//
//  Destructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

FNJoin::~FNJoin ()
{
  delete one;
  delete two;
  delete three;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy ()
// Parameters:	()							  
//
//  Virtual Copy Constructor 
//         								    
//////////////////////////////////////////////////////////////////////////////

Function *FNJoin::Copy () const
{
  FNJoin* temp = new FNJoin (*this);
  temp->setVariableNum(variableNum);
  return temp;
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  deepeq
// Parameters:  ()
//
// Value-based equality.
// this is computed recursively.
// Return true if two trees have identical structures, otherwise return false.
// No unification occurs here.
//
//////////////////////////////////////////////////////////////////////////////

int FNJoin::deepeq (NonType * other) 
{
  NameUnion* name_ = other->Myname();
  if ((name_->tag == FNAME ) && (name_->fn == FUN_NJOIN))
  {
    delete name_;
    return ((one->deepeq(((FNJoin*)other)->One())) 
         && (two->deepeq(((FNJoin*)other)->Two()))
         && (three->deepeq(((FNJoin*)other)->Three())));
  }
  else
    {
      delete name_;
      return F;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 4 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////
 
NameUnion* FNJoin::Myname()
{
	NameUnion* a = new NameUnion(FNAME, FUN_NJOIN);
	return a;
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  nextChild
// Parameters:  ()
//
// Returns "currChild++"th child.
// When currChild is equal to the number of children
// (when current child is the last child), it returns NULL.
//
//////////////////////////////////////////////////////////////////////////////

NonType* FNJoin::nextChild()
{
  if (currChild == 0)
	{
	  currChild++;
	  return one;
	}
  else if (currChild == 1)
	{
	  currChild++;
	  return two;
	}
  else if (currChild == 2)
	{
	  currChild++;
	  return three;
	}
  else return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	()							  
//
/////////////////////////////////////////////////////////////////////////////

// OResult* FNJoin::reduce (OResult* a)
// {

//  if (! (a -> isPair ())) {

//     char *or = one -> representation ();
//     char *tr = two -> representation ();
//     char *thr = three -> representation ();
//     char *r = new char [strlen (or) + strlen (tr) + strlen (thr) + strlen ("NJOIN (, , )") + 1];
//     sprintf (r, "NJOIN (%s, %s, %s)", or, tr, thr);
//     OResult *res = new OResBB (r, a -> Copy ());
//     delete [] or;
//     delete [] tr;
//     delete [] thr;
//     delete [] r;
//     delete a;
//     return res;
    
//  } else {

//   OResVAR *v1 = new OResVAR ();
//   OResVAR *v2 = new OResVAR ();
//   OResPAIR *a2 = ((OResPAIR *) a);

//   OResSET *res = 
//       new OResSET
//        (new OResPAIR (v1 -> Copy (), 
// 					  three -> reduce (new OResSET (two -> reduce (v2 -> Copy ()),
// 													new BResCOMP (new BResINF (v2 -> Copy (), IN, a2 -> One () -> Copy ()),
// 																  SAND,
// 																  one -> reduce (new OResPAIR (v1 -> Copy (), v2 -> Copy ()))
// 																  )
// 													)
// 									   )
// 					  ),
// 		new BResINF (v1 -> Copy (), IN, a2 -> Two () -> Copy ()));
	   
//   delete v1;
//   delete v2;
//   delete a2;
//   return res;
//   }
// }

// ////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  "FFO: FNJoin (" + one -> representation () + ")"
//         								    
//////////////////////////////////////////////////////////////////////////////

char* FNJoin::representation ()
{
  char *orr = one -> representation ();
  char *tr = two -> representation ();
  char *thr = three -> representation ();
  char *res = new char [strlen (orr) + strlen (tr)  + strlen (thr) + strlen ("NJOIN (, , )") + 1];
  sprintf (res, "NJOIN (%s, %s, %s)", orr, tr, thr);
  delete [] orr;    
  delete [] tr;    
  delete [] thr;    
  return res;
}

/////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  Return coresponding Nontype object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* FNJoin::transform(CState* s)
{
  Predicate* first = (Predicate*) One() -> transform(s);
  Function* second = (Function*) Two() -> transform(s);
  Function* third = (Function*) Three() -> transform(s);
  FNJoin* temp = new FNJoin(first,second,third,Type());
  temp->setVariableNum(variableNum);
  return temp;
    
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  transformDeep
// Parameters:  (CState*, CIEnvStackDataType*)
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly.
//  The difference between this method and transform method is a search
//  method used in VAR classes. (Take a look at transform and transformDeep
//  methods in BVar, FVar, PVar, or OVar.)
//  Transform method searches the top frame only,
//  while TransformDeep methods searches entire stack starting from
//  a given point.
//
//////////////////////////////////////////////////////////////////////////////

NonType* FNJoin::transformDeep(CState* s, CIEnvStackDataType* startingPt)
{
  Predicate* first = (Predicate*) One() -> transformDeep(s, startingPt);
  Function* second = (Function*) Two() -> transformDeep(s, startingPt);
  Function* third = (Function*) Three() -> transformDeep(s, startingPt);
  FNJoin* temp = new FNJoin(first,second,third,Type());
  temp->setVariableNum(variableNum);
  return temp;
    
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for rule matching only.
// Rule matching is different from statement matching.
// Duplicated variable entry is allowed here.
//
// For example, foo: g o g --> g o g is a legal rule.
//                  -------   -------
//                    |         |_\
//                    |_\         / building part
//                      / matching part
//
// Assume that we apply this rule to a kola tree of the form "ID o ID."
// When unification occurs in a matching part, first we store first g and ID pair
// into the env stack. Then the next time we face another g (second one),
// we search the stack for the matching pair with variable name g and simply check
// if the matching pair in the stack has same structure as the matching pair we have.
// In this case, second pair is (g, ID) as well and the rule gets fired successfully.
//
// Now assume that we apply the same rule to a kola tree of the form "ID o (ID o ID)."
// In this case, the rule fails since the second pair is (g, ID o ID),
// while the first pair is (g, ID).
//
// Note that even in a case of failure, the rule itself is legal.
//
// "Duplication of a variable is legal in a rule section."
//
//////////////////////////////////////////////////////////////////////////////

int FNJoin::compares(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion* tmp_name = nt->Myname();
  
  if ((tmp_name->tag == FNAME) && (tmp_name->fn == FUN_NJOIN))
	{
	  // do it for first child.
	  if (one->compares(((FNJoin*)nt)->One(), a))
		// compare succeeded.
		{
		  if (isChildVar(one))
			// child (one) is a variable.
			{
			  if ((a->IFind(((FVar*)one)->Name())) == NULL)
				{
				  // child is a new variable.
				  a->IInsert(((FVar*)one)->Name(), new CIdent((FNJoin*)nt,((FNJoin*)nt)->One(),1));
				  // Push entry into the stack.
				}
			  else 
				{
				  // Child is a variable but not new one.
				  // In this case, I need to make sure that what's
				  // already in the stack and what I have now is the same.
				  if (!(((a->IFind(((FVar*)one)->Name()))->Child_Look_Up())->deepeq(((FNJoin*)nt)->One())))
					return_val++;
				  // match failed.
				  // else do none.
				}
			}
		  // else do none.
		  // no child is variable.
		}
	  else return_val++;
	  // compare failed
	  
	  // do it for second child.
	  if (two->compares(((FNJoin*)nt)->Two(), a))
		{
		  if (isChildVar(two))
			{
			  if ((a->IFind(((FVar*)two)->Name())) == NULL)
				{
				  a->IInsert(((FVar*)two)->Name(), new CIdent((FNJoin*)nt, ((FNJoin*)nt)->Two(), 2));
				}
			  else 
				{
				  if (!(((a->IFind(((FVar*)two)->Name()))->Child_Look_Up())->deepeq(((FNJoin*)nt)->Two())))
					return_val++;
				}
			}
		}
	  else return_val++;

	  // do it for third child.
	  if (three->compares(((FNJoin*)nt)->Three(), a))
		{
		  if (isChildVar(three))
			{
			  if ((a->IFind(((FVar*)three)->Name())) == NULL)
				{
				  a->IInsert(((FVar*)three)->Name(), new CIdent((FNJoin*)nt, ((FNJoin*)nt)->Three(), 3));
				}
			  else 
				{
				  if (!(((a->IFind(((FVar*)three)->Name()))->Child_Look_Up())->deepeq(((FNJoin*)nt)->Three())))
					return_val++;
				}
			}
		}
	  else return_val++;
	}
  else return_val++;
  // When one of it's child is a variable and faild match, return_val
  // is set to some number bigger than 0.
  // In this case, return false(0).
  // Otherwise, return true(1).
  
  delete tmp_name;
 
  if (return_val)
	return F;
  else
	return T;
  
} 

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: comparesSTM
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for statement matching only.
// Statement matching is different from rule matching.
// No duplicated variable entry is allowed here.
//
// For example,
//             match f ! _O,
//                   f =  g o g  is a illegal statement (equation).
//                       -------
//                         |
//                         |_\
//                           / matching part
//
// Assume that we have a query tree of the form "ID o ID ! 3."
// Variable f gets bounded to "ID o ID" and we have to match "ID o ID" to "g o g."
// Matching "ID o ID" to "g o g" is perfectly legal in a rule section.
// However, it is illegal in statement section of the program.
// Remember that every variable on the right hand side of an equation must be a fresh one.
// However, after we match first g to ID, the second g is no longer fresh variable.
//
// "No duplication of a variable is legal in a statement section."
//
//////////////////////////////////////////////////////////////////////////////

int FNJoin::comparesSTM(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion* tmp_name = nt->Myname();
  
  if ((tmp_name->tag == FNAME) && (tmp_name->fn == FUN_NJOIN))
	{
	  // do it for first child.
	  if (one->comparesSTM(((FNJoin*)nt)->One(), a))
		// compare succeeded.
		{
		  if (isChildVar(one))
			// child (one) is a variable.
			{
			  if ((a->IFind(((FVar*)one)->Name())) == NULL)
				{
				  // child is a new variable.
				  a->IInsert(((FVar*)one)->Name(), new CIdent((FNJoin*)nt,((FNJoin*)nt)->One(),1));
				  // Push entry into the stack.
				}
			  else 
				{
				  return_val++;
				  cout << "Error : Variable " << ((FVar*)one)->Name() << " is used twice. " << endl;
				}
			}
		  // else do none.
		  // no child is variable.
		}
	  else return_val++;
	  // compare failed
	  
	  // do it for second child.
	  if (two->comparesSTM(((FNJoin*)nt)->Two(), a))
		{
		  if (isChildVar(two))
			{
			  if ((a->IFind(((FVar*)two)->Name())) == NULL)
				{
				  a->IInsert(((FVar*)two)->Name(), new CIdent((FNJoin*)nt, ((FNJoin*)nt)->Two(), 2));
				}
			  else 
				{
				  return_val++;
				  cout << "Error : Variable " << ((FVar*)two)->Name() << " is used twice. " << endl;
				}
			}
		}
	  else return_val++;
	  
	  // do it for third child.
	  if (two->comparesSTM(((FNJoin*)nt)->Three(), a))
		{
		  if (isChildVar(three))
			{
			  if ((a->IFind(((FVar*)three)->Name())) == NULL)
				{
				  a->IInsert(((FVar*)three)->Name(), new CIdent((FNJoin*)nt, ((FNJoin*)nt)->Three(), 3));
				}
			  else 
				{
				  return_val++;
				  cout << "Error : Variable " << ((FVar*)three)->Name() << " is used twice. " << endl;
				}
			}
		}
	  else return_val++;
	}
  else return_val++;
  // When one of it's child is a variable and faild match, return_val
  // is set to some number bigger than 0.
  // In this case, return false(0).
  // Otherwise, return true(1).
  
  delete tmp_name;
 
  if (return_val)
	return F;
  else
	return T;

} 

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  setDEPnInvDEP
// Parameters:  (char*, CState*)
//
// set dependency list (ancestor list) and inverse dependency list (descendant list)
//
//////////////////////////////////////////////////////////////////////////////

void FNJoin::setDEPnInvDEP(char* vn, CState* s)
{
  one->setDEPnInvDEP(vn,s);
  two->setDEPnInvDEP(vn,s);
  three->setDEPnInvDEP(vn,s);
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  undoSetInvDEP
// Parameters:  (char*, CState*)
//
// undo changes made on descendant list (inverse dependency list).
// undo for dependency list (ancestor list) is not necessary, since
// any changes on dependency list is undone by poping up
// top frame from the environment stack.
//
//////////////////////////////////////////////////////////////////////////////

void FNJoin::undoSetInvDEP(char* vn, CState* s)
{
  one->undoSetInvDEP(vn,s);
  two->undoSetInvDEP(vn,s);
  three->undoSetInvDEP(vn,s);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void FNJoin::replaceChild(NonType* child, int which_child)
{
  if ( which_child == 1 )
	SetOne(child);
  else if ( which_child == 2 )
	SetTwo(child);
  else if ( which_child == 3 )
	SetThree(child);
}

#ifdef _OSTORE_
//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: exec
//  Parameters:(OSObject*)
//
//  Implements the njoin function former.
//
//////////////////////////////////////////////////////////////////////////////

OSObject* FNJoin::exec(OSObject* x)
{
  return NULL;
}

#endif // _OSTORE_
  
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* FNJoin::prolog ()
{
  char *orr = one -> prolog ();
  char *tr = two -> prolog ();
  char *thr = three -> prolog ();
  char *res = new char [strlen (orr) + strlen (tr) + strlen (thr) + strlen ("fnjoin(, , )") + 1];
  sprintf (res, "fnjoin(%s, %s, %s)", orr, tr, thr);
  delete [] orr;    
  delete [] tr;    
  delete [] thr;    
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref FNJoin::sp_prolog ()
{
  SP_term_ref or = one -> sp_prolog ();
  SP_term_ref tr = two -> sp_prolog ();
  SP_term_ref thr = three -> sp_prolog ();
  SP_term_ref res = SP_new_term_ref();
  SP_cons_functor(res, SP_atom_from_string("fnjoin"),3,or,tr,thr);
  return res;
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	(koObject *o)
//
// Return the SQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* FNJoin::sql (koObject *o)
{
    NameUnion *nu;
    nu = o -> Myname ();
    if (nu -> on == OBJ_PAIR) {
	  delete nu;
	  koObject *o1 = o -> asPair () -> One ();
	  koObject *o2 = o -> asPair () -> Two ();

	  OVar *x   = new OVar (0); 
	  FIterate *f = new FIterate (new PCElt (one -> Copy (), x), two -> Copy ());
	  
	  char *sel = (three -> inv (two -> inv (o2))) -> sql ();
	  char *fro = o1 -> sql ();
	  
	  if (strncmp (sel, "ERROR", 5) == 0) {
		delete [] fro;
		delete f;            // Deleting f deletes x
		return sel;
	  } else if (strncmp (fro, "ERROR", 5) == 0) {
		delete [] sel;
		delete f;            // Deleting f deletes x
		return fro;
	  } else {
		char *res = new char [strlen ("(select distinct ,  from  )") +
							 strlen (sel) + strlen (fro) + (2 * strlen (x -> Name ())) + 1];
		sprintf (res, "(select distinct %s, %s from %s %s)", x -> Name (), sel, fro, x -> Name ());
		delete [] sel;
		delete [] fro;
		delete f;           // Deleting f deletes x
		return res;
	  }
	} else {
	  delete nu;
	  cerr << "ERROR: Calling NJOIN on non-Pair Object" << endl;
	  return NULL;
	}
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	(koObject *o)
//
// Return the OQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* FNJoin::oql (koObject *o)
{
    NameUnion *nu;
    nu = o -> Myname ();
    if (nu -> on == OBJ_PAIR) {
	  delete nu;
	  koObject *o1 = o -> asPair () -> One ();
	  koObject *o2 = o -> asPair () -> Two ();

	  OVar *x   = new OVar (0); 
	  FIterate *f = new FIterate (new PCElt (one -> Copy (), x), two -> Copy ());
	  
	  char *sel = (three -> inv (two -> inv (o2))) -> oql ();
	  char *fro = o1 -> oql ();
	  
	  if (strncmp (sel, "ERROR", 5) == 0) {
		delete [] fro;
		delete f;            // Deleting f deletes x
		return sel;
	  } else if (strncmp (fro, "ERROR", 5) == 0) {
		delete [] sel;
		delete f;            // Deleting f deletes x
		return fro;
	  } else {
		char *res = new char [strlen ("(select distinct struct (one: , two: ) from  )") +
							 strlen (sel) + strlen (fro) + (2 * strlen (x -> Name ())) + 1];
		sprintf (res, "(select distinct struct (one: %s, two: %s) from %s %s)", x -> Name (), sel, fro, x -> Name ());
		delete [] sel;
		delete [] fro;
		delete f;           // Deleting f deletes x
		return res;
	  }
	} else {
	  delete nu;
	  cerr << "ERROR: Calling NJOIN on non-Pair Object" << endl;
	  return NULL;
	}
}
