/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. lee (jl@cs.brown.edu)
// FILE: KOFShiftL.C
// PROJ: COKO Compiler, KOLA Parser
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOFShiftL.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  FShftl
// Parameters:	()							  
//
//  Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

FShiftL::FShiftL (TFunc * ftype)  : Function (ftype, FUN_SHIFTL, "SHL")
{
  variableNum = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  FShftl
// Parameters:	(FShftl& other)							  
//
//  Copy Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

FShiftL::FShiftL (const FShiftL& other) : Function ((Function &) other)
{
  variableNum = other.getVariableNum();
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~FShftl
// Parameters:	()							  
//
//  Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

FShiftL::~FShiftL ()
{
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy
// Parameters:	()							  
//
//  return copy of this as Function*
//         								    
//////////////////////////////////////////////////////////////////////////////

Function* FShiftL::Copy () const
{
  FShiftL* temp = new FShiftL (*this);
  temp->setVariableNum(variableNum);
  return temp;
}


//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  deepeq
// Parameters:  ()
//
// Value-based equality.
// this is computed recursively.
// Return true if two trees have identical structures, otherwise return false.
// No unification occurs here.
//
//////////////////////////////////////////////////////////////////////////////

int FShiftL::deepeq (NonType * other) 
{ 
  NameUnion* name_ = other->Myname();
  if ((name_->tag == FNAME ) && (name_->fn == FUN_SHIFTL))
  {
    delete name_;
    return T;
  }
  else
    {
      delete name_;
      return F;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 4 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////

NameUnion* FShiftL::Myname()
{
	NameUnion* a = new NameUnion(FNAME, FUN_SHIFTL);
	return a;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  "FPrim: SHFTL"
//         								    
//////////////////////////////////////////////////////////////////////////////

char* FShiftL::representation ()
{

  char *res = new char [strlen ("SHL") + 1];
  sprintf (res, "%s", "SHL");

  return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	()							  
//
/////////////////////////////////////////////////////////////////////////////

// OResult* FShiftL::reduce (OResult* a)
// {

//   if (a -> isPair () && ((OResPAIR *) a) -> Two () -> isPair ()) {

//     OResPAIR *a2 = ((OResPAIR *) a);
//     OResPAIR *a3 = (OResPAIR *) (((OResPAIR *) a2) -> Two ());
//     OResult *res =  new OResPAIR (new OResPAIR (a2 -> One () -> Copy (), a3 -> One () -> Copy ()), a3 -> Two () -> Copy ());
//     delete a;
//     return res;
	
//   } else {  

//     char *r = new char [strlen ("SHIFTL") + 1];
//     sprintf (r, "SHIFTL");
//     OResult *res = new OResBB (r, a -> Copy ());
//     delete [] r;
//     delete a;
//     return res;

//   }
    
// }

// //////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* FShiftL::transform(CState* s)
{
  FShiftL* temp = new FShiftL(Type());
  temp->setVariableNum(variableNum);
  return temp;
    
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for rule matching only.
// Rule matching is different from statement matching.
// Duplicated variable entry is allowed here.
//
// For example, foo: g o g --> g o g is a legal rule.
//                  -------   -------
//                    |         |_\
//                    |_\         / building part
//                      / matching part
//
// Assume that we apply this rule to a kola tree of the form "ID o ID."
// When unification occurs in a matching part, first we store first g and ID pair
// into the env stack. Then the next time we face another g (second one),
// we search the stack for the matching pair with variable name g and simply check
// if the matching pair in the stack has same structure as the matching pair we have.
// In this case, second pair is (g, ID) as well and the rule gets fired successfully.
//
// Now assume that we apply the same rule to a kola tree of the form "ID o (ID o ID)."
// In this case, the rule fails since the second pair is (g, ID o ID),
// while the first pair is (g, ID).
//
// Note that even in a case of failure, the rule itself is legal.
//
// "Duplication of a variable is legal in a rule section."
//
//////////////////////////////////////////////////////////////////////////////

int FShiftL::compares(NonType* nt, CState* a)
{
  NameUnion* tmp_name = nt->Myname();
  if ((tmp_name->tag == FNAME) && (tmp_name->fn == FUN_SHIFTL))
	{
	  delete tmp_name;
	  return T;
	}
  else
	{
	  delete tmp_name;
	  return F;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void FShiftL::replaceChild(NonType* child, int which_child)
{
}

#ifdef _OSTORE_
//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: invoke
//  Parameters:(koObject*)
//
//  Restructures the pair object by shifting the "inner" pair left.
//
//////////////////////////////////////////////////////////////////////////////

koObject* FShiftL::invoke(koObject* x)
{
  return new OInvoke(this,x);
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: exec
//  Parameters:(OSObject*)
//
//  Restructures the pair object by shifting the "inner" pair left.
//
//////////////////////////////////////////////////////////////////////////////

OSObject* FShiftL::exec(OSObject* x)
{
  OSObject *i = ((OSPair*)x)->One();
  OSObject *j = ((OSPair*)((OSPair*)x)->Two())->One();
  OSObject *k = ((OSPair*)((OSPair*)x)->Two())->Two();

  return new OSPair(new OSPair(i,j), k);
}

#endif // _OSTORE_
  
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* FShiftL::prolog ()
{
  char *res = new char [strlen ("fshiftl") + 1];
  sprintf (res, "fshiftl");
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref FShiftL::sp_prolog ()
{
  SP_term_ref res = SP_new_term_ref();
  SP_put_string(res,"fshiftl");
  return res;
}
#endif // _PROLOG_


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  inv
// Parameters:	(koObject *o)
//
// Symbolic invocation of this on o.  Eventually,
// this method will replace "invoke" presently
// only defined with _OSTORE_ flag set
//   [a, [b, c]] --> [[a, b], c]
//
/////////////////////////////////////////////////////////////////////////////

koObject *FShiftL::inv (koObject *o) 
{
  koObject *one, *two;
  NameUnion *nu = o -> Myname ();

  if (nu ->on == OBJ_PAIR) {   
	delete nu;
    one = o -> asPair () -> One ();
    two = o -> asPair () -> Two ();
    nu = two -> Myname ();
    if (nu ->on == OBJ_PAIR) {
	  delete nu;
	  return new OPair (new OPair (one -> Copy (), two -> asPair () -> One () -> Copy ()), two -> asPair () -> Two () -> Copy ());
	} else {
	  delete nu;
	  return new OInvoke (Copy (), o -> Copy ());
	}
  } else {
	delete nu;
	return new OInvoke (Copy (), o -> Copy ());
  }
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	(koObject *o)
//
// Return the SQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* FShiftL::sql (koObject *o)
{    
    NameUnion *nu;
    koObject *one, *two;
    nu = o -> Myname ();
    if (nu -> on == OBJ_PAIR) {
	  delete nu;
	  one = o -> asPair () -> One ();
	  two = o -> asPair () -> Two ();
	  nu = two -> Myname ();
	  if (nu ->on == OBJ_PAIR) {
		delete nu;
	    char *arg1 = one -> sql ();
	    char *arg2 = two -> asPair () -> One () -> sql ();
	    char *arg3 = two -> asPair () -> Two () -> sql ();
		if (strncmp (arg1, "ERROR", 5) == 0)
		  return arg1;
		else if (strncmp (arg2, "ERROR", 5) == 0)
		  return arg2;
		else if (strncmp (arg3, "ERROR", 5) == 0)
		  return arg3;
		else {
		  char *res = new char [strlen (", , ") + strlen (arg1) + strlen (arg2) + strlen (arg3) + 1];
		  sprintf (res, "%s, %s, %s", arg1, arg2, arg3);
		  delete [] arg1;
		  delete [] arg2;
		  delete [] arg3;
		  return res;
		} 
	  } else {
		delete nu;
		char *res = new char [strlen ("ERROR: SHL applied to pair of improper structure") + 1];
		sprintf (res, "ERROR: SHL applied to pair of improper structure");
		return res;
	  }
	} else {
	  delete nu;
	  char *res = new char [strlen ("ERROR: SHL applied to non-pair") + 1];
	  sprintf (res, "ERROR: SHL applied to non-pair");
	  return res;
	}
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	(koObject *o)
//
// Return the OQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* FShiftL::oql (koObject *o)
{    
    NameUnion *nu;
    koObject *one, *two;
    nu = o -> Myname ();
    if (nu -> on == OBJ_PAIR) {
	  delete nu;
	  one = o -> asPair () -> One ();
	  two = o -> asPair () -> Two ();
	  nu = two -> Myname ();
	  if (nu ->on == OBJ_PAIR) {
		delete nu;
	    char *arg1 = one -> oql ();
	    char *arg2 = two -> asPair () -> One () -> oql ();
	    char *arg3 = two -> asPair () -> Two () -> oql ();
		if (strncmp (arg1, "ERROR", 5) == 0)
		  return arg1;
		else if (strncmp (arg2, "ERROR", 5) == 0)
		  return arg2;
		else if (strncmp (arg3, "ERROR", 5) == 0)
		  return arg3;
		else {
		  char *res = new char [strlen ("struct (one: struct (one: , two: ), two: )") + strlen (arg1) + strlen (arg2) + strlen (arg3) + 1];
		  sprintf (res, "struct (one: struct (one: %s, two: %s), two: %s)", arg1, arg2, arg3);
		  delete [] arg1;
		  delete [] arg2;
		  delete [] arg3;
		  return res;
		} 
	  } else {
		delete nu;
	    char *arg1 = one -> oql ();
	    char *arg2 = two -> oql ();
		if (strncmp (arg1, "ERROR", 5) == 0)
		  return arg1;
		else if (strncmp (arg2, "ERROR", 5) == 0)
		  return arg2;
		else {
		  char *res = new char [strlen ("struct (one: struct (one: , two: .one), two: .two)") + strlen (arg1) + (2 * strlen (arg2)) + 1];
		  sprintf (res, "struct (one: struct (one: %s, two: %s.one), two: %s.two)", arg1, arg2, arg2);
		  delete [] arg1;
		  delete [] arg2;
		  return res;
		}
	  }
	} else {
	  delete nu;
	  char *arg1 = o -> oql ();
	  if (strncmp (arg1, "ERROR", 5) == 0)
		return arg1;
	  else {
		char *res = new char [strlen ("struct (one: struct (one: %s.one, two: %s.two.one), two: %s.two.two))") + (3 * strlen (arg1)) + 1];
		sprintf (res, "struct (one: struct (one: %s.one, two: %s.two.one), two: %s.two.two))", arg1, arg1, arg1);
		delete [] arg1;
		return res;
	  }
	}
}
