/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)
// FILE: KOOStr.c
// PROJ: COKO Compiler, KOLA Parcer
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOOStr.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OStr
// Parameters:	(char* v)							  
//
// Constructor
//         								    
//////////////////////////////////////////////////////////////////////////////

OStr::OStr (char *v, TObj * t) : koObject (t, v)
{
  if (v == NULL) val = NULL;
  else {
    val = new char [strlen(v) + 1];
    sprintf (val, "%s", v);
  }
  variableNum = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OStr
// Parameters:	(OStr& other)
//
// Copy Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

OStr::OStr (const OStr& other) : koObject ((koObject &) other)
{
  if (other.Val () == NULL) val = NULL;
  else {
    val = new char [strlen(other.Val()) + 1];
    sprintf (val, "%s", other.Val ());
  }
  variableNum = other.getVariableNum();
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~OStr
// Parameters:	()							  
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

OStr::~OStr ()
{
  if (val) {delete [] val; val = NULL;}
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy
// Parameters:	()							  
//
//  return copy of this as koObject*
//         								    
//////////////////////////////////////////////////////////////////////////////

koObject* OStr::Copy () const
{
  OStr* temp = new OStr (*this);
  temp->setVariableNum(variableNum);
  return temp;
}


//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  deepeq
// Parameters:  ()
//
// Value-based equality.
// this is computed recursively
//
//////////////////////////////////////////////////////////////////////////////
 
int OStr::deepeq (NonType * other) 
{
  NameUnion* name_ = other->Myname();
  if ((name_->tag == ONAME ) && (name_->on == OBJ_STR))
  {
    delete name_;
    if (!strcmp(((OStr*)other)->Val(),val))
      return T;
    else
      return F; 
  }
  else
    {
      delete name_;
      return F;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 4 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////

NameUnion* OStr::Myname()
{
	NameUnion* a = new NameUnion(ONAME, OBJ_STR);
	return a;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	()							  
//
/////////////////////////////////////////////////////////////////////////////

// OResult* OStr::reduce ()
// {
//   return new OResSTR (Val());
// }

// //////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  Return val as a string
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OStr::representation ()
{
  char *res = new char [strlen (Val()) + 1];
  sprintf (res, "%s", Val());
  return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* OStr::transform(CState* s)
{
   
  // Shouldn't this be OStr (val, Type ())? 
  OStr* temp = new OStr(val,Type());
  temp->setVariableNum(variableNum);
  return temp;
    
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for rule matching only.
// Rule matching is different from statement matching.
// Duplicated variable entry is allowed here. 
//
//////////////////////////////////////////////////////////////////////////////

int OStr::compares(NonType* nt, CState* a)
{
  NameUnion* tmp_name = nt->Myname();
 
  if ((tmp_name->tag == ONAME) && (tmp_name->on == OBJ_STR) && (!strcmp(((OStr*)nt)->Val(), val)))
	{
	  delete tmp_name;
	  return T;
	}
  else 
	{
	  delete tmp_name;
	  return F;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void OStr::replaceChild(NonType* child, int which_child)
{
}
  
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* OStr::prolog ()
{
  char *res = new char [strlen (Val()) + strlen("ostr()") + 1];
  sprintf (res, "ostr(%s)", Val());
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref OStr::sp_prolog ()
{
  char* text = new char [strlen (Val()) + 1];
  sprintf (text, "%s", Val());
  
  SP_term_ref nam = SP_new_term_ref();
  SP_term_ref res = SP_new_term_ref();
  SP_put_string(nam,text);
  
  SP_cons_functor(res, SP_atom_from_string("ostr"),1,nam);
  
  if (text) { delete text; text = NULL; }
  
  return res;
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	()
//
//  Return the SQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OStr::sql ()
{
  char *res = new char [strlen (val) + 1];
  sprintf (res, val);
  return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	()
//
//  Return the OQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OStr::oql ()
{
  char *res = new char [strlen (val) + 1];
  sprintf (res, val);
  return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Val
// Parameters:	()
//
// Selector - return a copy of val
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OStr::Val ()
{
  char *res = new char [strlen (val) + 1];
  sprintf (res, "%s", val);
  return res;
}
