/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)
// FILE: KOObject.C
// PROJ: COKO Compiler, KOLA Parcer
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOObject.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  koObject
// Parameters:	(TObj *t, char * n)
//
// Constructor.
//
//////////////////////////////////////////////////////////////////////////////

koObject::koObject (TObj *t, char * n)
{
  type = t;

  if (n == NULL) namestring = NULL;
  else {
     namestring = new char [strlen (n) + 1];
     sprintf (namestring, "%s", n);
  }
  variableNum = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  koObject
// Parameters:	(TObj *t, int n)
//
// Constructor.
//
//////////////////////////////////////////////////////////////////////////////

koObject::koObject (int n, TObj *t)
{
  type = t;
  
  namestring = new char [MAXINTLENGTH+1];  
  // 32(MAXINTLENGTH) bytes should be enough!
  sprintf (namestring, "%d", n); 
  variableNum = 0;
 
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  koObject
// Parameters:	(koObject& other)							  
//
// Copy Constructor.
//
//////////////////////////////////////////////////////////////////////////////

koObject::koObject (const koObject& other)
{
  if (other.Type() == NULL) type = NULL;
  else type = other.Type()->Copy();

  namestring = new char [strlen (other.Name()) + 1];
  sprintf (namestring, "%s", other.Name());
  variableNum = other.getVariableNum();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~koObject
// Parameters:	()							  
//
// Destructor.
//
//////////////////////////////////////////////////////////////////////////////

koObject::~koObject()
{
  if (type) { delete type; type = NULL; }
  if (namestring) { delete [] namestring; namestring = NULL; }
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	(koObject *o)
//
//  Returns SQL representation of result of invoking this on o
//         								    
//////////////////////////////////////////////////////////////////////////////

char* koObject::sql () 
{
  char *res = new char [strlen ("ERROR: sql () called on ") + strlen (namestring) + 1];
  sprintf (res, "ERROR: sql () called on %s", namestring);
  return res;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	()
//
//  Return the OQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* koObject::oql ()
{
  char *res = new char [strlen ("ERROR: oql () called on ") + strlen (namestring) + 1];
  sprintf (res, "ERROR: oql () called on %s", namestring);
  return res;
}



//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  obj
// Parameters:	()
//
// Symbolic simplification of this.  Eventually,
// this method will replace "object" presently
// only defined with _OSTORE_ flag set
//  
//////////////////////////////////////////////////////////////////////////////

koObject* koObject::obj ()
{
  return Copy ();
}

