/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)
// FILE: KOPGreaterEqual.C
// PROJ: COKO Compiler, KOLA Parser
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOPGreaterEqual.H"
//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  PGreaterEqual
// Parameters:	()							  
//
//  Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

PGreaterEqual::PGreaterEqual (TPred * t) : Predicate (t, PRED_GREATEREQ,">=")
{    
    variableNum = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  PGreaterEqual
// Parameters:	(PGreaterEqual& other)							  
//
//  Copy Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

PGreaterEqual::PGreaterEqual (const PGreaterEqual& other)
                                  : Predicate ((Predicate &) other)
{
    variableNum = other.getVariableNum();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~PGreaterEqual
// Parameters:	()							  
//
//  Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

PGreaterEqual::~PGreaterEqual ()
{
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy
// Parameters:	()							  
//
//  return copy of this as Predicate*
//         								    
//////////////////////////////////////////////////////////////////////////////

Predicate* PGreaterEqual::Copy () const
{
  PGreaterEqual* temp = new PGreaterEqual (*this);
  temp->setVariableNum(variableNum);
  return temp;
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:deepeq
// Parameters:(Nontype*)
//
// Value-based equality.
// this is computed recursively
//
//////////////////////////////////////////////////////////////////////////////


int PGreaterEqual::deepeq (NonType * other) 
{
  NameUnion* name_ = other->Myname();
  if ((name_->tag == PNAME ) && (name_->pn == PRED_GREATEREQ))
  {
    delete name_;
    return T;
  }
  else
  {
    delete name_;
    return F;
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 4 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////

NameUnion* PGreaterEqual::Myname()
{
	NameUnion* a = new NameUnion(PNAME, PRED_GREATEREQ);
	return a;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							 
//
//  "PPRIM: GREATER"
//         								    
//////////////////////////////////////////////////////////////////////////////

char* PGreaterEqual::representation ()
{
  char *res = new char [strlen (">=") + 1];
  sprintf (res, "%s", ">=");
  return res;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	(OResult* a)							 
//  return new BResINF (a -> fst -> reduce (), ">=", 
//  		    a -> snd -> reduce ())
//
//////////////////////////////////////////////////////////////////////////////

// BResult* PGreaterEqual::reduce (OResult* a)
// {
//   if (a -> isPair ()) {
//     OResPAIR *a2 = ((OResPAIR *) a);
//     BResINF *res = new BResINF
//       (a2 -> One () -> Copy (), ">=", a2 -> Two () -> Copy ());
//     delete a;
//     return res;
//   } else {
//     BResBB *res = new BResBB ("GREATEREQ", a -> Copy ());
//     delete a;
//     return res;
//   }
// }

// //////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* PGreaterEqual::transform(CState* s)
{
   
  PGreaterEqual* temp = new PGreaterEqual(Type());
  temp->setVariableNum(variableNum);
  return temp;
    
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for rule matching only.
// Rule matching is different from statement matching.
// Duplicated variable entry is allowed here. 
//
//////////////////////////////////////////////////////////////////////////////

int PGreaterEqual::compares(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion* tmp_name = nt->Myname();

  if ((tmp_name->tag == PNAME) && (tmp_name->pn == PRED_GREATEREQ))
	{
	  delete tmp_name;
	  return T;
	}
  else 
	{
	  delete tmp_name;
	  return F;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void PGreaterEqual::replaceChild(NonType* child, int which_child)
{
}

#ifdef _OSTORE_
//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: invoke
//  Parameters:(koObject*)
//
//  Implements the greater than or equal predicate primitive.
//
//////////////////////////////////////////////////////////////////////////////

koBool* PGreaterEqual::invoke(koObject* x)
{
  return new BInvoke(this,x);
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: exec
//  Parameters:(OSObject*)
//
//  Implements the greater than predicate primitive.
//
//////////////////////////////////////////////////////////////////////////////

OSBool* PGreaterEqual::exec(OSObject* x)
{
  koObject *i = (((OSPair*)x)->One())->kola();
  koObject *j = (((OSPair*)x)->Two())->kola();
  if (i->greaterequal(j))
    return new OSBPrimitive(1);
  else
    return new OSBPrimitive(0);
}

#endif // _OSTORE_
  
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* PGreaterEqual::prolog ()
{
  char *res = new char [strlen ("pgreaterequal") + 1];
  sprintf (res, "pgreaterequal");
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref PGreaterEqual::sp_prolog ()
{
  SP_term_ref res = SP_new_term_ref();
  SP_put_string(res,"pgreaterequal");
  return res;
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	(koObject *o)
//
// Return the SQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* PGreaterEqual::sql (koObject *o)
{    
    NameUnion *nu;
    nu = o -> Myname ();
    if (nu -> on == OBJ_PAIR) {
	  delete nu;
	  char *arg1 = o -> asPair () -> One () -> sql ();
	  char *arg2 = o -> asPair () -> Two () -> sql ();
	  if (strncmp (arg1, "ERROR", 5) == 0) {
		delete [] arg2;
		return arg1;
	  } else if (strncmp (arg2, "ERROR", 5) == 0) {
		delete [] arg1;
		return arg2;
	  } else {
		char *res = new char [strlen ("( >= )") + strlen (arg1) + strlen (arg2) + 1];
		sprintf (res, "(%s >= %s)", arg1, arg2);
		delete [] arg1;
		delete [] arg2;
		return res;
	  }
	} else {
	  delete nu;
	  cerr << "ERROR: Calling GREATEREQUAL on non-Pair Object" << endl;
	  return NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	(koObject *o)
//
// Return the OQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* PGreaterEqual::oql (koObject *o)
{    
    NameUnion *nu;
    nu = o -> Myname ();
    if (nu -> on == OBJ_PAIR) {
	  delete nu;
	  char *arg1 = o -> asPair () -> One () -> oql ();
	  char *arg2 = o -> asPair () -> Two () -> oql ();
	  if (strncmp (arg1, "ERROR", 5) == 0) {
		delete [] arg2;
		return arg1;
	  } else if (strncmp (arg2, "ERROR", 5) == 0) {
		delete [] arg1;
		return arg2;
	  } else {
		char *res = new char [strlen ("( >= )") + strlen (arg1) + strlen (arg2) + 1];
		sprintf (res, "(%s >= %s)", arg1, arg2);
		delete [] arg1;
		delete [] arg2;
		return res;
	  }
	} else {
	  delete nu;
	  cerr << "ERROR: Calling GREATEREQUAL on non-Pair Object" << endl;
	  return NULL;
	}
}

