/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)
//        Joon S. Lee (jl)					 	    
//  FILE: KOPredicate.H 
//  PROJ: COKO, KOLA
//  DATE: 5/97		                                                    
//
// SYNOPSIS
//  The type of Predicate objects which can be built
//  in complex ways and invoked.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_PREDICATE_HEADER
#define KOLA_PREDICATE_HEADER 

#include "KGlobals.H"
#include "KONameUnion.H"
#include "KONonType.H"
#include "KOObject.H"
#include "TYTPred.H"

#ifdef _OSTORE_
#include "OSObject.H"
#include "OSBool.H"
#endif // _OSTORE_

#include "PCState.H"

class Predicate : public NonType
{
public: 
  Predicate (TPred *t = NULL, PredName pn = PRED_INVALID, char * nm = NULL);
  // Constructor
    
  Predicate (const Predicate& other);
  // Copy Constructor.
    
  virtual ~Predicate ();
  // Destructor
    
  virtual Predicate* Copy() const = 0;
  // Virtual Copy Constructor.

  NonType* Copy_kTree() const { return Copy(); }
    
  PredName Identity () const {return myname;}
  //  What kind of predicate are you?
  //  Return myname.
  
  TPred* Type () const {return type;}
  // Selector.
    
  //  virtual BResult* reduce (OResult* a) =0;
  //  Return string denoting the semantics of the predicate
  //  when applied to expression a.

  virtual char *oql (koObject *o);
  // Return the OQL string equivalent of invoking this on o

  virtual char *sql (koObject *o);
  // Return the SQL string equivalent of invoking this on o

  virtual koBool *inv (koObject *o);
  // Symbolic invocation of this on o.  Eventually,
  // this method will replace "invoke" presently
  // only defined with _OSTORE_ flag set

#ifdef _OSTORE_
  virtual koBool* invoke(koObject* x);
  //  Invokes the symbolic evaluator to transform
  //  to a more efficient expression.

  virtual OSBool* exec(OSObject* x) = 0;
  //  Executes the predicate on x.
#endif // _OSTORE_

protected:
  TPred* type;
  //  The type of a predicate is determined by its 
  //  domain.
  
   PredName myname;
  //  What kind of predicate are you?

  char * namestring;
  // The string naming this predicate, for display.
};


#endif




