/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)
//        Xiaoyu Wang (wangxy)
//  FILE: KOTAlt.H 
//  PROJ: COKO, KOLA
//  DATE: 9/00		                                                    
//
// SYNOPSIS
// TagExp: TagExp \| TagExp
//
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_TALT_HEADER
#define KOLA_TALT_HEADER 

#include "KGlobals.H"
#include "KOTagExp.H"
#include "KONameUnion.H"
#include "KONonType.H"

#include "PCState.H"
#include <string.h>

class  TAlt: public TagExp
{
public:
  TAlt (TagExp* te1, TagExp* te2);
  // Constructor.

  int compares(NonType* nt, CState* a);
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This method is for rule matching only.
  // Rule matching is different from statement matching.
  // Duplicated variable entry is allowed here. 
 
  NonType* transform(CState* s);
  //  Return coresponding Nontype object.
  //  CState is only used in VAR classes.
  //  No other classes need CState.

  void setDEPnInvDEP(char*, CState*) {}
  // set dependency list and inverse dependency list

  void undoSetInvDEP(char*, CState*) {}
  // undo changes made on inverse dependency list.
  // undo for dependency list is not necessary, since 
  // any changes on dependency list is undone by poping up
  // top frame from the environment stack.

  TAlt (const TAlt& other);
  // Copy Constructor.
  
  ~TAlt ();
  // Destructor.
  
  TagExp* Copy () const;
  // Virtual copy constructor.
    
  int deepeq(NonType * other);
  // Value-based equality

  NameUnion* Myname();
  // return myname wrapped in NameUnion object
  
  int numChildren() const { return 2; }
  // the number of children

  TagExp* One() const { return one; }
  TagExp* Two() const { return two; }  

  void setOne(NonType* te) { delete one; one = (TagExp*)te; }  
  void setTwo(NonType* te) { delete two; two = (TagExp*)te; } 
  
#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
  // convert kola tree into prolog term and return it.
#endif // _PROLOG_ 

  char* prolog ();
  // return string representation in prolog term
  
  char* representation ();
  //  Return TagExp1 as a string + " ALT " + TagExp2 as a string

  void initCurrChild() { currChild = 0;}
  //  Set current child to 0, call it before 1st time call nextChild 

  NonType* nextChild();
  // Get the next child, of current child is the last one, return NULL; 
  
  const char * Name() const { return namestring; }
  // Returns the text representation of the tagexp.
  
  NonType* Transform(CState *s);
  
  int compareSTM(NonType* other, CState* s);

  void replaceChild(NonType *child, int which_child);
  // replace the specified child with the parameter child

#ifdef _OSTORE_
  TagExp* tagexp() { return NULL; }
  // Returns the kola equivalent of an object.
  
  OSObject *result() { return NULL; }
  // Returns the OSObject equivalent of itself.

#endif // _OSTORE_

protected:
  TagExp* one;
  // what is the 1st child? 
  
  TagExp* two;
  // what is the 2nd child?
  
  int currChild;
  // which is the current child
};


#endif
