
/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)
//        Xiaoyu Wang (wangxy)
//  FILE: KOTSt.H 
//  PROJ: COKO, KOLA
//  DATE: 9/00		                                                    
//
//  TagExp: TagExp st Predicate
//
//
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_TST_HEADER
#define KOLA_TST_HEADER 

#include "KGlobals.H"
#include "KOTagExp.H"
#include "KOPredicate.H"
#include "KONameUnion.H"
#include "KONonType.H"

#include "PCState.H"
#include <string.h>

class  TSt: public TagExp
{
public:
  TSt (TagExp* te, Predicate* p);
  // Constructor.

  int compares(NonType* nt, CState* a);
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This method is for rule matching only.
  // Rule matching is different from statement matching.
  // Duplicated variable entry is allowed here. 
 
  NonType* transform(CState* s);
  //  Return coresponding Nontype object.
  //  CState is only used in VAR classes.
  //  No other classes need CState.

  void setDEPnInvDEP(char*, CState*) {}
  // set dependency list and inverse dependency list

  void undoSetInvDEP(char*, CState*) {}
  // undo changes made on inverse dependency list.
  // undo for dependency list is not necessary, since 
  // any changes on dependency list is undone by poping up
  // top frame from the environment stack.

  TSt (const TSt& other);
  // Copy Constructor.
  
  ~TSt ();
  // Destructor.

  TagExp* Copy () const;
  // Virtual copy constructor.
    
  int deepeq(NonType * other);
  // Value-based equality

  NameUnion* Myname();
  // returns myname wrapped inside of a class NameUnion.

  int numChildren() const { return 2; }
  //  Number of arguments.

  TagExp* One() const { return one; }
  Predicate* Two() const { return two; }  

  void setOne(NonType* te) { delete one; one = (TagExp*)te; }  
  void setTwo(NonType* p) { delete two; two = (Predicate*)p; } 
  
  int compareSTM(NonType* other, CState* s);
  //  OResult* reduce ();
  //  Return as_string (val)	
  
#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
    // convert kola tree into prolog term and return it.
#endif // _PROLOG_ 

  char* prolog ();
  // return string representation in prolog term

  char* representation ();

  void replaceChild(NonType* child, int which_child);

  void initCurrChild() { currChild = 0;}
  //  Set current child to 0, call it before 1st time call nextChild 

  NonType* nextChild();
  // Get the next child, of current child is the last one, return NULL; 
 
  const char * Name() const { return namestring; }
  // Returns the text representation of the tagexp.

#ifdef _OSTORE_
  TagExp* tagexp() { return NULL; }
  // Returns the kola equivalent of an object.

  OSObject *result() { return NULL; }
  // Returns the OSObject equivalent of itself.

#endif // _OSTORE_

protected:
  TagExp* one;
  Predicate* two;
  int currChild;
};


#endif
