/*
 * Copyright 1995, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *	Name:
 *	Organization:
 *	Address (postal and/or electronic):
 *
 * To:
 *	Software Librarian
 *	Computer Science Department, Box 1910
 *	Brown University
 *	Providence, RI 02912
 *
 *		or
 *
 *	brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.

 \'[A-Za-z0-9]\'                                          { return CHARCON; @{ @CHARCON.string@ = lexeme(); @}}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc@cs.brown.edu)
//  FILE: QL.l 
//  PROJ: The "Karbonator"
//  DATE: 7/98		                                                    
//
//  QLite -> KOLA translator.  Lexical scanning file.
//
//////////////////////////////////////////////////////////////////////////////
*/

%{

#include "oxout.tab.h"
#include "KGlobals.H"
#include "KONonType.H"
#include <string.h>
#include <stdio.h>

int lineno3 = 1;
extern "C" {int atoi (const char *s);}

char *lexeme ()
{
  char *res = new char [yyleng + 1];
  sprintf (res, "%s", yytext);
  return res;
}
			       
%}

%%
[ \t]+						                             {}
[\n\f]                                                   { lineno3++; }
[-][-][^\n]+                                             { return COMMENT; }

[+-]?[0-9]+                                              { return INTCON; @{ @INTCON.val@ = atoi (yytext); @}}
\"[^\"]*[\"]                                             { return STRINGCON; @{ @STRINGCON.string@ = lexeme(); @}}
\`[^\']*[\']                                             { return STRINGCON; @{ @STRINGCON.string@ = lexeme(); @}}
[+-]?(([0-9]+)|([0-9]*\.[0-9]+)([eE][-+]?[0-9]+)?)   	 { return FLOATCON; @{ @FLOATCON.string@ = lexeme(); @}}
[Tt][Rr][Uu][Ee]					                     { return TRUEV; }  
[Ff][Aa][Ll][Ss][Ee]					                 { return FALSEV; }  
[Nn][Uu][Ll][Ll]					                     { return NIL; }  
[Nn][Ii][Ll]  					                         { return NIL; }  

"{"                                                      { return ('{'); }  
"}"                                                      { return ('}'); }     
"("                                                      { return ('('); }  
")"                                                      { return (')'); }  
","                                                      { return (','); }  
":"                                                      { return (':'); }  
 
"+"                                                      { return ('+'); }  
"-"                                                      { return ('-'); }  
"*"                                                      { return ('*'); }  
"/"                                                      { return ('/'); }
"."                                                      { return (DOT); }  
".."                                                     { return (DOT); }  
"->"                                                     { return (DOT); }  
"="                                                      { return ('='); }  
"<"                                                      { return ('<'); }  
">"                                                      { return ('>'); }  
"<="                                                     { return LE; } 
">="                                                     { return GE; } 
"!="                                                     { return NE; }

[Nn][Oo][Tt]  	      	   	     	       	    	     { return (NOT); }  
[Aa][Nn][Dd]  	   	      	   	     	       	         { return (AND); }  
[Oo][Rr]  	       	   	   	     	       	             { return (OR); }  
[Ll][Ii][Kk][Ee]   	      	   	     	       	         { return (LIKE); }  

[Ee][Ll][Ee][Mm][Ee][Nn][Tt]				             { return (ELT); }  
[Ff][Ll][Aa][Tt][Tt][Ee][Nn]				             { return (FLATTEN); }  
[Uu][Nn][Ii][Oo][Nn]                                     { return (UNION); }  
[Ii][Nn][Tt][Ee][Rr][Ss][Ee][Cc][Tt]                     { return (INTERSECT); }  
[Ee][Xx][Cc][Ee][Pp][Tt]                                 { return (EXCEPT); }  
[Ss][Ee][Tt]                                             { return (SET); }  
[Ee][Xx][Ii][Ss][Tt][Ss]  	       	       	     	     { return (EXISTS); }  
[Ff][Oo][Rr][ \t]+[Aa][Ll][Ll]      	      	   	     { return (FORALL); }  
[Ii][Nn]                                                 { return (INP); }  
[Ss][Ee][Ll][Ee][Cc][Tt]                                 { return (SELECT); }  
[Dd][Ii][Ss][Tt][Ii][Nn][Cc][Tt]                         { return (DISTINCT); }  
[Ff][Rr][Oo][Mm]                                         { return (FROM); }  
[Ww][Hh][Ee][Rr][Ee]                                     { return (WHERE); }
[Gg][Rr][Oo][Uu][Pp]                                     { return (GROUP); }  
[Oo][Rr][Dd][Ee][Rr]                                     { return (ORDER); }
[Ss][Tt][Rr][Uu][Cc][Tt]  	       	       	     	     { return (STRUCT); }  
[Aa][Ss][Cc]                                             { return (ASC); }
[Dd][Ee][Ss][Cc]                                         { return (DESC); }

[Mm][Oo][Dd]					                         { return (MOD); }  
[Aa][Bb][Ss]					                         { return (ABS); }
[Cc][Oo][Uu][Nn][Tt]                                     { return (COUNT); }  
[Ss][Uu][Mm]                                             { return (SUM); }  
[Mm][Ii][Nn]                                             { return (MIN); }  
[Mm][Aa][Xx]                                             { return (MAX); }  
[Aa][Vv][Gg]                                             { return (AVG); }

[Ii][Ss]                                                 { return (IS); }
[Bb][Yy]                                                 { return (BY); }
[:]?[a-zA-Z][a-z_\'A-Z0-9]*               		         { return (IDENT); @{ @IDENT.string@ = lexeme(); @}}
.                    					                 { return (ERROR); }
%%



int yywrap ()  {return 1;}


